@@ -22,7 +22,7 @@ buildLib()
     cd ${LIBNAME}
     ./configure --prefix=${AVG_PATH} ${CONFIG_ARGS}
     make clean
-    make -j3
+    make -j5
     make install
     cd ..
 }
@@ -34,7 +34,7 @@ buildlibjpeg()
     cp ${AVG_PATH}/share/libtool/config/config.guess .
     ./configure --prefix=${AVG_PATH}
     make clean
-    make -j3
+    make -j5
     make install-lib
     make install-headers
     ranlib ../../lib/libjpeg.a
@@ -46,17 +46,17 @@ buildlibpng()
     cd libpng-1.2.41
     ./configure --prefix=${AVG_PATH} --disable-shared
     make clean
-    make -j3
+    make -j5
     make install
     cd ..
 }
 
 buildglib()
 {
-    cd glib-2.21.3
+    cd glib-2.29.2
     LDFLAGS="-framework ApplicationServices $LDFLAGS -lresolv" ./configure  --prefix=${AVG_PATH} --disable-shared --enable-static 
     make clean
-    make -j3
+    make -j5
     make install
     cd ..
 }
@@ -67,25 +67,36 @@ buildfontconfig()
     automake
     LDFLAGS="-framework ApplicationServices ${LDFLAGS}" ./configure --prefix=${AVG_PATH} --disable-shared --with-add-fonts=/Library/Fonts,/System/Library/Fonts,~/Library/Fonts --with-confdir=/etc/fonts --with-cache-dir=~/.fontconfig --with-cache-dir=~/.fontconfig
     make clean
-    make -j3
+    make -j5
     sudo make install
     sudo chown -R `whoami` ~/.fontconfig
     cd ..    
 }
 
+buildgdkpixbuf()
+{
+    cd gdk-pixbuf-2.23.3
+    LDFLAGS="-framework ApplicationServices $LDFLAGS -lresolv" ./configure --prefix=${AVG_PATH} --disable-shared --with-included-loaders
+    make clean
+    make -j5
+    make install
+    cd ..
+}
+
 buildboost()
 {
     cd boost_1_41_0
-    # Filesystem and system libraries needed only for ColladaDOM.
-    ./bootstrap.sh --prefix=${AVG_PATH} --with-libraries=python,thread,filesystem,system 
+    ./bootstrap.sh --prefix=${AVG_PATH} --with-libraries=python,thread,date_time
     ./bjam clean
     ./bjam install
     cd ..
     rm -f ../lib/libboost_thread.dylib
     rm -f ../lib/libboost_python.dylib
+    rm -f ../lib/libboost_date_time.dylib
 }
 if [[ x"${AVG_PATH}" == "x" ]]
 then
+    echo ${AVG_PATH}
     echo Please set AVG_PATH and call 'source mac/avg_env.sh' before calling this script.
     exit -1 
 fi
@@ -108,10 +119,8 @@ buildLib autoconf-2.63
 buildLib automake-1.11
 buildlibjpeg
 buildLib tiff-3.8.2 --disable-shared 
-buildLib zlib-1.2.3
 buildlibpng
 buildLib libxml2-2.6.32 --disable-shared
-buildLib GraphicsMagick-1.1.10 "--without-x --without-perl --disable-shared --disable-delegate-build --without-modules --without-bzlib --without-dps --without-gslib --without-wmf --without-xml --without-ttf --with-quantum-depth=8"
 buildLib pkg-config-0.20
 if [[ "${DARWINMAJORVER}" == "10" ]]
 then
@@ -120,16 +129,22 @@ else
     buildLib ffmpeg "--disable-debug --enable-pthreads --disable-ffserver --disable-muxer=matroska --disable-demuxer=matroska --disable-muxer=matroska_audio"
 fi
 
-buildLib SDL-1.2.14 "--disable-shared --disable-cdrom --disable-threads --disable-file --disable-video-x11 --without-x"
-buildLib gettext-0.14.6 "--disable-shared --with-included-gettext --disable-csharp  --disable-libasprintf"
+buildLib SDL-1.2.15 "--disable-shared --disable-cdrom --disable-threads --disable-file --disable-video-x11 --without-x"
+buildLib gettext-0.18.1.1 "--disable-shared --with-included-gettext --disable-csharp  --disable-libasprintf"
 buildglib
 
-buildLib freetype-2.3.9 "--disable-shared --with-old-mac-fonts"
+buildLib freetype-2.4.4 "--disable-shared --with-old-mac-fonts"
 buildLib expat-2.0.0 --disable-shared
 
 buildfontconfig
 
+buildLib pixman-0.22.0 --disable-shared
+buildLib cairo-1.10.2 "--disable-shared --enable-xlib=no --enable-xlib-xrender=no --enable-quartz=no --enable-quartz-font=no --enable-quartz-image=no --enable-ps=no --enable-pdf=no --enable-svg=no"
 buildLib pango-1.24.4 "--disable-shared --without-x --with-included-modules=yes"
+
+buildgdkpixbuf
+buildLib librsvg-2.34.0 --disable-shared
+
 buildboost
 
 buildLib libdc1394-2.0.2 "--disable-shared --disable-doxygen-doc --without-x"
