@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -48,13 +48,6 @@ ParallelAnim::~ParallelAnim()
     }
 }
 
-AnimPtr ParallelAnim::create(const vector<AnimPtr>& anims,
-            const object& startCallback, const object& stopCallback,
-            long long maxAge)
-{
-    return AnimPtr(new ParallelAnim(anims, startCallback, stopCallback, maxAge));
-}
-
 void ParallelAnim::start(bool bKeepAttr)
 {
     Anim::start();
