@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,20 +30,17 @@ namespace avg {
 
 class AVG_API WaitAnim: public Anim {
 public:
-    virtual ~WaitAnim();
-   
-    static AnimPtr create(long long duration = -1,
+    WaitAnim(long long duration = -1,
             const boost::python::object& startCallback=boost::python::object(), 
             const boost::python::object& stopCallback=boost::python::object());
-
+    virtual ~WaitAnim();
+   
     virtual void start(bool bKeepAttr=false);
     virtual void abort();
     
     virtual bool step();
 
 private:
-    WaitAnim(long long duration, const boost::python::object& startCallback, 
-            const boost::python::object& stopCallback);
     long long m_Duration;
     long long m_StartTime;
 };
