@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,13 +24,16 @@
 
 #include "../api.h"
 #include <vector>
-#include <math.h>
 
-#define PI 3.14159265358979323846
-#define EPSILON 0.00001
+#include <math.h>
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
 
 namespace avg {
 
+static const double EPSILON = 0.00001;
+
 bool ispow2(int n);
 
 int nextpow2(int n);
