@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 #define _Point_H_
 
 #include "../api.h"
+#include "MathHelper.h"
 
 #include <ostream>
 #include <vector>
@@ -98,7 +99,10 @@ template<class NUM>
 double calcDist(const Point<NUM>& pt1, const Point<NUM>& pt2);
 
 template<class NUM>
-double calcDistSquared(const Point<NUM>& pt1, const Point<NUM>& pt2);
+double calcDistSquared(const Point<NUM>& pt1, const Point<NUM>& pt2)
+{
+    return sqr(pt1.x-pt2.x)+sqr(pt1.y-pt2.y);
+}
 
 typedef Point<double> DPoint;
 typedef Point<int> IntPoint;
@@ -111,6 +115,8 @@ double dotProduct(const DPoint& pt1, const DPoint pt2)
     return pt1.x*pt2.x+pt1.y*pt2.y;
 }
 
+double vecAngle(const DPoint& pt1, const DPoint pt2);
+
 typedef std::vector<DPoint> DPointVector;
 
 }
