@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,7 @@
 
 namespace avg {
 
-class AVG_API UTF8String: public std::string
+class AVG_TEMPLATE_API UTF8String: public std::string
 {
 public:
     UTF8String();
