@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine.
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,7 +32,6 @@
 #include "StringHelper.h"
 #include "MathHelper.h"
 #include "CubicSpline.h"
-#include "BicubicSpline.h"
 #include "BezierCurve.h"
 #include "Signal.h"
 #include "Backtrace.h"
@@ -331,18 +330,21 @@ public:
             DPoint v2(DPoint(1,0));
             TEST(getLineLineIntersection(p1, v1, p2, v2) == DPoint(1,1));
         }
-        TEST(almostEqual(DPoint(10,0).getRotatedPivot(PI, DPoint(15,5)), DPoint(20,10)));
-        TEST(almostEqual(DPoint(10,0).getRotatedPivot(PI*0.5, DPoint(15,5)),
+        TEST(almostEqual(DPoint(10,0).getRotatedPivot(M_PI, DPoint(15,5)), DPoint(20,10)));
+        TEST(almostEqual(DPoint(10,0).getRotatedPivot(M_PI*0.5, DPoint(15,5)),
                 DPoint(20,0)));
-        TEST(almostEqual(DPoint(10,0).getRotatedPivot(PI*1.5, DPoint(15,5)), 
+        TEST(almostEqual(DPoint(10,0).getRotatedPivot(M_PI*1.5, DPoint(15,5)), 
                 DPoint(10,10)));
-        TEST(almostEqual(DPoint(10,0).getRotatedPivot(PI*2, DPoint(15,5)), 
+        TEST(almostEqual(DPoint(10,0).getRotatedPivot(M_PI*2, DPoint(15,5)), 
                 DPoint(10,0)));
-        TEST(almostEqual(DPoint(23,0).getRotatedPivot(PI*0.5), DPoint(0,23)));
+        TEST(almostEqual(DPoint(23,0).getRotatedPivot(M_PI*0.5), DPoint(0,23)));
 
         TEST(almostEqual(DPoint(10,0), DPoint::fromPolar(0, 10)));
-        TEST(almostEqual(DPoint(0,10), DPoint::fromPolar(PI*0.5, 10)));
-        TEST(almostEqual(DPoint(0,-1), DPoint::fromPolar(PI*1.5, 1)));
+        TEST(almostEqual(DPoint(0,10), DPoint::fromPolar(M_PI*0.5, 10)));
+        TEST(almostEqual(DPoint(0,-1), DPoint::fromPolar(M_PI*1.5, 1)));
+        TEST(almostEqual(vecAngle(DPoint(0,1),DPoint(1,0)), M_PI*0.5));
+        TEST(almostEqual(vecAngle(DPoint(0,-1),DPoint(1,0)), M_PI*1.5));
+        TEST(almostEqual(vecAngle(DPoint(0,2),DPoint(1,0)), M_PI*0.5));
     }
 };
 #ifdef _MSC_VER
@@ -393,6 +395,8 @@ public:
         TEST(!tri.isInside(DPoint(2,1)));
         TEST(!tri.isInside(DPoint(-2,5)));
         TEST(!tri.isInside(DPoint(5,5)));
+        tri = Triangle(DPoint(0,0), DPoint(4,8), DPoint(4,4));
+        TEST(tri.isInside(DPoint(3,4)));
 
         DPoint polyArray[] = {DPoint(0,0), DPoint(8,2), DPoint(9,0), DPoint(9,3), 
                 DPoint(1,1), DPoint(0,3)}; 
@@ -468,7 +472,7 @@ public:
         try {
             stringToInt("5a");
         } catch (const Exception& e) {
-            if (e.GetCode() == AVG_ERR_TYPE ) {
+            if (e.getCode() == AVG_ERR_TYPE ) {
                 bExceptionThrown = true;
             }
         }
@@ -520,59 +524,6 @@ public:
             TEST(almostEqual(spline.interpolate(9), 7.5));
             TEST(almostEqual(spline.interpolate(10), 9));
         }
-        {
-            double xd[] = {0,1,2,3};
-            vector<double> x = vectorFromCArray(4, xd);
-            double yd[] = {0,1,2,3}; 
-            vector<double> y = vectorFromCArray(4, yd);
-            double fd[] = {0,0,0,0,
-                           0,0,1,0,
-                           0,0,0,0,
-                           0,0,0,0
-                          };
-            vector<vector<double> > f = vector2DFromCArray(4, 4, fd);
-            BicubicSpline spline(x, y, f);
-/*
-            cerr << spline.interpolate(DPoint(1,1)) << endl;
-            cerr << spline.interpolate(DPoint(2,1)) << endl;
-            cerr << spline.interpolate(DPoint(1.5,1)) << endl;
-            cerr << spline.interpolate(DPoint(0.5,1)) << endl;
-            cerr << spline.interpolate(DPoint(1,2)) << endl;
-            cerr << spline.interpolate(DPoint(10,0)) << endl;
-*/
-            TEST(almostEqual(spline.interpolate(DPoint(0,0)), 0));
-            TEST(almostEqual(spline.interpolate(DPoint(1,1)), 0));
-            TEST(almostEqual(spline.interpolate(DPoint(2,1)), 1));
-            TEST(almostEqual(spline.interpolate(DPoint(1,2)), 0));
-            TEST(spline.interpolate(DPoint(1.5,1)) > 0.5);
-            TEST(spline.interpolate(DPoint(1.5,1)) < 1);
-            TEST(spline.interpolate(DPoint(0.5,1)) < 0);
-            TEST(almostEqual(spline.interpolate(DPoint(-1,0)), 0));
-            TEST(almostEqual(spline.interpolate(DPoint(-10,0)), 0));
-            TEST(almostEqual(spline.interpolate(DPoint(0,-1)), 0));
-            TEST(almostEqual(spline.interpolate(DPoint(0,-10)), 0));
-            TEST(almostEqual(spline.interpolate(DPoint(0,10)), 0));
-            TEST(almostEqual(spline.interpolate(DPoint(10,0)), 0));
-        }
-        {
-            double xd[] = {0,2,4,6};
-            vector<double> x = vectorFromCArray(4, xd);
-            double yd[] = {0,3,6,9}; 
-            vector<double> y = vectorFromCArray(4, yd);
-            double fd[] = {0,0,0,0,
-                           0,0,1,0,
-                           0,0,0,0,
-                           0,0,0,0
-                          };
-            vector<vector<double> > f = vector2DFromCArray(4, 4, fd);
-            BicubicSpline spline(x, y, f);
-            TEST(almostEqual(spline.interpolate(DPoint(2,3)), 0));
-            TEST(almostEqual(spline.interpolate(DPoint(4,3)), 1));
-            TEST(almostEqual(spline.interpolate(DPoint(2,6)), 0));
-            TEST(spline.interpolate(DPoint(3,3)) > 0.5);
-            TEST(spline.interpolate(DPoint(3,3)) < 1);
-            TEST(spline.interpolate(DPoint(1,3)) < 0);
-        }
     }
 };
 
