@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,11 @@
 #include "../base/UTF8String.h"
 
 #include <boost/shared_ptr.hpp>
+
+#if defined(__SSE__) || defined(_WIN32)
+#include <xmmintrin.h>
+#endif
+
 #include <stdlib.h>
 #include <string>
 #include <vector>
@@ -42,11 +47,14 @@ namespace avg {
 typedef std::vector<int> Histogram;
 typedef boost::shared_ptr<Histogram> HistogramPtr;
 
+class Bitmap;
+typedef boost::shared_ptr<Bitmap> BitmapPtr;
+
 class AVG_API Bitmap
 {
 public:
-    Bitmap(DPoint size, PixelFormat pf, const UTF8String& sName="");
-    Bitmap(IntPoint size, PixelFormat pf, const UTF8String& sName="");
+    Bitmap(DPoint size, PixelFormat pf, const UTF8String& sName="", int stride=0);
+    Bitmap(IntPoint size, PixelFormat pf, const UTF8String& sName="", int stride=0);
     Bitmap(IntPoint size, PixelFormat pf, unsigned char * pBits, 
             int stride, bool bCopyBits, const UTF8String& sName="");
     Bitmap(const Bitmap& origBmp);
@@ -58,8 +66,8 @@ public:
     Bitmap &operator =(const Bitmap & origBmp);
     
     // Does pixel format conversion if nessesary.
-    void copyPixels(const Bitmap & origBmp);
-    void copyYUVPixels(const Bitmap & yBmp, const Bitmap& uBmp, const Bitmap& vBmp,
+    void copyPixels(const Bitmap& origBmp);
+    void copyYUVPixels(const Bitmap& yBmp, const Bitmap& uBmp, const Bitmap& vBmp,
             bool bJPEG);
     void save(const UTF8String& sName);
     
@@ -67,15 +75,14 @@ public:
     int getStride() const;
     PixelFormat getPixelFormat() const;
     void setPixelFormat(PixelFormat pf);
-    unsigned char * getPixels();
-    const unsigned char * getPixels() const;
+    unsigned char* getPixels();
+    const unsigned char* getPixels() const;
     std::string getPixelsAsString() const;
-    void setPixels(const unsigned char * pPixels);
+    void setPixels(const unsigned char* pPixels);
     void setPixelsFromString(const std::string& sPixels);
     bool ownsBits() const;
     const std::string& getName() const;
     int getBytesPerPixel() const;
-    static int getBytesPerPixel(PixelFormat pf);
     int getLineLen() const;
     int getMemNeeded() const;
     bool hasAlpha() const;
@@ -89,17 +96,18 @@ public:
     template<class PIXEL>
     void drawLine(IntPoint p0, IntPoint p1, PIXEL color);
 
-    Bitmap * subtract(const Bitmap* pOtherBmp);
-    void blt(const Bitmap* pOtherBmp, const IntPoint& pos);
+    BitmapPtr subtract(const Bitmap& pOtherBmp);
+    void blt(const Bitmap& otherBmp, const IntPoint& pos);
     double getAvg() const;
+    double getChannelAvg(int channel) const;
     double getStdDev() const;
 
-    bool operator ==(const Bitmap & otherBmp);
+    bool operator ==(const Bitmap& otherBmp);
     void dump(bool bDumpPixels=false) const;
 
 private:
-    void initWithData(unsigned char * pBits, int stride, bool bCopyBits);
-    void allocBits();
+    void initWithData(unsigned char* pBits, int stride, bool bCopyBits);
+    void allocBits(int stride=0);
     void YCbCrtoBGR(const Bitmap& origBmp);
     void YCbCrtoI8(const Bitmap& origBmp);
     void I8toI16(const Bitmap& origBmp);
@@ -114,15 +122,14 @@ private:
     IntPoint m_Size;
     int m_Stride;
     PixelFormat m_PF;
-    unsigned char * m_pBits;
+    unsigned char* m_pBits;
     bool m_bOwnsBits;
     UTF8String m_sName;
 
     static bool s_bMagickInitialized;
+    static bool s_bGTKInitialized;
 };
 
-typedef boost::shared_ptr<Bitmap> BitmapPtr;
-
 BitmapPtr YCbCr2RGBBitmap(BitmapPtr pYBmp, BitmapPtr pUBmp, BitmapPtr pVBmp);
 
 template<class PIXEL>
@@ -170,6 +177,9 @@ void Bitmap::drawLine(IntPoint p0, IntPoint p1, PIXEL color)
         }
     }
 }
+#if defined(__SSE__) || defined(_WIN32)
+std::ostream& operator<<(std::ostream& os, const __m64 &val);
+#endif
 
 }
 #endif
