@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,8 @@
 #include "FBO.h"
 
 #include "OGLHelper.h"
+#include "GLContext.h"
+
 #include "../base/Exception.h"
 #include "../base/StringHelper.h"
 #include "../base/ObjectCounter.h"
@@ -29,6 +31,7 @@
 #include <stdio.h>
 
 using namespace std;
+using namespace boost;
 
 namespace avg {
 
@@ -49,11 +52,9 @@ FBO::FBO(const IntPoint& size, PixelFormat pf, unsigned numTextures,
 
     for (unsigned i=0; i<numTextures; ++i) {
         GLTexturePtr pTex = GLTexturePtr(new GLTexture(size, pf, bMipmap));
-        if (bMipmap) {
-            // Workaround for NVidia driver bug - GL_TEX_MIN_FILTER without
-            // glGenerateMipmaps causes FBO creation to fail(?!).
-            pTex->generateMipmaps();
-        }
+        // Workaround for NVidia driver bug - GL_TEX_MIN_FILTER without
+        // glGenerateMipmaps causes FBO creation to fail(?!).
+        pTex->generateMipmaps();
         m_pTextures.push_back(pTex);
     }
     init();
@@ -62,13 +63,37 @@ FBO::FBO(const IntPoint& size, PixelFormat pf, unsigned numTextures,
 FBO::~FBO()
 {
     ObjectCounter::get()->decRef(&typeid(*this));
-    glproc::DeleteFramebuffers(1, &m_FBO);
-    if (m_MultisampleSamples > 1) {
-        glproc::DeleteRenderbuffers(1, &m_ColorBuffer);
-        glproc::DeleteFramebuffers(1, &m_OutputFBO);
+    
+    unsigned oldFBOID;
+    glGetIntegerv(GL_DRAW_FRAMEBUFFER_BINDING, (GLint*)&oldFBOID);
+    glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, m_FBO);
+    
+    for (unsigned i=0; i<m_pTextures.size(); ++i) {
+        glproc::FramebufferTexture2D(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT+i, 
+                GL_TEXTURE_2D, 0, 0);
     }
-    if (m_bUsePackedDepthStencil && isPackedDepthStencilSupported()) {
-        glproc::DeleteRenderbuffers(1, &m_StencilBuffer);
+   
+    GLContext* pContext = GLContext::getCurrent();
+    if (pContext) {
+        pContext->returnFBOToCache(m_FBO);
+        if (m_MultisampleSamples > 1) {
+            glproc::DeleteRenderbuffers(1, &m_ColorBuffer);
+            pContext->returnFBOToCache(m_OutputFBO);
+        }
+        if (m_bUsePackedDepthStencil && isPackedDepthStencilSupported()) {
+            glproc::DeleteRenderbuffers(1, &m_StencilBuffer);
+            glproc::FramebufferRenderbuffer(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, 
+                    GL_RENDERBUFFER_EXT, 0);
+            glproc::FramebufferRenderbuffer(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT,
+                    GL_RENDERBUFFER_EXT, 0);
+            if (m_MultisampleSamples > 1) {
+                glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, m_OutputFBO);
+                glproc::FramebufferTexture2D(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, 
+                        GL_TEXTURE_2D, 0, 0);
+            }
+        }
+        glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, oldFBOID);
+        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "~FBO");
     }
 }
 
@@ -79,40 +104,6 @@ void FBO::activate() const
     checkError("activate");
 }
 
-void FBO::deactivate() const
-{
-    glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, 0);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::deactivate: BindFramebuffer()");
-}
-
-void FBO::setupImagingProjection() const
-{
-    glViewport(0, 0, m_Size.x, m_Size.y);
-    
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    gluOrtho2D(0, m_Size.x, 0, m_Size.y);
-    
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
-    glScaled(m_Size.x, m_Size.y, 1);
-    
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::setupImagingProjection()");
-}
-
-void FBO::drawImagingVertexes()
-{
-    if (!m_pImagingVA) {
-        m_pImagingVA = VertexArrayPtr(new VertexArray);
-        m_pImagingVA->appendPos(DPoint(0,0), DPoint(0,0));
-        m_pImagingVA->appendPos(DPoint(0,1), DPoint(0,1));
-        m_pImagingVA->appendPos(DPoint(1,1), DPoint(1,1));
-        m_pImagingVA->appendPos(DPoint(1,0), DPoint(1,0));
-        m_pImagingVA->appendQuadIndexes(1,0,2,3);
-    }
-    m_pImagingVA->draw();
-}
-
 PixelFormat FBO::getPF() const
 {
     return m_PF;
@@ -142,34 +133,48 @@ void FBO::copyToDestTexture() const
 
 BitmapPtr FBO::getImage(int i) const
 {
+    moveToPBO(i);
+    return getImageFromPBO();
+}
+
+void FBO::moveToPBO(int i) const
+{
+    // Get data directly from the FBO using glReadBuffer. At least on NVidia/Linux, this 
+    // is faster than reading stuff from the texture.
     copyToDestTexture();
-    if (m_MultisampleSamples != 1) {
-        glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, m_OutputFBO);
-    } else {
-        glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, m_FBO);
-    }
-    PixelFormat pf = m_pOutputPBO->getPF();
-    IntPoint size = m_pOutputPBO->getSize();
-    BitmapPtr pBmp(new Bitmap(size, pf));
-
-    m_pOutputPBO->activate();
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::getImage BindBuffer()");
-    glReadBuffer(GL_COLOR_ATTACHMENT0_EXT+i);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::getImage ReadBuffer()");
-
-    glReadPixels (0, 0, size.x, size.y, GLTexture::getGLFormat(pf), 
-            GLTexture::getGLType(pf), 0);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::getImage ReadPixels()");
-    void * pPBOPixels = glproc::MapBuffer(GL_PIXEL_PACK_BUFFER_EXT, GL_READ_ONLY);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::getImage MapBuffer()");
-    Bitmap PBOBitmap(size, pf, (unsigned char *)pPBOPixels, 
-            size.x*Bitmap::getBytesPerPixel(pf), false);
-    pBmp->copyPixels(PBOBitmap);
-    glproc::UnmapBuffer(GL_PIXEL_PACK_BUFFER_EXT);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::getImage: UnmapBuffer()");
-    m_pOutputPBO->deactivate();
-    glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, 0);
-    return pBmp;
+    if (m_MultisampleSamples != 1) { 
+        glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, m_OutputFBO); 
+    } else { 
+        glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, m_FBO); 
+    } 
+    PixelFormat pf = m_pOutputPBO->getPF(); 
+    IntPoint size = m_pOutputPBO->getSize(); 
+ 
+    m_pOutputPBO->activate(); 
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::moveToPBO BindBuffer()"); 
+    glReadBuffer(GL_COLOR_ATTACHMENT0_EXT+i); 
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::moveToPBO ReadBuffer()"); 
+ 
+    glReadPixels(0, 0, size.x, size.y, GLTexture::getGLFormat(pf),  
+            GLTexture::getGLType(pf), 0); 
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::moveToPBO ReadPixels()");     
+}
+ 
+BitmapPtr FBO::getImageFromPBO() const
+{
+    m_pOutputPBO->activate(); 
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::getImageFromPBO BindBuffer()"); 
+    PixelFormat pf = m_pOutputPBO->getPF(); 
+    IntPoint size = m_pOutputPBO->getSize(); 
+    BitmapPtr pBmp(new Bitmap(size, pf)); 
+    void * pPBOPixels = glproc::MapBuffer(GL_PIXEL_PACK_BUFFER_EXT, GL_READ_ONLY); 
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::getImageFromPBO MapBuffer()"); 
+    Bitmap PBOBitmap(size, pf, (unsigned char *)pPBOPixels,  
+            size.x*getBytesPerPixel(pf), false); 
+    pBmp->copyPixels(PBOBitmap); 
+    glproc::UnmapBuffer(GL_PIXEL_PACK_BUFFER_EXT); 
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::getImageFromPBO UnmapBuffer()"); 
+    return pBmp; 
 }
 
 GLTexturePtr FBO::getTex(int i) const
@@ -184,6 +189,7 @@ const IntPoint& FBO::getSize() const
 
 void FBO::init()
 {
+    GLContext* pContext = GLContext::getCurrent();
     if (m_bUsePackedDepthStencil && !isPackedDepthStencilSupported()) {
         throw Exception(AVG_ERR_UNSUPPORTED, "OpenGL implementation does not support offscreen cropping (GL_EXT_packed_depth_stencil).");
     }
@@ -192,7 +198,7 @@ void FBO::init()
     }
     m_pOutputPBO = PBOPtr(new PBO(m_Size, m_PF, GL_STREAM_READ));
 
-    glproc::GenFramebuffers(1, &m_FBO);
+    m_FBO = pContext->genFBO();
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "FBO::init: GenFramebuffers()");
 
     glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, m_FBO);
@@ -200,7 +206,7 @@ void FBO::init()
 
     if (m_MultisampleSamples == 1) {
         glDisable(GL_MULTISAMPLE);
-        for (unsigned i = 0; i < m_pTextures.size(); ++i) {
+        for (unsigned i=0; i<m_pTextures.size(); ++i) {
             glproc::FramebufferTexture2D(GL_FRAMEBUFFER_EXT,
                     GL_COLOR_ATTACHMENT0_EXT+i, GL_TEXTURE_2D, 
                     m_pTextures[i]->getID(), 0);
@@ -251,7 +257,7 @@ void FBO::init()
                     "FBO::init: FramebufferRenderbuffer(STENCIL)");
         }
         checkError("init multisample");
-        glproc::GenFramebuffers(1, &m_OutputFBO);
+        m_OutputFBO = pContext->genFBO();
         glproc::BindFramebuffer(GL_FRAMEBUFFER_EXT, m_OutputFBO);
         glproc::FramebufferTexture2D(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, 
                 GL_TEXTURE_2D, m_pTextures[0]->getID(), 0);
@@ -333,6 +339,6 @@ void FBO::checkError(const string& sContext) const
     AVG_ASSERT(false);
 }
 
-} // namespace avg
+}
 
 
