@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,9 +27,11 @@
 #include "GLTexture.h"
 #include "PBO.h"
 #include "VertexArray.h"
+
 #include "../base/Point.h"
 
 #include <boost/shared_ptr.hpp>
+
 #include <vector>
 
 namespace avg {
@@ -43,14 +45,13 @@ public:
     virtual ~FBO();
 
     void activate() const;
-    void deactivate() const;
-    void setupImagingProjection() const;
-    void drawImagingVertexes();
     PixelFormat getPF() const;
     unsigned getNumTextures() const;
 
     void copyToDestTexture() const;
     BitmapPtr getImage(int i=0) const;
+    void moveToPBO(int i=0) const;
+    BitmapPtr getImageFromPBO() const;
     GLTexturePtr getTex(int i=0) const;
     const IntPoint& getSize() const;
 
@@ -58,6 +59,9 @@ public:
     static bool isMultisampleFBOSupported();
     static bool isPackedDepthStencilSupported();
 
+    void initCache();
+    static void deleteCache();
+
 private:
     void init();
     void checkError(const std::string& sContext) const;
@@ -77,13 +81,10 @@ private:
     unsigned m_ColorBuffer;
     unsigned m_OutputFBO;
 
-    VertexArrayPtr m_pImagingVA;
 };
 
 typedef boost::shared_ptr<FBO> FBOPtr;
 
 }
 
-
 #endif 
-
