@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -39,8 +39,6 @@
 
 #include "../base/TimeSource.h"
 
-#include <Magick++.h>
-
 #include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,18 +46,16 @@
 using namespace avg;
 using namespace std;
 
-#define NUM_RUNS 50
-
 template<class TEST>
-void runPerformanceTest()
+void runPerformanceTest(int numRuns=500)
 {
     TEST PerfTest;
     long long StartTime = TimeSource::get()->getCurrentMicrosecs();
-    for (int i = 0; i < NUM_RUNS; ++i) {
+    for (int i = 0; i < numRuns; ++i) {
         PerfTest.run();
     }
     double ActiveTime = (TimeSource::get()->getCurrentMicrosecs()-StartTime)/1000.; 
-    cerr << PerfTest.getName() << ": " << ActiveTime/NUM_RUNS << " ms" << endl;
+    cerr << PerfTest.getName() << ": " << ActiveTime/numRuns << " ms" << endl;
     
 }
 
@@ -79,6 +75,19 @@ private:
     std::string m_sName;
 };
 
+class LoadPNGPerfTest: public PerfTestBase {
+public:
+    LoadPNGPerfTest()
+        : PerfTestBase("LoadPNGPerfTest")
+    {
+    }
+
+    void run()
+    {
+        BitmapPtr pBmp(new Bitmap("testfiles/rgb24alpha-64x64.png"));
+    }
+};
+
 class FillI8PerfTest: public PerfTestBase {
 public:
     FillI8PerfTest() 
@@ -87,7 +96,8 @@ public:
         m_pBmp = BitmapPtr(new Bitmap(IntPoint(1024,1024), I8));
     }
 
-    void run() {
+    void run()
+    {
         FilterFill<Pixel8> Filter = FilterFill<Pixel8>(Pixel8(0));
         Filter.applyInPlace(m_pBmp);
     }
@@ -104,7 +114,8 @@ public:
         m_pBmp = BitmapPtr(new Bitmap(IntPoint(1024,1024), R8G8B8));
     }
 
-    void run() {
+    void run()
+    {
         FilterFill<Pixel24> Filter = FilterFill<Pixel24>(Pixel24(0,0,0));
         Filter.applyInPlace(m_pBmp);
     }
@@ -113,6 +124,24 @@ private:
     BitmapPtr m_pBmp;
 };
 
+class FillRGBAPerfTest: public PerfTestBase {
+public:
+    FillRGBAPerfTest() 
+        : PerfTestBase("FillRGBAPerfTest")
+    {
+        m_pBmp = BitmapPtr(new Bitmap(IntPoint(1024,1024), R8G8B8A8));
+    }
+
+    void run()
+    {
+        FilterFill<Pixel32> Filter = FilterFill<Pixel32>(Pixel32(0,0,0,0));
+        Filter.applyInPlace(m_pBmp);
+    }
+
+private:
+    BitmapPtr m_pBmp;
+};
+
 class EqualityI8PerfTest: public PerfTestBase {
 public:
     EqualityI8PerfTest() 
@@ -121,7 +150,8 @@ public:
         m_pBmp = BitmapPtr(new Bitmap(IntPoint(1024,1024), I8));
     }
 
-    void run() {
+    void run() 
+    {
         Bitmap CopyBmp = *m_pBmp;
     }
 
@@ -137,8 +167,9 @@ public:
         m_pBmp = BitmapPtr(new Bitmap(IntPoint(1024,1024), I8));
     }
 
-    void run() {
-        Bitmap CopyBmp (*m_pBmp);
+    void run()
+    {
+        Bitmap CopyBmp(*m_pBmp);
     }
 
 private:
@@ -153,8 +184,26 @@ public:
         m_pBmp = BitmapPtr(new Bitmap(IntPoint(1024,1024), R8G8B8));
     }
 
-    void run() {
-        Bitmap CopyBmp (*m_pBmp);
+    void run()
+    {
+        Bitmap CopyBmp(*m_pBmp);
+    }
+
+private:
+    BitmapPtr m_pBmp;
+};
+
+class CopyRGBAPerfTest: public PerfTestBase {
+public:
+    CopyRGBAPerfTest() 
+        : PerfTestBase("CopyRGBAPerfTest")
+    {
+        m_pBmp = BitmapPtr(new Bitmap(IntPoint(1024,1024), R8G8B8A8));
+    }
+
+    void run()
+    {
+        Bitmap CopyBmp(*m_pBmp);
     }
 
 private:
@@ -171,7 +220,8 @@ public:
         m_pVBmp = BitmapPtr(new Bitmap(IntPoint(512, 512), I8));
     }
 
-    void run() {
+    void run()
+    {
         Bitmap RGBBmp(IntPoint(1024, 1024), B8G8R8X8);
         RGBBmp.copyYUVPixels(*m_pYBmp, *m_pUBmp, *m_pVBmp, false);
     }
@@ -185,12 +235,15 @@ private:
 
 void runPerformanceTests()
 {
+    runPerformanceTest<LoadPNGPerfTest>();
     runPerformanceTest<FillI8PerfTest>();
     runPerformanceTest<FillRGBPerfTest>();
+    runPerformanceTest<FillRGBAPerfTest>();
     runPerformanceTest<EqualityI8PerfTest>();
     runPerformanceTest<CopyI8PerfTest>();
     runPerformanceTest<CopyRGBPerfTest>();
-    runPerformanceTest<YUV2RGBPerfTest>();
+    runPerformanceTest<CopyRGBAPerfTest>();
+    runPerformanceTest<YUV2RGBPerfTest>(200);
 }
 
 int main(int nargs, char** args)
