@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -142,8 +142,8 @@ void getVideoFormatAndMode(IntPoint& Size, PixelFormat pf,
     }
     if (*pVideoMode == -1 || *pVideoFormat == -1) { 
         throw Exception(AVG_ERR_INVALID_ARGS,
-                "Unsupported or illegal value ("+toString(Size.x)+", "+toString(Size.y)+
-                "), "+getPixelFormatString(pf)+"\" for camera mode.");
+                "Unsupported or illegal camera mode ("+toString(Size.x)+", "+toString(Size.y)+
+                "), "+getPixelFormatString(pf)+".");
     }
 }
 
