@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -19,10 +19,11 @@
 //  Current versions can be found at www.libavg.de
 //
 
+#include <InitGuid.h>
+
 #include "DSHelper.h"
 #include "DSCamera.h"
-#include "Ks.h"
-#include "KsMedia.h"
+#include "DSSampleGrabber.h"
 
 #include "../base/Logger.h"
 #include "../base/Exception.h"
@@ -40,16 +41,15 @@ namespace avg {
 
 using namespace std;
 
-DSCamera::DSCamera(std::string sDevice, IntPoint Size, PixelFormat camPF, 
-        PixelFormat destPF, double FrameRate)
+DSCamera::DSCamera(std::string sDevice, IntPoint size, PixelFormat camPF, 
+        PixelFormat destPF, double frameRate)
     : Camera(camPF, destPF),
       m_sDevice(sDevice),
-      m_Size(Size),
-      m_FrameRate(FrameRate),
+      m_Size(size),
+      m_FrameRate(frameRate),
       m_pGraph(0),
       m_pCapture(0),
-      m_pCameraPropControl(0),
-      m_pSampleQueue(0)
+      m_pCameraPropControl(0)
 {
     open();
 }
@@ -68,50 +68,32 @@ void DSCamera::open()
         hr = m_pGraph->AddFilter(m_pSrcFilter, L"Video Capture");
         checkForDShowError(hr, "DSCamera::open()::Add capture filter");
 
-        // Create and configure the sample grabber that delivers the frames to
-        // the app.
-        hr = CoCreateInstance(CLSID_SampleGrabber, NULL, 
-                CLSCTX_INPROC_SERVER, IID_IBaseFilter, (LPVOID *)&m_pGrabFilter);
+        // Create and configure the sample grabber that delivers the frames to the app.
+        m_pGrabFilter = new CSampleGrabber(NULL, &hr);
+        m_pGrabFilter->AddRef();
         checkForDShowError(hr, "DSCamera::open()::Create SampleGrabber");
-        hr = m_pGrabFilter->QueryInterface(IID_ISampleGrabber, 
+        hr = m_pGrabFilter->QueryInterface(IID_IlibavgGrabber, 
                 (void **)&m_pSampleGrabber);
         checkForDShowError(hr, "DSCamera::open()::Create SampleGrabber 2");
 
         hr = m_pSrcFilter->QueryInterface(IID_IAMVideoProcAmp, 
                 (void **)&m_pCameraPropControl);
-        checkForDShowError(hr, "DSCamera::getImage()::get IAMVideoProcAmp");
+        checkForDShowError(hr, "DSCamera::open()::get IAMVideoProcAmp");
         hr = m_pSrcFilter->QueryInterface(IID_IAMCameraControl, 
                 (void **)&m_pAMCameraControl);
-        checkForDShowError(hr, "DSCamera::getImage()::get IAMCameraControl");
-
+        checkForDShowError(hr, "DSCamera::open()::get IAMCameraControl");
 
         hr = m_pGraph->AddFilter(m_pGrabFilter, L"Sample Grabber");
         checkForDShowError(hr, "DSCamera::open()::Add Grabber");
         setCaptureFormat();
 
-        AM_MEDIA_TYPE mt;
-        ZeroMemory(&mt, sizeof(AM_MEDIA_TYPE));
-        mt.majortype = MEDIATYPE_Video;
-        if (!m_bCameraIsColor) {
-            mt.subtype = MEDIASUBTYPE_Y800;
-        }
-
-        hr = m_pSampleGrabber->SetMediaType(&mt);
         checkForDShowError(hr, "DSCamera::open()::SetMediaType");
 
-        m_pSampleQueue = new DSSampleQueue(m_Size, getCamPF(), getDestPF());
-        m_pSampleGrabber->SetCallback(m_pSampleQueue, 0);
-
-/*
-        cerr << "Grabber type: " << mediaTypeToString(mt.majortype) << 
-                ", subtype: " << mediaSubtypeToString(mt.subtype) << 
-                ", formattype: " << mediaFormattypeToString(mt.formattype) <<
-                endl;
-*/
+        m_pSampleGrabber->SetCallback(this);
 
         IBaseFilter * pNull;
         hr = CoCreateInstance(CLSID_NullRenderer, NULL, CLSCTX_INPROC_SERVER, 
-                                       IID_IBaseFilter, (LPVOID*) &pNull);
+                IID_IBaseFilter, (LPVOID*) &pNull);
         checkForDShowError(hr, "DSCamera::open()::Create null filter");
         m_pGraph->AddFilter(pNull, L"NullRender");
         pNull->Release();
@@ -124,13 +106,17 @@ void DSCamera::open()
         hr = AddGraphToRot(m_pGraph, &m_GraphRegisterID);
         checkForDShowError(hr, "DSCamera::open()::AddGraphToRot");
 
-        hr = m_pMediaControl->Run(); // Start capturing
-        checkForDShowError(hr, "DSCamera::open()::Run");
     } else {
         throw (Exception(AVG_ERR_CAMERA_NONFATAL, "DS Camera unavailable"));
     }
 }
 
+void DSCamera::startCapture()
+{
+    HRESULT hr = m_pMediaControl->Run();
+    checkForDShowError(hr, "DSCamera::open()::Run");
+}
+
 void DSCamera::close()
 {
     if (m_pAMCameraControl) {
@@ -147,7 +133,6 @@ void DSCamera::close()
     m_pSrcFilter->Release();
     m_pGrabFilter->Release();
     m_pSampleGrabber->Release();
-    delete m_pSampleQueue;
 }
 
 IntPoint DSCamera::getImgSize()
@@ -157,11 +142,9 @@ IntPoint DSCamera::getImgSize()
 
 BitmapPtr DSCamera::getImage(bool bWait)
 {
-
-    bool bGotFrame = true;
     BitmapPtr pBmp;
     try {
-        pBmp = m_pSampleQueue->getImage(bWait);
+        pBmp = m_BitmapQ.pop(bWait);
     } catch (Exception&) {
         return BitmapPtr();
     }
@@ -170,112 +153,99 @@ BitmapPtr DSCamera::getImage(bool bWait)
 }
 
 void DSCamera::setCaptureFormat()
-{
-    // First, look for a camera image format that matches the color selection
-    bool bFound = selectMediaType(m_bColor, false);
-    // Then, look for one that doesn't match and convert the images afterwards.
-    if (!bFound) {
-        selectMediaType(!m_bColor, true);
-    }
-}
-
-bool DSCamera::selectMediaType(bool bColor, bool bForce) 
 {
     IAMStreamConfig *pSC;
-    HRESULT hr = m_pCapture->FindInterface(&PIN_CATEGORY_CAPTURE,
-            &MEDIATYPE_Video, m_pSrcFilter, IID_IAMStreamConfig, (void **)&pSC);
+    HRESULT hr = m_pCapture->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, 
+            m_pSrcFilter, IID_IAMStreamConfig, (void **)&pSC);
     checkForDShowError(hr, "DSCamera::setCaptureFormat::FindInterface");
 
-    int Count = 0;
-    int Size = 0;
-    hr = pSC->GetNumberOfCapabilities(&Count, &Size);
+    int numCaps = 0;
+    int capsSize = 0;
+    hr = pSC->GetNumberOfCapabilities(&numCaps, &capsSize);
     checkForDShowError(hr, "DSCamera::dumpMediaTypes::GetNumberOfCapabilities");
 
-    AVG_ASSERT(Size == sizeof(VIDEO_STREAM_CONFIG_CAPS));
+    AVG_ASSERT(capsSize == sizeof(VIDEO_STREAM_CONFIG_CAPS));
     bool bFormatFound = false;
     bool bCloseFormatFound = false;
-    AM_MEDIA_TYPE *pmtConfig;
-    AM_MEDIA_TYPE *pmtCloseConfig;
+    AM_MEDIA_TYPE* pmtConfig;
+    AM_MEDIA_TYPE* pmtCloseConfig;
     vector<string> sImageFormats;
-    VIDEOINFOHEADER * pvih;
+    VIDEOINFOHEADER* pvih;
     BITMAPINFOHEADER bih;
-    for (int i = 0; i < Count; i++) {
+    PixelFormat capsPF;
+    for (int i = 0; i < numCaps; i++) {
         VIDEO_STREAM_CONFIG_CAPS scc;
         hr = pSC->GetStreamCaps(i, &pmtConfig, (BYTE*)&scc);
         checkForDShowError(hr, "DSCamera::dumpMediaTypes::GetStreamCaps");
         pvih = (VIDEOINFOHEADER*)(pmtConfig->pbFormat);
         bih = pvih->bmiHeader;
-        double FrameRate = double(10000000L/pvih->AvgTimePerFrame);
-        stringstream ss;
-        ss << "  " << i << ": (" << bih.biWidth << "x" << bih.biHeight << "), " << 
-                mediaSubtypeToString(pmtConfig->subtype) << 
-                ", " << 10000000./pvih->AvgTimePerFrame << " fps.";
-        sImageFormats.push_back(ss.str());
+        double frameRate = double(10000000L/pvih->AvgTimePerFrame);
+        capsPF = mediaSubtypeToPixelFormat(pmtConfig->subtype);
 
-        if (bih.biWidth == m_Size.x && bih.biHeight == m_Size.y && 
-                ((getCamPF() == I8 || getCamPF() == BAYER8_GBRG) && pmtConfig->subtype == MEDIASUBTYPE_Y800) ||
-                 (getCamPF() == YCbCr422 && pmtConfig->subtype == MEDIASUBTYPE_UYVY)
-            )
+        if (capsPF != NO_PIXELFORMAT && bih.biWidth != 0) {
+            sImageFormats.push_back(camImageFormatToString(pmtConfig));
+        }
+
+        bool bFormatUsed = false;
+        int height = bih.biHeight;
+        if (height < 0) {
+            height = -height;
+        }
+        if (bih.biWidth == m_Size.x && height == m_Size.y && 
+                (getCamPF() == capsPF || (getCamPF() == BAYER8_GBRG && capsPF == I8)))
         {
-            if (fabs(m_FrameRate-FrameRate) < 0.001) {
+            if (fabs(m_FrameRate-frameRate) < 0.001) {
                 bFormatFound = true;
                 break;
             } else if (!bCloseFormatFound) {
                 // The current format fits everything but the framerate.
-                // Some cameras (Point Grey FireFly, for instance) don't report
-                // all framerates, so we're going to try that as well.
+                // Not all framerates are reported, so we're going to try this one as 
+                // well.
                 bCloseFormatFound = true;
+                bFormatUsed = true;
                 pmtCloseConfig = pmtConfig;
-            } else {
-                CoTaskMemFree((PVOID)pmtConfig->pbFormat);
-                CoTaskMemFree(pmtConfig);
             }
-        } else {
+        }
+        if (!bFormatUsed) {
             CoTaskMemFree((PVOID)pmtConfig->pbFormat);
             CoTaskMemFree(pmtConfig);
         }
     }
     if (bFormatFound) {
-        AVG_TRACE(Logger::CONFIG, "Camera image format: (" << bih.biWidth 
-                << "x" << bih.biHeight << "), "
-                << mediaSubtypeToString(pmtConfig->subtype) << ", " 
-                << 10000000./pvih->AvgTimePerFrame << " fps.");
+        AVG_TRACE(Logger::CONFIG, "Camera image format: "
+                << camImageFormatToString(pmtConfig));
+        int height = ((VIDEOINFOHEADER*)(pmtConfig->pbFormat))->bmiHeader.biHeight;
+//        m_bUpsideDown = (height < 0);
         hr = pSC->SetFormat(pmtConfig);
         checkForDShowError(hr, "DSCamera::dumpMediaTypes::SetFormat");
         CoTaskMemFree((PVOID)pmtConfig->pbFormat);
         CoTaskMemFree(pmtConfig);
-        m_bCameraIsColor = bColor;
     } else {
         if (bCloseFormatFound) {
             // Set the framerate manually.
             pvih = (VIDEOINFOHEADER*)(pmtCloseConfig->pbFormat);
-            bih = pvih->bmiHeader;
             pvih->AvgTimePerFrame = REFERENCE_TIME(10000000/m_FrameRate);
-            AVG_TRACE(Logger::CONFIG, "Camera image format: (" << bih.biWidth 
-                    << "x" << bih.biHeight << "), "
-                    << mediaSubtypeToString(pmtCloseConfig->subtype) << ", " 
-                    << 10000000./pvih->AvgTimePerFrame << " fps (set manually).");
+            int height = pvih->bmiHeader.biHeight;
+//            m_bUpsideDown = (height < 0);
             hr = pSC->SetFormat(pmtCloseConfig);
             checkForDShowError(hr, "DSCamera::dumpMediaTypes::SetFormat");
+            AVG_TRACE(Logger::CONFIG, "Camera image format: " 
+                    << camImageFormatToString(pmtCloseConfig));
             CoTaskMemFree((PVOID)pmtCloseConfig->pbFormat);
             CoTaskMemFree(pmtCloseConfig);
-            m_bCameraIsColor = bColor;
 
-            // Check if the framerate is what we wanted.
-//            pSC->GetFormat(&p
-            bFormatFound = true;
-        } else if (bForce) {
+            // TODO: Check if framerate is actually attained.
+        } else {
             AVG_TRACE(Logger::WARNING, 
-                "Possibly incomplete list of image formats supported by camera: ");
-            for (unsigned i=0; i<sImageFormats.size(); i++) {
-                AVG_TRACE(Logger::WARNING, sImageFormats[i]);
+                "Possibly incomplete list of camera image formats: ");
+            for (unsigned i = 0; i < sImageFormats.size(); i++) {
+                AVG_TRACE(Logger::WARNING, "  " << sImageFormats[i]);
             }
             throw Exception(AVG_ERR_CAMERA_NONFATAL, 
-                    "Could not find suitable camera image format.");
+                    "Could not find requested camera image format.");
         }
     }
     pSC->Release();
-    return bFormatFound;
 }
 
 
@@ -295,60 +265,61 @@ double DSCamera::getFrameRate() const
     return m_FrameRate;
 }
 
-int DSCamera::getFeature(CameraFeature Feature) const
+int DSCamera::getFeature(CameraFeature feature) const
 {
-    long Prop = getDSFeatureID(Feature);
-    long Val;
-    long Flags;
+    long prop = getDSFeatureID(feature);
+    long val;
+    long flags;
     HRESULT hr;
-    if (isDSFeatureCamControl(Feature)) {
-        hr = m_pAMCameraControl->Get(Prop, &Val, &Flags);
+    if (isDSFeatureCamControl(feature)) {
+        hr = m_pAMCameraControl->Get(prop, &val, &flags);
     } else {
-        hr = m_pCameraPropControl->Get(Prop, &Val, &Flags);
+        hr = m_pCameraPropControl->Get(prop, &val, &flags);
     }
     if (!SUCCEEDED(hr)) {
         AVG_TRACE(Logger::WARNING, "DSCamera::getFeature "
-                +cameraFeatureToString(Feature)+" failed.");
+                + cameraFeatureToString(feature)+" failed.");
         return 0;
     }
-    return Val;
+    return val;
 }
 
-void DSCamera::setFeature(CameraFeature Feature, int Value, bool bIgnoreOldValue)
+void DSCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValue)
 {
-    long Prop = getDSFeatureID(Feature);
+    long prop = getDSFeatureID(feature);
     if (!m_pCameraPropControl) {
         return;
     }
-    long Flags;
-    if (Value == -1) {
-        Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
+    long flags;
+    if (value == -1) {
+        flags = VideoProcAmp_Flags_Auto;
     } else {
-        Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
+        flags = VideoProcAmp_Flags_Manual;
     }
     HRESULT hr;
-    if (isDSFeatureCamControl(Feature)) {
-        hr = m_pAMCameraControl->Set(Prop, Value, Flags);
+    if (isDSFeatureCamControl(feature)) {
+        hr = m_pAMCameraControl->Set(prop, value, flags);
     } else {
-        hr = m_pCameraPropControl->Set(Prop, Value, Flags);
+        hr = m_pCameraPropControl->Set(prop, value, flags);
     }
     switch (hr) {
         case E_INVALIDARG:
             // TODO: Throw exception
             AVG_TRACE(Logger::ERROR, "DSCamera::setFeature(" 
-                    << cameraFeatureToString(Feature) << ", " << Value << ") failed.");
+                    << cameraFeatureToString(feature) << ", " << value << ") failed.");
             break;
         case E_PROP_ID_UNSUPPORTED:  
         case E_PROP_SET_UNSUPPORTED:
             AVG_TRACE(Logger::ERROR, "DSCamera::setFeature(" 
-                << cameraFeatureToString(Feature) << ") failed: Feature not supported by camera.");
+                << cameraFeatureToString(feature)
+                << ") failed: Feature not supported by camera.");
             break;
         default:
             checkForDShowError(hr, "DSCamera::setFeature()::Set value");
     }
 }
 
-void DSCamera::setFeatureOneShot(CameraFeature Feature)
+void DSCamera::setFeatureOneShot(CameraFeature feature)
 {
     AVG_TRACE(Logger::WARNING, 
             "OneShot feature setting not implemented for DirectShow camera driver.");
@@ -374,6 +345,27 @@ void DSCamera::setWhitebalance(int u, int v, bool bIgnoreOldValue)
             "Whitebalance not implemented for DirectShow camera driver.");
 }
 
+void DSCamera::onSample(IMediaSample * pSample)
+{
+    unsigned char * pData;
+
+    // Get the current image.
+    pSample->GetPointer(&pData);
+
+    int stride = m_Size.x*getBytesPerPixel(getCamPF());
+    Bitmap camBmp(m_Size, getCamPF(), pData, stride, false, "CameraImage");
+    // Copy over to bitmap queue, doing pixel format conversion if necessary.
+    BitmapPtr pDestBmp = BitmapPtr(new Bitmap(m_Size, getDestPF(), 
+            "ConvertedCameraImage"));
+    pDestBmp->copyPixels(camBmp);
+/*
+    if (m_bUpsideDown) {
+        FilterFlip().applyInPlace(pDestBmp);
+    }
+*/
+    m_BitmapQ.push(pDestBmp);
+}
+
 void DSCamera::dumpCameras()
 {
     HRESULT hr = S_OK;
@@ -383,21 +375,20 @@ void DSCamera::dumpCameras()
 
     // Create the system device enumerator
     ICreateDevEnum *pDevEnum =NULL;
-    hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC,
+    hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC, 
             IID_ICreateDevEnum, (void **) &pDevEnum);
     checkForDShowError(hr, "DSCamera::dumpCameras()::CreateDevEnum");
 
     // Create an enumerator for the video capture devices
     IEnumMoniker *pClassEnum = NULL;
-    hr = pDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, 
-            &pClassEnum, 0);
+    hr = pDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &pClassEnum, 0);
     checkForDShowError(hr, "DSCamera::dumpCameras()::CreateClassEnumerator");
 
     if (pClassEnum == NULL) {
         return;
     }
 
-    IMoniker* pMoniker =NULL;
+    IMoniker* pMoniker = NULL;
     bool bFirst = true;
     while (pClassEnum->Next(1, &pMoniker, NULL) == S_OK) {
         if (bFirst) {
@@ -405,7 +396,7 @@ void DSCamera::dumpCameras()
             cerr << "DirectShow cameras: " << endl;
             bFirst = false;
         }
-        IPropertyBag *pPropBag;
+        IPropertyBag* pPropBag;
         hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void**)(&pPropBag));
         checkForDShowError(hr, "DSCamera::dumpCameras()::BindToStorage");
         cerr << "  ----------------------------" << endl;
@@ -421,18 +412,16 @@ void DSCamera::dumpCameras()
 
 void DSCamera::initGraphBuilder()
 {
-    HRESULT hr;
-    // TODO: Check if the threading model is ok.
-    hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
+    HRESULT hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
     checkForDShowError(hr, "DSCamera::initGraphBuilder()::CoInitializeEx");
 
     // Create the filter graph
-    hr = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC,
-            IID_IGraphBuilder, (void **) &m_pGraph);
+    hr = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC, IID_IGraphBuilder, 
+            (void **) &m_pGraph);
     checkForDShowError(hr, "DSCamera::initGraphBuilder()::GraphBuilder");
     // Create the capture graph builder
     hr = CoCreateInstance (CLSID_CaptureGraphBuilder2 , NULL, CLSCTX_INPROC,
-                           IID_ICaptureGraphBuilder2, (void **) &m_pCapture);   
+            IID_ICaptureGraphBuilder2, (void **) &m_pCapture);   
     checkForDShowError(hr, "DSCamera::initGraphBuilder()::CaptureGraphBuilder2");
 
     hr = m_pCapture->SetFiltergraph(m_pGraph);
@@ -445,18 +434,18 @@ void DSCamera::findCaptureDevice(IBaseFilter ** ppSrcFilter)
 {
     HRESULT hr = S_OK;
     IBaseFilter * pSrc = NULL;
-    IMoniker* pMoniker =NULL;
-    ICreateDevEnum *pDevEnum =NULL;
+    IMoniker* pMoniker = NULL;
+    ICreateDevEnum *pDevEnum = NULL;
     IEnumMoniker *pClassEnum = NULL;
 
     // Create the system device enumerator
-    hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC,
-                           IID_ICreateDevEnum, (void **) &pDevEnum);
+    hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC, 
+            IID_ICreateDevEnum, (void **) &pDevEnum);
     checkForDShowError(hr, "DSCamera::findCaptureDevice()::CreateDevEnum");
 
     // Create an enumerator for the video capture devices
-    hr = pDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, 
-            &pClassEnum, 0);
+    hr = pDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &pClassEnum, 
+            0);
     checkForDShowError(hr, "DSCamera::findCaptureDevice()::CreateClassEnumerator");
 
     // If there are no enumerators for the requested type, then 
@@ -476,7 +465,9 @@ void DSCamera::findCaptureDevice(IBaseFilter ** ppSrcFilter)
         string sFriendlyName = getStringProp(pPropBag, L"FriendlyName");
         string sDevicePath = getStringProp(pPropBag, L"DevicePath");
 
-        if (m_sDevice == sDescription  || m_sDevice == sFriendlyName || sDevicePath.find(m_sDevice) != -1 || m_sDevice == "") {
+        if (m_sDevice == sDescription  || m_sDevice == sFriendlyName || 
+                sDevicePath.find(m_sDevice) != -1 || m_sDevice == "")
+        {
             bFound = true;
         } else {
             pMoniker->Release();
@@ -495,7 +486,8 @@ void DSCamera::findCaptureDevice(IBaseFilter ** ppSrcFilter)
             checkForDShowError(hr, "DSCamera::findCaptureDevice()::BindToStorage");
             pPropBag->Release();
         } else {
-            throw(Exception(AVG_ERR_CAMERA_NONFATAL, "No DirectShow Capture Device found"));
+            throw(Exception(AVG_ERR_CAMERA_NONFATAL, 
+                    "No DirectShow Capture Device found"));
         }
     }
 
@@ -523,13 +515,13 @@ void DSCamera::connectFilters(IGraphBuilder *pGraph, IBaseFilter *pSrc,
     IPin *pIn = 0;
     getUnconnectedPin(pDest, PINDIR_INPUT, &pIn);
     
-    HRESULT hr = pGraph->Connect(pOut, pIn);
+    HRESULT hr = pGraph->ConnectDirect(pOut, pIn, 0);
     checkForDShowError(hr, "DSCamera::ConnectFilters::Connect");
     pOut->Release();
     pIn->Release();
 }
 
-void DSCamera::getUnconnectedPin(IBaseFilter *pFilter, PIN_DIRECTION PinDir, IPin **ppPin)
+void DSCamera::getUnconnectedPin(IBaseFilter *pFilter, PIN_DIRECTION pinDir, IPin **ppPin)
 {
     *ppPin = 0;
     IEnumPins *pEnum = 0;
@@ -538,11 +530,10 @@ void DSCamera::getUnconnectedPin(IBaseFilter *pFilter, PIN_DIRECTION PinDir, IPi
     checkForDShowError(hr, "DSCamera::ConnectFilters::Connect");
     while (pEnum->Next(1, &pPin, NULL) == S_OK)
     {
-        PIN_DIRECTION ThisPinDir;
-        pPin->QueryDirection(&ThisPinDir);
-        if (ThisPinDir == PinDir)
-        {
-            IPin *pTmp = 0;
+        PIN_DIRECTION thisPinDir;
+        pPin->QueryDirection(&thisPinDir);
+        if (thisPinDir == pinDir) {
+            IPin* pTmp = 0;
             hr = pPin->ConnectedTo(&pTmp);
             if (SUCCEEDED(hr)) { // Already connected, not the pin we want.
                 pTmp->Release();
@@ -560,7 +551,7 @@ void DSCamera::getUnconnectedPin(IBaseFilter *pFilter, PIN_DIRECTION PinDir, IPi
 }
 
 #pragma warning(disable : 4995)
-void DSCamera::checkForDShowError(HRESULT hr, const string & sLocation)
+void DSCamera::checkForDShowError(HRESULT hr, const string& sLocation)
 {
     if (SUCCEEDED(hr)) {
         return;
