@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,7 @@
 #ifndef _DSCamera_H_
 #define _DSCamera_H_
 
-#include "DSSampleQueue.h"
+#include "IDSSampleCallback.h"
 
 #include "../avgconfigwrapper.h"
 
@@ -33,7 +33,6 @@
 
 #include "../base/Point.h"
 #include "../base/Queue.h"
-#include "../base/WorkerThread.h"
 
 #include <string>
 #define _WIN32_DCOM
@@ -43,11 +42,15 @@
 
 namespace avg {
 
-class DSCamera: public Camera {
+struct IlibavgGrabber;
+
+class DSCamera: public Camera, IDSSampleCallback
+{
 public:
-    DSCamera(std::string sDevice, IntPoint Size, PixelFormat camPF, 
-            PixelFormat destPF, double FrameRate);
+    DSCamera(std::string sDevice, IntPoint size, PixelFormat camPF, PixelFormat destPF,
+            double frameRate);
     virtual ~DSCamera();
+    virtual void startCapture();
 
     virtual IntPoint getImgSize();
     virtual BitmapPtr getImage(bool bWait);
@@ -56,13 +59,16 @@ public:
     virtual const std::string& getDriverName() const; 
     virtual double getFrameRate() const;
 
-    virtual int getFeature(CameraFeature Feature) const;
-    virtual void setFeature(CameraFeature Feature, int Value, bool bIgnoreOldValue=false);
-    virtual void setFeatureOneShot(CameraFeature Feature);
+    virtual int getFeature(CameraFeature feature) const;
+    virtual void setFeature(CameraFeature feature, int value, 
+            bool bIgnoreOldValue=false);
+    virtual void setFeatureOneShot(CameraFeature feature);
     virtual int getWhitebalanceU() const;
     virtual int getWhitebalanceV() const;
     virtual void setWhitebalance(int u, int v, bool bIgnoreOldValue=false);
 
+    virtual void onSample(IMediaSample * pSample);
+
     static void dumpCameras();
 
 private:
@@ -72,31 +78,27 @@ private:
     void findCaptureDevice(IBaseFilter ** ppSrcFilter);
 
     void setCaptureFormat();
-    bool selectMediaType(bool bColor, bool bForce);
-    void connectFilters(IGraphBuilder *pGraph, IBaseFilter *pSrc, 
-            IBaseFilter *pDest);
-    void getUnconnectedPin(IBaseFilter *pFilter, PIN_DIRECTION PinDir, 
-            IPin **ppPin);
+    void connectFilters(IGraphBuilder *pGraph, IBaseFilter *pSrc, IBaseFilter *pDest);
+    void getUnconnectedPin(IBaseFilter *pFilter, PIN_DIRECTION pinDir, IPin **ppPin);
     static void checkForDShowError(HRESULT hr, const std::string & sAppMsg);
 
     std::string m_sDevice;
     IntPoint m_Size;
     double m_FrameRate;
-    bool m_bColor;
-    bool m_bCameraIsColor;
 
     IGraphBuilder * m_pGraph;
     ICaptureGraphBuilder2 * m_pCapture;
     IMediaControl * m_pMediaControl;
     IBaseFilter * m_pSrcFilter;
     IBaseFilter * m_pGrabFilter;
-    ISampleGrabber * m_pSampleGrabber;
+    IlibavgGrabber * m_pSampleGrabber;
     IAMVideoProcAmp * m_pCameraPropControl;
     IAMCameraControl * m_pAMCameraControl;
 
-    DSSampleQueue * m_pSampleQueue;
+    Queue<Bitmap> m_BitmapQ;
 
     DWORD m_GraphRegisterID;
+//    bool m_bUpsideDown;
 };
 
 }
