@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,6 @@
 #include <string>
 
 #define _WIN32_DCOM
-#include <Winsock2.h>
 #include <windows.h>
 #include <dshow.h>
 #include "Qedit.h"
@@ -43,19 +42,20 @@ static const GUID MEDIASUBTYPE_Y160 =
 
 namespace avg {
 
-HRESULT AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister);
+HRESULT AddGraphToRot(IUnknown* pUnkGraph, DWORD* pdwRegister);
 void RemoveGraphFromRot(DWORD pdwRegister);
 
-std::string getStringProp(IPropertyBag *pPropBag, LPCOLESTR pszPropName);
+std::string getStringProp(IPropertyBag* pPropBag, LPCOLESTR pszPropName);
 
-PixelFormat mediaSubtypeToPixelFormat(const GUID& Subtype, const std::string DesiredPF);
+PixelFormat mediaSubtypeToPixelFormat(const GUID& subtype);
 
-std::string mediaTypeToString(const GUID& Type);
-std::string mediaSubtypeToString(const GUID& Subtype);
-std::string mediaFormattypeToString(const GUID & Formattype);
+std::string mediaTypeToString(const GUID& type);
+std::string mediaSubtypeToString(const GUID& subtype);
+std::string mediaFormattypeToString(const GUID& formattype);
+std::string camImageFormatToString(const AM_MEDIA_TYPE* pMediaType);
 
-bool isDSFeatureCamControl(CameraFeature Feature);
-long getDSFeatureID(CameraFeature Feature);
+bool isDSFeatureCamControl(CameraFeature feature);
+long getDSFeatureID(CameraFeature feature);
 
 }
 
