@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,6 +29,7 @@
 #include "../base/ScopeTimer.h"
 #include "../base/StringHelper.h"
 
+#include <string.h>
 #include <sstream>
 #include <iomanip>
 
@@ -46,6 +47,10 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
 {
 #ifdef AVG_ENABLE_1394_2
     m_FrameRateConstant = getFrameRateConst(m_FrameRate);
+    if (camPF == I16) {
+        throw Exception(AVG_ERR_CAMERA_NONFATAL, 
+                "I16 pixel format is not supported for firewire cameras.");
+    }
     m_Mode = getCamMode(size, camPF);
     dc1394camera_list_t * pCameraList;
 
@@ -82,7 +87,7 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
             }       
         }
         if (id_to_use == -1) {
-            AVG_TRACE(Logger::WARNING, "Firewire GUID=" << guid 
+            AVG_TRACE(Logger::WARNING, "Firewire GUID=" << hex << guid << dec
                     << " requested but not found on bus. Using first camera");
             id_to_use = 0;
         }
@@ -103,7 +108,6 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
 
     dc1394_camera_free_list(pCameraList);
 
-    dumpCameraInfo();
     if (bFW800) {
         dc1394_video_set_operation_mode(m_pCamera, DC1394_OPERATION_MODE_1394B);
         err = dc1394_video_set_iso_speed(m_pCamera, DC1394_ISO_SPEED_800);
@@ -151,7 +155,26 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
         dc1394_free(m_pDC1394);
         throw Exception(AVG_ERR_CAMERA_NONFATAL, "Failed to initialize camera");
     }
-    err = dc1394_video_set_transmission(m_pCamera, DC1394_ON);
+#else
+    AVG_ASSERT(false);
+#endif
+}
+
+FWCamera::~FWCamera()
+{
+#ifdef AVG_ENABLE_1394_2
+    dc1394_video_set_transmission(m_pCamera, DC1394_OFF);
+    dc1394_capture_stop(m_pCamera);
+    dc1394_camera_free(m_pCamera);
+    dc1394_free(m_pDC1394);
+#endif
+    AVG_TRACE(Logger::CONFIG, "Firewire camera closed.");
+}
+
+void FWCamera::startCapture()
+{
+#ifdef AVG_ENABLE_1394_2
+    int err = dc1394_video_set_transmission(m_pCamera, DC1394_ON);
     AVG_ASSERT(err == DC1394_SUCCESS);
 
     dc1394switch_t status = DC1394_OFF;
@@ -168,6 +191,7 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
     }
     // Default to turning off any camera sharpness manipulation.
     setFeature(CAM_FEATURE_SHARPNESS, 0);
+
     // Turn off possible auto exposure.
     dc1394_feature_set_mode(m_pCamera, DC1394_FEATURE_EXPOSURE, 
             DC1394_FEATURE_MODE_MANUAL);
@@ -179,7 +203,7 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
     }
     setWhitebalance(m_WhitebalanceU, m_WhitebalanceV, true);
     
-    if (camPF == BAYER8) {
+    if (getCamPF() == BAYER8) {
         if (strcmp(m_pCamera->model, "DFx 31BF03") == 0) {
             AVG_TRACE(Logger::CONFIG,
                     "Applying bayer pattern fixup for IS DFx31BF03 camera");
@@ -190,22 +214,9 @@ FWCamera::FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size,
             enablePtGreyBayer();
         }
     }
-#else
-    AVG_ASSERT(false);
 #endif
 }
 
-FWCamera::~FWCamera()
-{
-#ifdef AVG_ENABLE_1394_2
-    dc1394_video_set_transmission(m_pCamera, DC1394_OFF);
-    dc1394_capture_stop(m_pCamera);
-    dc1394_camera_free(m_pCamera);
-    dc1394_free(m_pDC1394);
-#endif
-    AVG_TRACE(Logger::CONFIG, "Firewire camera closed.");
-}
-
 IntPoint FWCamera::getImgSize()
 {
 #ifdef AVG_ENABLE_1394_2
@@ -239,7 +250,7 @@ BitmapPtr FWCamera::getImage(bool bWait)
         if (getCamPF() == YCbCr411) {
             lineLen = m_Size.x*1.5;
         } else {
-            lineLen = m_Size.x*Bitmap::getBytesPerPixel(getCamPF());
+            lineLen = m_Size.x*getBytesPerPixel(getCamPF());
         }
         BitmapPtr pCamBmp(new Bitmap(m_Size, getCamPF(), pCaptureBuffer, lineLen, false,
                 "TempCameraBmp"));
@@ -298,22 +309,42 @@ int FWCamera::getFeature(CameraFeature feature) const
 #endif
 }
 
+bool FWCamera::hasFeature(CameraFeature feature)
+{
+#ifdef AVG_ENABLE_1394_2
+    if (feature == CAM_FEATURE_STROBE_DURATION) {
+        // FIXME
+        return true; 
+    } else {
+        dc1394feature_t featureID = getFeatureID(feature);
+        dc1394bool_t bAvailable;
+        dc1394_feature_is_present(m_pCamera, featureID, &bAvailable);
+        return bAvailable;
+    }
+#else
+    return false;
+#endif
+}
+
 void FWCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValue)
 {
 #ifdef AVG_ENABLE_1394_2
-    if (bIgnoreOldValue || m_Features[feature] != value) {
-        m_Features[feature] = value;
-        if (feature == CAM_FEATURE_STROBE_DURATION) {
-            try {
-                setStrobeDuration(value);
-            } catch (Exception& e) {
-                AVG_TRACE(Logger::WARNING, 
-                        string("Camera: Setting strobe duration failed. ")+e.GetStr());
+    if (hasFeature(feature)) { 
+        if (bIgnoreOldValue || m_Features[feature] != value) {
+            m_Features[feature] = value;
+            if (feature == CAM_FEATURE_STROBE_DURATION) {
+                try {
+                    setStrobeDuration(value);
+                } catch (Exception& e) {
+                    AVG_TRACE(Logger::WARNING, 
+                            string("Camera: Setting strobe duration failed. ") + 
+                            e.getStr());
+                }
+            } else {
+                dc1394feature_t featureID = getFeatureID(feature);
+                setFeature(featureID, value);
+                //        dumpCameraInfo();
             }
-        } else {
-            dc1394feature_t FeatureID = getFeatureID(feature);
-            setFeature(FeatureID, value);
-            //        dumpCameraInfo();
         }
     }
 #endif
@@ -322,8 +353,8 @@ void FWCamera::setFeature(CameraFeature feature, int value, bool bIgnoreOldValue
 void FWCamera::setFeatureOneShot(CameraFeature feature)
 {
 #ifdef AVG_ENABLE_1394_2
-    dc1394feature_t FeatureID = getFeatureID(feature);
-    dc1394error_t err = dc1394_feature_set_mode(m_pCamera, FeatureID, 
+    dc1394feature_t featureID = getFeatureID(feature);
+    dc1394error_t err = dc1394_feature_set_mode(m_pCamera, featureID, 
             DC1394_FEATURE_MODE_ONE_PUSH_AUTO);
     if (err != DC1394_SUCCESS) {
         AVG_TRACE(Logger::WARNING, "Camera: Unable to set one-shot for " 
@@ -351,21 +382,23 @@ int FWCamera::getWhitebalanceV() const
 void FWCamera::setWhitebalance(int u, int v, bool bIgnoreOldValue)
 {
 #ifdef AVG_ENABLE_1394_2
-    if (bIgnoreOldValue || u != m_WhitebalanceU || v != m_WhitebalanceV) {
-        m_WhitebalanceU = u;
-        m_WhitebalanceV = v;
-        dc1394error_t err;
-        if (u == -1) {
-            err = dc1394_feature_set_mode(m_pCamera, DC1394_FEATURE_WHITE_BALANCE,
-                    DC1394_FEATURE_MODE_AUTO);
-        } else {
-            err = dc1394_feature_set_mode(m_pCamera, DC1394_FEATURE_WHITE_BALANCE, 
-                    DC1394_FEATURE_MODE_MANUAL);
-            err = dc1394_feature_whitebalance_set_value(m_pCamera, u, v);
-        }
-        if (err != DC1394_SUCCESS) {
-            AVG_TRACE(Logger::WARNING,
-                    "Camera: Unable to set whitebalance. Error was " << err);
+    if (hasFeature(CAM_FEATURE_WHITE_BALANCE)) {
+        if (bIgnoreOldValue || u != m_WhitebalanceU || v != m_WhitebalanceV) {
+            m_WhitebalanceU = u;
+            m_WhitebalanceV = v;
+            dc1394error_t err;
+            if (u == -1) {
+                err = dc1394_feature_set_mode(m_pCamera, DC1394_FEATURE_WHITE_BALANCE,
+                        DC1394_FEATURE_MODE_AUTO);
+            } else {
+                err = dc1394_feature_set_mode(m_pCamera, DC1394_FEATURE_WHITE_BALANCE, 
+                        DC1394_FEATURE_MODE_MANUAL);
+                err = dc1394_feature_whitebalance_set_value(m_pCamera, u, v);
+            }
+            if (err != DC1394_SUCCESS) {
+                AVG_TRACE(Logger::WARNING,
+                        "Camera: Unable to set whitebalance. Error was " << err);
+            }
         }
     }
 #endif
@@ -475,19 +508,6 @@ void FWCamera::enablePtGreyBayer()
 #endif
 }
 
-void FWCamera::dumpCameraInfo()
-{
-#ifdef AVG_ENABLE_1394_2
-    dc1394error_t err;
-    dc1394featureset_t FeatureSet;
-    err = dc1394_feature_get_all(m_pCamera, &FeatureSet);
-    AVG_ASSERT(err == DC1394_SUCCESS);
-    // TODO: do this using AVG_TRACE
-    dc1394_feature_print_all(&FeatureSet, stderr);
-
-#endif
-}
-
 void FWCamera::dumpCameras()
 {
 #ifdef AVG_ENABLE_1394_2
@@ -506,7 +526,9 @@ void FWCamera::dumpCameras()
                         id.unit);
                 if (pCamera) {
                     dc1394_camera_print_info(pCamera, stderr);
+                    dumpCameraInfo(pCamera);
                     dc1394_camera_free(pCamera);
+                    cerr << endl;
                 }
             }
         }
@@ -516,6 +538,18 @@ void FWCamera::dumpCameras()
 #endif
 }
 
+#ifdef AVG_ENABLE_1394_2
+void FWCamera::dumpCameraInfo(dc1394camera_t * pCamera)
+{
+    dc1394error_t err;
+    dc1394featureset_t FeatureSet;
+    err = dc1394_feature_get_all(pCamera, &FeatureSet);
+    AVG_ASSERT(err == DC1394_SUCCESS);
+    // TODO: do this using AVG_TRACE
+    dc1394_feature_print_all(&FeatureSet, stderr);
+}
+#endif
+
 void FWCamera::resetBus()
 {
 #ifdef AVG_ENABLE_1394_2
