@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,6 @@
 #include "../graphics/Pixel24.h"
 
 #include "../base/Point.h"
-#include "../base/WorkerThread.h"
 
 #ifdef AVG_ENABLE_1394_2
 #include <dc1394/control.h>
@@ -53,6 +52,7 @@ public:
     FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size, PixelFormat camPF, 
             PixelFormat destPF, double frameRate);
     virtual ~FWCamera();
+    virtual void startCapture();
 
     virtual IntPoint getImgSize();
     virtual BitmapPtr getImage(bool bWait);
@@ -62,6 +62,7 @@ public:
     virtual double getFrameRate() const;
 
     virtual int getFeature(CameraFeature feature) const;
+    bool hasFeature(CameraFeature feature);
     virtual void setFeature(CameraFeature feature, int value, bool bIgnoreOldValue=false);
     virtual void setFeatureOneShot(CameraFeature feature);
     virtual int getWhitebalanceU() const;
@@ -85,8 +86,8 @@ private:
     dc1394camera_t * m_pCamera;
     dc1394framerate_t m_FrameRateConstant; 
     dc1394video_mode_t m_Mode;            
+    static void dumpCameraInfo(dc1394camera_t * pCamera);
 #endif
-    void dumpCameraInfo();
 
     FeatureMap m_Features;
     int m_WhitebalanceU;
