@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,6 @@
 //
 
 #include "CameraNode.h"
-#include "DisplayEngine.h"
 #include "OGLSurface.h"
 #include "NodeDefinition.h"
 
@@ -30,6 +29,8 @@
 #include "../base/XMLHelper.h"
 
 #include "../graphics/Filterfill.h"
+#include "../graphics/TextureMover.h"
+#include "../graphics/GLTexture.h"
 
 #include "../imaging/Camera.h"
 #include "../imaging/FWCamera.h"
@@ -47,7 +48,7 @@ namespace avg {
 
 NodeDefinition CameraNode::createDefinition()
 {
-    return NodeDefinition("camera", VisibleNode::buildNode<CameraNode>)
+    return NodeDefinition("camera", Node::buildNode<CameraNode>)
         .extendDefinition(RasterNode::createDefinition())
         .addArg(Arg<string>("driver", "firewire"))
         .addArg(Arg<string>("device", ""))
@@ -70,7 +71,8 @@ NodeDefinition CameraNode::createDefinition()
 CameraNode::CameraNode(const ArgList& args)
     : m_bIsPlaying(false),
       m_FrameNum(0),
-      m_bIsAutoUpdateCameraImage(true)
+      m_bIsAutoUpdateCameraImage(true),
+      m_bNewBmp(false)
 {
     args.setMembers(this);
     string sDriver = args.getArgVal<string>("driver");
@@ -115,15 +117,31 @@ CameraNode::~CameraNode()
     m_pCamera = CameraPtr();
 }
 
-void CameraNode::setRenderingEngines(DisplayEngine * pDisplayEngine, 
-        AudioEngine * pAudioEngine)
+void CameraNode::connectDisplay()
 {
-    RasterNode::setRenderingEngines(pDisplayEngine, pAudioEngine);
+    RasterNode::connectDisplay();
     if (m_bIsPlaying) {
         open();
     }
 }
 
+void CameraNode::connect(CanvasPtr pCanvas)
+{
+    if (!m_pCamera) {
+        throw Exception(AVG_ERR_UNSUPPORTED, 
+                "Can't use camera node after disconnect(True).");
+    }
+    RasterNode::connect(pCanvas);
+}
+
+void CameraNode::disconnect(bool bKill)
+{
+    if (bKill) {
+        m_pCamera = CameraPtr();
+    }
+    RasterNode::disconnect(bKill);
+}
+
 void CameraNode::play()
 {
     if (getState() == NS_CANRENDER) {
@@ -245,6 +263,16 @@ IntPoint CameraNode::getMediaSize()
     return m_pCamera->getImgSize();
 }
 
+BitmapPtr CameraNode::getBitmap()
+{
+    if (m_pCurBmp) {
+        return m_pCurBmp;
+    } else {
+        throw Exception(AVG_ERR_CAMERA_NONFATAL, 
+                "CameraNode.getBitmap: No camera image available.");
+    }
+}
+
 void CameraNode::dumpCameras()
 {
     avg::dumpCameras();
@@ -262,19 +290,24 @@ double CameraNode::getFPS() const
 
 void CameraNode::open()
 {
+    m_pCamera->startCapture();
     setViewport(-32767, -32767, -32767, -32767);
     PixelFormat pf = getPixelFormat();
-    getSurface()->create(getMediaSize(), pf);
-    
+    IntPoint size = getMediaSize();
+    bool bMipmap = getMaterial().getUseMipmaps();
+    m_pTex = GLTexturePtr(new GLTexture(size, pf, bMipmap));
+    m_pTex->enableStreaming();
+    getSurface()->create(pf, m_pTex);
+
+    BitmapPtr pBmp = m_pTex->lockStreamingBmp();
     if (pf == B8G8R8X8 || pf == B8G8R8A8) {
         FilterFill<Pixel32> Filter(Pixel32(0,0,0,255));
-        Filter.applyInPlace(getSurface()->lockBmp());
-        getSurface()->unlockBmps();
+        Filter.applyInPlace(pBmp);
     } else if (pf == I8) {
         FilterFill<Pixel8> Filter(0);
-        Filter.applyInPlace(getSurface()->lockBmp());
-        getSurface()->unlockBmps();
-    }
+        Filter.applyInPlace(pBmp);
+    } 
+    m_pTex->unlockStreamingBmp(true);
 }
 
 int CameraNode::getFeature(CameraFeature feature) const
@@ -293,38 +326,38 @@ int CameraNode::getFrameNum() const
 }
 
 static ProfilingZoneID CameraFetchImage("Camera fetch image");
+static ProfilingZoneID CameraDownloadProfilingZone("Camera tex download");
 
 void CameraNode::preRender()
 {
-    VisibleNode::preRender();
-    ScopeTimer Timer(CameraFetchImage);
+    Node::preRender();
     if (isAutoUpdateCameraImage()) {
+        ScopeTimer Timer(CameraFetchImage);
         updateToLatestCameraImage();
     }
+    if (m_bNewBmp && isVisible()) {
+        ScopeTimer Timer(CameraDownloadProfilingZone);
+        m_FrameNum++;
+        BitmapPtr pBmp = m_pTex->lockStreamingBmp();
+        if (pBmp->getPixelFormat() != m_pCurBmp->getPixelFormat()) {
+            cerr << "Surface: " << pBmp->getPixelFormat() << ", CamDest: "
+                << m_pCurBmp->getPixelFormat() << endl;
+        }
+        AVG_ASSERT(pBmp->getPixelFormat() == m_pCurBmp->getPixelFormat());
+        pBmp->copyPixels(*m_pCurBmp);
+        m_pTex->unlockStreamingBmp(true);
+        bind();
+        renderFX(getSize(), Pixel32(255, 255, 255, 255), false);
+        m_bNewBmp = false;
+    }
 }
 
 static ProfilingZoneID CameraProfilingZone("Camera::render");
-static ProfilingZoneID CameraDownloadProfilingZone("Camera tex download");
 
 void CameraNode::render(const DRect& rect)
 {
     if (m_bIsPlaying) {
         ScopeTimer Timer(CameraProfilingZone);
-        if (m_pCurBmp) {
-            m_FrameNum++;
-            BitmapPtr pBmp = getSurface()->lockBmp();
-            if (pBmp->getPixelFormat() != m_pCurBmp->getPixelFormat()) {
-                cerr << "Surface: " << pBmp->getPixelFormat() << ", CamDest: "
-                    << m_pCurBmp->getPixelFormat() << endl;
-            }
-            AVG_ASSERT(pBmp->getPixelFormat() == m_pCurBmp->getPixelFormat());
-            pBmp->copyPixels(*m_pCurBmp);
-            getSurface()->unlockBmps();
-            {
-                ScopeTimer Timer(CameraDownloadProfilingZone);
-                bind();
-            }
-        }
         blt32(getSize(), getEffectiveOpacity(), getBlendMode());
     }
 }
@@ -336,12 +369,11 @@ PixelFormat CameraNode::getPixelFormat()
 
 void CameraNode::updateToLatestCameraImage()
 {
-    m_pCurBmp = m_pCamera->getImage(false);
-    if (m_pCurBmp) {
-       BitmapPtr pTempBmp;
-       while (pTempBmp = m_pCamera->getImage(false)) {
-           m_pCurBmp = pTempBmp;
-       }
+    BitmapPtr pTmpBmp = m_pCamera->getImage(false);
+    while (pTmpBmp) {
+        m_bNewBmp = true;
+        m_pCurBmp = pTmpBmp;
+        pTmpBmp = m_pCamera->getImage(false);
     }
 }
 
