@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -36,6 +36,9 @@
 
 namespace avg {
 
+class TextureMover;
+typedef boost::shared_ptr<TextureMover> TextureMoverPtr;
+
 class AVG_API CameraNode : public RasterNode
 {
     public:
@@ -44,8 +47,9 @@ class AVG_API CameraNode : public RasterNode
         CameraNode(const ArgList& args);
         virtual ~CameraNode();
 
-        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine,
-                AudioEngine * pAudioEngine);
+        virtual void connectDisplay();
+        virtual void connect(CanvasPtr pCanvas);
+        virtual void disconnect(bool bKill);
                 
         void play();
         void stop();
@@ -95,6 +99,7 @@ class AVG_API CameraNode : public RasterNode
 
         int getFrameNum() const;
         IntPoint getMediaSize();
+        virtual BitmapPtr getBitmap();
 
         static void dumpCameras();
         static void resetFirewireBus();
@@ -115,8 +120,10 @@ class AVG_API CameraNode : public RasterNode
         CameraPtr m_pCamera;
         int m_FrameNum;
         BitmapPtr m_pCurBmp;
-        BitmapPtr m_pTmpBmp;
         bool m_bIsAutoUpdateCameraImage;
+        bool m_bNewBmp;
+
+        GLTexturePtr m_pTex;
 };
 
 }
