@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -39,7 +39,7 @@ namespace avg {
 
 NodeDefinition CurveNode::createDefinition()
 {
-    return NodeDefinition("curve", VisibleNode::buildNode<CurveNode>)
+    return NodeDefinition("curve", Node::buildNode<CurveNode>)
         .extendDefinition(VectorNode::createDefinition())
         .addArg(Arg<DPoint>("pos1", DPoint(0,0), false, offsetof(CurveNode, m_P1)))
         .addArg(Arg<DPoint>("pos2", DPoint(0,0), false, offsetof(CurveNode, m_P2)))
