@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,6 @@
 #include "Image.h"
 #include "DivNode.h"
 
-#include "../player/SDLDisplayEngine.h"
 #include "../base/ScopeTimer.h"
 #include "../base/Logger.h"
 #include "../base/Exception.h"
@@ -66,12 +65,11 @@ FilledVectorNode::~FilledVectorNode()
 {
 }
 
-void FilledVectorNode::setRenderingEngines(DisplayEngine * pDisplayEngine, 
-        AudioEngine * pAudioEngine)
+void FilledVectorNode::connectDisplay()
 {
-    VectorNode::setRenderingEngines(pDisplayEngine, pAudioEngine);
+    VectorNode::connectDisplay();
     m_FillColor = colorStringToColor(m_sFillColorName);
-    m_pFillShape->moveToGPU(getDisplayEngine());
+    m_pFillShape->moveToGPU();
     m_OldOpacity = -1;
 }
 
@@ -87,9 +85,9 @@ void FilledVectorNode::disconnect(bool bKill)
 
 void FilledVectorNode::checkReload()
 {
-    VisibleNode::checkReload(m_FillTexHRef, m_pFillShape->getImage());
-    if (getState() == VisibleNode::NS_CANRENDER) {
-        m_pFillShape->moveToGPU(getDisplayEngine());
+    Node::checkReload(m_FillTexHRef, m_pFillShape->getImage());
+    if (getState() == Node::NS_CANRENDER) {
+        m_pFillShape->moveToGPU();
         setDrawNeeded();
     }
     VectorNode::checkReload();
@@ -107,7 +105,7 @@ void FilledVectorNode::setFillTexHRef(const UTF8String& href)
     setDrawNeeded();
 }
 
-void FilledVectorNode::setFillBitmap(const Bitmap * pBmp)
+void FilledVectorNode::setFillBitmap(BitmapPtr pBmp)
 {
     m_FillTexHRef = "";
     m_pFillShape->setBitmap(pBmp);
@@ -149,8 +147,8 @@ void FilledVectorNode::setFillOpacity(double opacity)
 
 void FilledVectorNode::preRender()
 {
-    VisibleNode::preRender();
-    double curOpacity = getDivParent()->getEffectiveOpacity()*m_FillOpacity;
+    Node::preRender();
+    double curOpacity = getParent()->getEffectiveOpacity()*m_FillOpacity;
     VertexArrayPtr pFillVA;
     pFillVA = m_pFillShape->getVertexArray();
     if (isDrawNeeded() || curOpacity != m_OldOpacity) {
@@ -164,31 +162,12 @@ void FilledVectorNode::preRender()
     VectorNode::preRender();
 }
 
-void FilledVectorNode::maybeRender(const DRect& Rect)
-{
-    AVG_ASSERT(getState() == NS_CANRENDER);
-    if (getActive()) {
-        if (getEffectiveOpacity() > 0.01 || 
-                getDivParent()->getEffectiveOpacity()*m_FillOpacity > 0.01)
-        {
-            if (getID() != "") {
-                AVG_TRACE(Logger::BLTS, "Rendering " << getTypeStr() << 
-                        " with ID " << getID());
-            } else {
-                AVG_TRACE(Logger::BLTS, "Rendering " << getTypeStr()); 
-            }
-            getDisplayEngine()->setBlendMode(getBlendMode());
-            render(Rect);
-        }
-    }
-}
-
 static ProfilingZoneID RenderProfilingZone("FilledVectorNode::render");
 
 void FilledVectorNode::render(const DRect& rect)
 {
     ScopeTimer Timer(RenderProfilingZone);
-    double curOpacity = getDivParent()->getEffectiveOpacity()*m_FillOpacity;
+    double curOpacity = getParent()->getEffectiveOpacity()*m_FillOpacity;
     if (curOpacity > 0.01) {
         glColor4d(1.0, 1.0, 1.0, curOpacity);
         m_pFillShape->draw();
@@ -226,4 +205,10 @@ DPoint FilledVectorNode::calcFillTexCoord(const DPoint& pt, const DPoint& minPt,
     return texPt;
 }
 
+bool FilledVectorNode::isVisible() const
+{
+    return getActive() && (getEffectiveOpacity() > 0.01 || 
+            getParent()->getEffectiveOpacity()*m_FillOpacity > 0.01);
+}
+
 }
