@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -36,14 +36,13 @@ class AVG_API FilledVectorNode : public VectorNode
         
         FilledVectorNode(const ArgList& args);
         virtual ~FilledVectorNode();
-        virtual void setRenderingEngines(DisplayEngine * pDisplayEngine, 
-                AudioEngine * pAudioEngine);
+        virtual void connectDisplay();
         virtual void disconnect(bool bKill);
         virtual void checkReload();
 
         const UTF8String& getFillTexHRef() const;
         void setFillTexHRef(const UTF8String& href);
-        void setFillBitmap(const Bitmap * pBmp);
+        void setFillBitmap(BitmapPtr pBmp);
 
         const DPoint& getFillTexCoord1() const;
         void setFillTexCoord1(const DPoint& pt);
@@ -57,7 +56,6 @@ class AVG_API FilledVectorNode : public VectorNode
         void setFillOpacity(double opacity);
 
         virtual void preRender();
-        virtual void maybeRender(const DRect& Rect);
         virtual void render(const DRect& rect);
 
         virtual void calcFillVertexes(VertexArrayPtr& pVertexArray, Pixel32 color) = 0;
@@ -66,6 +64,7 @@ class AVG_API FilledVectorNode : public VectorNode
         Pixel32 getFillColorVal() const;
         DPoint calcFillTexCoord(const DPoint& pt, const DPoint& minPt, 
                 const DPoint& maxPt);
+        virtual bool isVisible() const;
 
     private:
         double m_OldOpacity;
