@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,8 @@
 
 #include "../api.h"
 
+#include "MaterialInfo.h"
+
 #include "../base/Point.h"
 #include "../graphics/Bitmap.h"
 
@@ -33,7 +35,6 @@
 namespace avg {
 
 class OGLSurface;
-class SDLDisplayEngine;
 class OffscreenCanvas;
 typedef boost::shared_ptr<OffscreenCanvas> OffscreenCanvasPtr;
 
@@ -47,17 +48,17 @@ class AVG_API Image
             TEXTURECOMPRESSION_B5G6R5
         };
 
-        Image(OGLSurface * pSurface);
+        Image(OGLSurface * pSurface, const MaterialInfo& material);
         virtual ~Image();
 
-        virtual void moveToGPU(SDLDisplayEngine* pEngine);
+        virtual void moveToGPU();
         virtual void moveToCPU();
 
         void discard();
         void setEmpty();
         void setFilename(const std::string& sFilename,
                 TextureCompression comp = TEXTURECOMPRESSION_NONE);
-        void setBitmap(const Bitmap * pBmp, 
+        void setBitmap(BitmapPtr pBmp, 
                 TextureCompression comp = TEXTURECOMPRESSION_NONE);
         void setCanvas(OffscreenCanvasPtr pCanvas);
         OffscreenCanvasPtr getCanvas() const;
@@ -69,7 +70,6 @@ class AVG_API Image
         OGLSurface* getSurface();
         State getState();
         Source getSource();
-        SDLDisplayEngine* getEngine();
 
         static TextureCompression string2compression(const std::string& s);
         static std::string compression2String(TextureCompression compression);
@@ -83,11 +83,11 @@ class AVG_API Image
         std::string m_sFilename;
         BitmapPtr m_pBmp;
         OGLSurface * m_pSurface;
-        SDLDisplayEngine * m_pEngine;
         OffscreenCanvasPtr m_pCanvas;
 
         State m_State;
         Source m_Source;
+        MaterialInfo m_Material;
 };
 
 typedef boost::shared_ptr<Image> ImagePtr;
