@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -27,18 +27,23 @@
 
 namespace avg {
 
+class SDLDisplayEngine;
+typedef boost::shared_ptr<SDLDisplayEngine> SDLDisplayEnginePtr;
+
 class AVG_API MainCanvas: public Canvas
 {
     public:
         MainCanvas(Player * pPlayer);
         virtual ~MainCanvas();
         virtual void setRoot(NodePtr pRootNode);
-        void initPlayback(SDLDisplayEngine* pDisplayEngine, AudioEngine* pAudioEngine);
+        void initPlayback(const SDLDisplayEnginePtr& pDisplayEngine);
         
         virtual BitmapPtr screenshot() const;
 
     private:
         virtual void render();
+
+        SDLDisplayEnginePtr m_pDisplayEngine;
 };
 
 }
