@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,6 @@
 #include "../api.h"
 #include "Event.h"
 #include "CursorEvent.h"
-#include "VisibleNode.h"
 
 namespace avg {
 
@@ -33,15 +32,15 @@ class AVG_API MouseEvent : public CursorEvent {
     public:
         MouseEvent(Event::Type eventType,
                 bool leftButtonState, bool middleButtonState, bool rightButtonState,
-                const IntPoint& pos, int button, const DPoint& speed);
+                const IntPoint& pos, int button, const DPoint& speed=DPoint(0,0),
+                int when=-1);
         virtual ~MouseEvent();
         
-        const DPoint& getSpeed() const;
-
         //REFACTORME: get*ButtonState -> getButtonState(num=*)
         bool getLeftButtonState() const;
         bool getMiddleButtonState() const;
         bool getRightButtonState() const;
+        bool isAnyButtonPressed() const;
         int getButton() const;
         virtual CursorEventPtr cloneAs(Type EventType) const;
         virtual void trace();
@@ -57,8 +56,6 @@ class AVG_API MouseEvent : public CursorEvent {
         bool m_MiddleButtonState;
         bool m_RightButtonState;
         int m_Button;
-
-        DPoint m_Speed;
 };
 
 typedef boost::shared_ptr<class MouseEvent> MouseEventPtr;
