@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -56,7 +56,7 @@ NodePtr NodeRegistry::createNode(const string& sType, const xmlNodePtr xmlNode)
     ArgList args(def.getDefaultArgs(), xmlNode);
     NodeBuilder builder = def.getBuilder();
     NodePtr pNode = builder(args);
-    pNode->setThis(pNode, &def);
+    pNode->setTypeInfo(&def);
     return pNode;
 }
 
@@ -66,7 +66,7 @@ NodePtr NodeRegistry::createNode(const string& sType, const boost::python::dict&
     ArgList args(def.getDefaultArgs(), pyDict);
     NodeBuilder builder = def.getBuilder();
     NodePtr pNode = builder(args);
-    pNode->setThis(pNode, &def);
+    pNode->setTypeInfo(&def);
     return pNode;
 }
 
