@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -73,7 +73,7 @@ string PluginManager::getSearchPath() const
     return m_sCurrentSearchPath;
 }
     
-void PluginManager::loadPlugin(const std::string& sPluginName)
+boost::python::object PluginManager::loadPlugin(const std::string& sPluginName)
 {
     // is it loaded aready?
     PluginMap::iterator i = m_LoadedPlugins.find(sPluginName);
@@ -89,6 +89,8 @@ void PluginManager::loadPlugin(const std::string& sPluginName)
         ++referenceCount;
         m_LoadedPlugins[sPluginName] = make_pair(i->second.first, referenceCount);
     }
+    boost::python::object sysModule(boost::python::handle<>(PyImport_ImportModule("sys")));
+    return sysModule.attr("modules")[sPluginName];
 }
 
 string PluginManager::locateSharedObject(const string& sFilename)
