@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,9 @@
 #define _PluginManager_H_
 
 #include "../api.h"
+
+#include "BoostPython.h"
+
 #include "../base/Exception.h"
 
 #include <map>
@@ -49,7 +52,7 @@ public:
     void setSearchPath(const std::string& aNewPath);
     std::string getSearchPath() const;
     
-    void loadPlugin(const std::string& aPluginName);
+    boost::python::object loadPlugin(const std::string& aPluginName);
 
 private:
     PluginManager();    
