@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,15 +22,16 @@
 #include "RasterNode.h"
 
 #include "NodeDefinition.h"
-#include "SDLDisplayEngine.h"
 #include "OGLSurface.h"
+#include "FXNode.h"
+
+#include "../graphics/ImagingProjection.h"
 
 #include "../base/MathHelper.h"
 #include "../base/Logger.h"
 #include "../base/XMLHelper.h"
 #include "../base/Exception.h"
-
-#include <Magick++.h>
+#include "../base/ScopeTimer.h"
 
 using namespace std;
 
@@ -65,7 +66,8 @@ RasterNode::RasterNode()
       m_Material(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE, false),
       m_bBound(false),
       m_TileSize(-1,-1),
-      m_pVertexes(0)
+      m_pVertexes(0),
+      m_bFXDirty(true)
 {
 }
 
@@ -86,31 +88,29 @@ void RasterNode::setArgs(const ArgList& args)
         throw Exception(AVG_ERR_OUT_OF_RANGE, 
                 "maxtilewidth and maxtileheight must be powers of two.");
     }
-    m_Material.setUseMipmaps(args.getArgVal<bool>("mipmap"));
+    bool bMipmap = args.getArgVal<bool>("mipmap");
+    m_Material = MaterialInfo(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE, bMipmap);
+    m_pSurface = new OGLSurface();
 }
 
-void RasterNode::setRenderingEngines(DisplayEngine* pDisplayEngine, 
-        AudioEngine* pAudioEngine)
+void RasterNode::connectDisplay()
 {
-    AreaNode::setRenderingEngines(pDisplayEngine, pAudioEngine);
+    checkMaskSupport(m_sMaskHref);
+    AreaNode::connectDisplay();
 
-    getSurface();
-    m_pSurface->attach(dynamic_cast<SDLDisplayEngine*>(pDisplayEngine));
+    m_pSurface->attach();
     m_bBound = false;
     if (m_MaxTileSize != IntPoint(-1, -1)) {
         m_TileSize = m_MaxTileSize;
     }
     calcVertexGrid(m_TileVertices);
-    m_pSurface->setMaterial(m_Material);
-    m_pSurface->downloadTexture();
     setBlendModeStr(m_sBlendMode);
-    if (m_Material.getHasMask()) {
-        m_pSurface->createMask(m_pMaskBmp->getSize());
+    if (m_pMaskBmp) {
         downloadMask();
         setMaskCoords();
     }
     m_pSurface->setColorParams(m_Gamma, m_Intensity, m_Contrast);
-    setupFX();
+    setupFX(true);
 }
 
 void RasterNode::disconnect(bool bKill)
@@ -123,6 +123,14 @@ void RasterNode::disconnect(bool bKill)
         m_pSurface->destroy();
     }
     m_pFBO = FBOPtr();
+    m_pImagingProjection = ImagingProjectionPtr();
+    if (bKill) {
+        m_pFXNode = FXNodePtr();
+    } else {
+        if (m_pFXNode) {
+            m_pFXNode->disconnect();
+        }
+    }
     AreaNode::disconnect(bKill);
 }
 
@@ -137,23 +145,29 @@ void RasterNode::checkReload()
             if (m_sMaskFilename != "") {
                 AVG_TRACE(Logger::MEMORY, "Loading " << m_sMaskFilename);
                 m_pMaskBmp = BitmapPtr(new Bitmap(m_sMaskFilename));
+                if (m_pMaskBmp->getPixelFormat() != I8) {
+                    BitmapPtr pTempBmp = m_pMaskBmp;
+                    m_pMaskBmp = BitmapPtr(new Bitmap(m_pMaskBmp->getSize(), I8));
+                    m_pMaskBmp->copyPixels(*pTempBmp);
+                }
                 setMaskCoords();
             }
-        } catch (Magick::Exception & ex) {
+        } catch (Exception & ex) {
+            if (ex.getCode() == AVG_ERR_VIDEO_GENERAL) {
+                throw;
+            }
             m_sMaskFilename = "";
-            if (getState() != VisibleNode::NS_UNCONNECTED) {
-                AVG_TRACE(Logger::ERROR, ex.what());
+            if (getState() != Node::NS_UNCONNECTED) {
+                AVG_TRACE(Logger::ERROR, ex.getStr());
             } else {
-                AVG_TRACE(Logger::MEMORY, ex.what());
+                AVG_TRACE(Logger::MEMORY, ex.getStr());
             }
         }
         if (m_sMaskFilename == "") {
             m_pMaskBmp = BitmapPtr();
-            m_Material.setMask(false);
-            setMaterial(m_Material);
+            getSurface()->setMask(GLTexturePtr());
         }
-        if (getState() == VisibleNode::NS_CANRENDER && m_Material.getHasMask()) {
-            m_pSurface->createMask(m_pMaskBmp->getSize());
+        if (getState() == Node::NS_CANRENDER && m_pMaskBmp) {
             downloadMask();
         }
     } else {
@@ -228,7 +242,7 @@ const std::string& RasterNode::getBlendModeStr() const
 void RasterNode::setBlendModeStr(const string& sBlendMode)
 {
     m_sBlendMode = sBlendMode;
-    m_BlendMode = DisplayEngine::stringToBlendMode(sBlendMode);
+    m_BlendMode = GLContext::stringToBlendMode(sBlendMode);
 }
 
 const UTF8String& RasterNode::getMaskHRef() const
@@ -238,6 +252,9 @@ const UTF8String& RasterNode::getMaskHRef() const
 
 void RasterNode::setMaskHRef(const UTF8String& sHref)
 {
+    if (GLContext::getCurrent()) {
+        checkMaskSupport(sHref);
+    }
     m_sMaskHref = sHref;
     checkReload();
 }
@@ -264,8 +281,7 @@ void RasterNode::setMaskSize(const DPoint& size)
     setMaskCoords();
 }
 
-void RasterNode::getElementsByPos(const DPoint& pos, 
-                vector<VisibleNodeWeakPtr>& pElements)
+void RasterNode::getElementsByPos(const DPoint& pos, vector<NodeWeakPtr>& pElements)
 {
     // Node isn't pickable if it's warped.
     if (m_MaxTileSize == IntPoint(-1, -1)) {
@@ -281,7 +297,7 @@ DTriple RasterNode::getGamma() const
 void RasterNode::setGamma(const DTriple& gamma)
 {
     m_Gamma = gamma;
-    if (m_pSurface) {
+    if (getState() == Node::NS_CANRENDER) {
         m_pSurface->setColorParams(m_Gamma, m_Intensity, m_Contrast);
     }
 }
@@ -294,7 +310,7 @@ DTriple RasterNode::getIntensity() const
 void RasterNode::setIntensity(const DTriple& intensity)
 {
     m_Intensity = intensity;
-    if (m_pSurface) {
+    if (getState() == Node::NS_CANRENDER) {
         m_pSurface->setColorParams(m_Gamma, m_Intensity, m_Contrast);
     }
 }
@@ -307,7 +323,7 @@ DTriple RasterNode::getContrast() const
 void RasterNode::setContrast(const DTriple& contrast)
 {
     m_Contrast = contrast;
-    if (m_pSurface) {
+    if (getState() == Node::NS_CANRENDER) {
         m_pSurface->setColorParams(m_Gamma, m_Intensity, m_Contrast);
     }
 }
@@ -321,40 +337,30 @@ void RasterNode::setEffect(FXNodePtr pFXNode)
         m_pFBO = FBOPtr();
     }
     m_pFXNode = pFXNode;
-    setupFX();
-}
-
-BitmapPtr RasterNode::getBitmap()
-{
-    if (m_pSurface) {
-        return m_pSurface->readbackBmp();
-    } else {
-        return BitmapPtr(); 
+    if (getState() == NS_CANRENDER) {
+        setupFX(true);
     }
 }
 
 void RasterNode::blt32(const DPoint& destSize, double opacity, 
-        DisplayEngine::BlendMode mode, bool bPremultipliedAlpha)
+        GLContext::BlendMode mode, bool bPremultipliedAlpha)
 {
     blt(destSize, mode, opacity, Pixel32(255, 255, 255, 255), bPremultipliedAlpha);
 }
 
 void RasterNode::blta8(const DPoint& destSize, double opacity, 
-        const Pixel32& color, DisplayEngine::BlendMode mode)
+        const Pixel32& color, GLContext::BlendMode mode)
 {
     blt(destSize, mode, opacity, color, false);
 }
 
-DisplayEngine::BlendMode RasterNode::getBlendMode() const
+GLContext::BlendMode RasterNode::getBlendMode() const
 {
     return m_BlendMode;
 }
 
 OGLSurface * RasterNode::getSurface()
 {
-    if (!m_pSurface) {
-        m_pSurface = new OGLSurface(m_Material);
-    }
     return m_pSurface;
 }
 
@@ -371,13 +377,11 @@ bool RasterNode::hasMask() const
 void RasterNode::setMaskCoords()
 {
     if (m_sMaskFilename != "") {
-        m_Material.setMask(true);
-        calcMaskCoords(m_Material);
-        setMaterial(m_Material);
+        calcMaskCoords();
     }
 }
 
-void RasterNode::calcMaskCoords(MaterialInfo& material)
+void RasterNode::calcMaskCoords()
 {
     DPoint maskSize;
     DPoint mediaSize = DPoint(getMediaSize());
@@ -387,7 +391,23 @@ void RasterNode::calcMaskCoords(MaterialInfo& material)
         maskSize = DPoint(m_MaskSize.x/mediaSize.x, m_MaskSize.y/mediaSize.y);
     }
     DPoint maskPos = DPoint(m_MaskPos.x/mediaSize.x, m_MaskPos.y/mediaSize.y);
-    material.setMaskCoords(maskPos, maskSize);
+    m_pSurface->setMaskCoords(maskPos, maskSize);
+}
+
+void RasterNode::checkMaskSupport(const string& sHref)
+{
+    if (!(GLContext::getCurrent()->isUsingShaders()) && sHref != "") {
+        throw Exception(AVG_ERR_UNSUPPORTED,
+                "Can't use masks - unsupported on this hardware/driver combination.");
+    }
+}
+
+void RasterNode::downloadMask()
+{
+    GLTexturePtr pTex(new GLTexture(m_pMaskBmp->getSize(), I8, 
+            m_Material.getUseMipmaps()));
+    pTex->moveBmpToTexture(m_pMaskBmp);
+    m_pSurface->setMask(pTex);
 }
 
 void RasterNode::bind() 
@@ -395,27 +415,68 @@ void RasterNode::bind()
     if (!m_bBound) {
         calcTexCoords();
     }
-    m_pSurface->downloadTexture();
     m_bBound = true;
 }
 
-void RasterNode::setMaterial(const MaterialInfo& material)
-{
-    m_Material = material;
-    getSurface()->setMaterial(m_Material);
-}
+static ProfilingZoneID FXProfilingZone("RasterNode::renderFX");
 
-void RasterNode::downloadMask()
+void RasterNode::renderFX(const DPoint& destSize, const Pixel32& color, 
+        bool bPremultipliedAlpha)
 {
-    BitmapPtr pBmp = m_pSurface->lockMaskBmp();
-    pBmp->copyPixels(*m_pMaskBmp);
-    m_pSurface->unlockMaskBmp();
-    m_pSurface->downloadMaskTexture();
+    ScopeTimer Timer(FXProfilingZone);
+    setupFX(false);
+    GLContext* pContext = GLContext::getCurrent();
+    pContext->enableGLColorArray(false);
+    pContext->enableTexture(true);
+    if (m_pFXNode && (m_bFXDirty || m_pSurface->isDirty() || m_pFXNode->isDirty())) {
+        if (!m_bBound) {
+            bind();
+        }
+        m_pSurface->activate(getMediaSize());
+
+        m_pFBO->activate();
+        clearGLBuffers(GL_COLOR_BUFFER_BIT);
+
+        glColor4d(double(color.getR())/256, double(color.getG())/256, 
+                double(color.getB())/256, 1);
+        if (bPremultipliedAlpha) {
+            glproc::BlendColor(1.0f, 1.0f, 1.0f, 1.0f);
+        }
+        pContext->setBlendMode(GLContext::BLEND_BLEND, bPremultipliedAlpha);
+
+        glMatrixMode(GL_MODELVIEW);
+        glPushMatrix();
+
+        m_pImagingProjection->activate();
+        m_pImagingProjection->draw();
+
+/*
+        static int i=0;
+        stringstream ss;
+        ss << "foo" << i << ".png";
+        BitmapPtr pBmp = m_pFBO->getImage(0);
+        pBmp->save(ss.str());
+  */  
+        m_pFXNode->apply(m_pFBO->getTex());
+        
+        glPopMatrix();
+        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "RasterNode::renderFX(): glPopMatrix");
+/*        
+        stringstream ss1;
+        ss1 << "bar" << ".png";
+        i++;
+        m_pFXNode->getImage()->save(ss1.str());
+*/
+        glproc::UseProgramObject(0);
+        m_bFXDirty = false;
+        m_pSurface->resetDirty();
+        m_pFXNode->resetDirty();
+    }
 }
 
 void RasterNode::checkDisplayAvailable(std::string sMsg)
 {
-    if (!(getState() == VisibleNode::NS_CANRENDER)) {
+    if (!(getState() == Node::NS_CANRENDER)) {
         throw Exception(AVG_ERR_UNSUPPORTED,
             string(sMsg) + ": cannot access vertex coordinates before node is bound.");
     }
@@ -425,103 +486,58 @@ void RasterNode::checkDisplayAvailable(std::string sMsg)
     }
 }
 
-void RasterNode::setupFX()
+void RasterNode::setupFX(bool bNewFX)
 {
     if (m_pSurface && m_pSurface->getSize() != IntPoint(-1, -1) && m_pFXNode) {
-        if (!getDisplayEngine()->isUsingShaders()) {
+        if (!GLContext::getCurrent()->isUsingShaders()) {
             throw Exception(AVG_ERR_UNSUPPORTED,
                     "Can't use FX - unsupported on this hardware/driver combination.");
         }
+        if (bNewFX || !m_pFBO || m_pFBO->getSize() != m_pSurface->getSize()) {
+            m_pFXNode->setSize(m_pSurface->getSize());
+            m_pFXNode->connect();
+            m_bFXDirty = true;
+        }
         if (!m_pFBO || m_pFBO->getSize() != m_pSurface->getSize()) {
             m_pFBO = FBOPtr(new FBO(IntPoint(m_pSurface->getSize()), B8G8R8A8, 1, 1,
                     false, getMipmap()));
-            m_pFXNode->setSize(m_pSurface->getSize());
-            m_pFXNode->connect(getDisplayEngine());
+            GLTexturePtr pTex = m_pFBO->getTex();
+            pTex->setWrapMode(GL_CLAMP_TO_BORDER, GL_CLAMP_TO_BORDER);
+            m_pImagingProjection = ImagingProjectionPtr(new ImagingProjection(
+                    m_pSurface->getSize()));
         }
     }
 }
 
-void RasterNode::blt(const DPoint& destSize, DisplayEngine::BlendMode mode,
+void RasterNode::blt(const DPoint& destSize, GLContext::BlendMode mode,
         double opacity, const Pixel32& color, bool bPremultipliedAlpha)
 {
     if (!m_bBound) {
         bind();
     }
-    setupFX();
-    getDisplayEngine()->enableGLColorArray(false);
-    getDisplayEngine()->enableTexture(true);
-    m_pSurface->activate(getMediaSize());
+    GLContext* pContext = GLContext::getCurrent();
+    pContext->enableGLColorArray(false);
+    pContext->enableTexture(true);
+    DRect destRect;
     if (m_pFXNode) {
-        m_pFBO->activate();
-        clearGLBuffers(GL_COLOR_BUFFER_BIT);
-
-        glColor4d(double(color.getR())/256, double(color.getG())/256, 
-                double(color.getB())/256, 1);
-        if (bPremultipliedAlpha) {
-            glproc::BlendColor(1.0f, 1.0f, 1.0f, 1.0f);
-        }
-        getDisplayEngine()->setBlendMode(DisplayEngine::BLEND_BLEND, 
-                bPremultipliedAlpha);
-        
-        glPushAttrib(GL_VIEWPORT_BIT | GL_ENABLE_BIT);
-        glDisable(GL_MULTISAMPLE);
-        glMatrixMode(GL_PROJECTION);
-        glPushMatrix();
-        glMatrixMode(GL_MODELVIEW);
-        glPushMatrix();
-
-        m_pFBO->setupImagingProjection();
-        m_pFBO->drawImagingVertexes();
-
-
-        m_pFBO->deactivate();
-        m_pFBO->copyToDestTexture();
-        m_pSurface->deactivate();
-/*
-        static int i=0;
-        stringstream ss;
-        ss << "foo" << i << ".png";
-        BitmapPtr pBmp = m_pFBO->getImage(0);
-        pBmp->save(ss.str());
-  */  
-        m_pFXNode->apply(m_pFBO->getTex());
-
-        glPopAttrib();
-        glMatrixMode(GL_PROJECTION);
-        glPopMatrix();
-        glMatrixMode(GL_MODELVIEW);
-        glPopMatrix();
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "RasterNode::blt(): glPopMatrix");
-
         m_pFXNode->getTex()->activate(GL_TEXTURE0);
-/*        
-        stringstream ss1;
-        ss1 << "bar" << i << ".png";
-        i++;
-        m_pFXNode->getImage()->save(ss1.str());
-*/
-        if (getDisplayEngine()->isUsingShaders()) {
-            glproc::UseProgramObject(0);
-        }
-        getDisplayEngine()->setBlendMode(mode, true);
+
+        pContext->setBlendMode(mode, true);
         glColor4d(1.0, 1.0, 1.0, opacity);
+        DRect relDestRect = m_pFXNode->getRelDestRect();
+        destRect = DRect(relDestRect.tl.x*destSize.x, relDestRect.tl.y*destSize.y,
+                relDestRect.br.x*destSize.x, relDestRect.br.y*destSize.y);
     } else {
-    //    pBmp->dump(true);
-        getDisplayEngine()->setBlendMode(mode, bPremultipliedAlpha);
+        m_pSurface->activate(getMediaSize(), bPremultipliedAlpha);
+        pContext->setBlendMode(mode, bPremultipliedAlpha);
         glColor4d(double(color.getR())/256, double(color.getG())/256, 
                 double(color.getB())/256, opacity);
+        destRect = DRect(DPoint(0,0), destSize);
     }
     glproc::BlendColor(1.0f, 1.0f, 1.0f, float(opacity));
-    FBOPtr pMainFBO = getDisplayEngine()->getMainFBO();
-    if (pMainFBO) {
-        pMainFBO->activate();
-    } else {
-        if (m_pFBO) {
-            m_pFBO->deactivate();
-        }
-    }
     glPushMatrix();
-    glScaled(destSize.x, destSize.y, 1);
+    glTranslated(destRect.tl.x, destRect.tl.y, 1);
+    glScaled(destRect.size().x, destRect.size().y, 1);
 
     if (m_bVertexArrayDirty) {
         m_pVertexes->reset();
@@ -540,7 +556,6 @@ void RasterNode::blt(const DPoint& destSize, DisplayEngine::BlendMode mode,
     }
 
     m_pVertexes->draw();
-    m_pSurface->deactivate();
 
     glPopMatrix();
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "RasterNode::blt(): glPopMatrix 2");
