@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,11 +25,9 @@
 #include "../base/Exception.h"
 
 #include "../graphics/Filterfliprgb.h"
+#include "../graphics/GLContext.h"
 
 #include "OGLSurface.h"
-#include "SDLDisplayEngine.h"
-
-#include <Magick++.h>
 
 #include <iostream>
 #include <sstream>
@@ -40,8 +38,8 @@ namespace avg {
 
 Shape::Shape(const MaterialInfo& material)
 {
-    m_pSurface = new OGLSurface(material);
-    m_pImage = ImagePtr(new Image(m_pSurface));
+    m_pSurface = new OGLSurface();
+    m_pImage = ImagePtr(new Image(m_pSurface, material));
 }
 
 Shape::~Shape()
@@ -49,7 +47,7 @@ Shape::~Shape()
     delete m_pSurface;
 }
 
-void Shape::setBitmap(const Bitmap* pBmp)
+void Shape::setBitmap(BitmapPtr pBmp)
 {
     Image::State prevState = m_pImage->getState();
     if (pBmp) {
@@ -58,22 +56,18 @@ void Shape::setBitmap(const Bitmap* pBmp)
         m_pImage->setEmpty();
     }
     if (m_pImage->getState() == Image::GPU) {
-        m_pSurface->downloadTexture();
         if (prevState != Image::GPU) {
             // TODO: This shouldn't happen.
-            m_pVertexArray = VertexArrayPtr(new VertexArray(100, 100));
+            m_pVertexArray = VertexArrayPtr(new VertexArray());
         }
     }
 }
 
-void Shape::moveToGPU(SDLDisplayEngine* pEngine)
+void Shape::moveToGPU()
 {
-    m_pSurface->attach(pEngine);
-    m_pImage->moveToGPU(pEngine);
-    if (m_pImage->getSource() != Image::NONE) {
-        m_pSurface->downloadTexture();
-    }
-    m_pVertexArray = VertexArrayPtr(new VertexArray(100, 100));
+    m_pSurface->attach();
+    m_pImage->moveToGPU();
+    m_pVertexArray = VertexArrayPtr(new VertexArray());
 }
 
 void Shape::moveToCPU()
@@ -100,16 +94,22 @@ VertexArrayPtr Shape::getVertexArray()
 void Shape::draw()
 {
     bool bIsTextured = isTextured();
+    GLContext* pContext = GLContext::getCurrent();
     if (bIsTextured) {
         m_pSurface->activate();
+    } else {
+        if (GLContext::getCurrent()->isUsingShaders()) {
+            glproc::UseProgramObject(0);
+        }
+        for (int i = 1; i < 5; ++i) {
+            glproc::ActiveTexture(GL_TEXTURE0 + i);
+            glDisable(GL_TEXTURE_2D);
+        }
+        glproc::ActiveTexture(GL_TEXTURE0);
     }
-    SDLDisplayEngine* pEngine = m_pImage->getEngine();
-    pEngine->enableTexture(bIsTextured);
-    pEngine->enableGLColorArray(!bIsTextured);
+    pContext->enableTexture(bIsTextured);
+    pContext->enableGLColorArray(!bIsTextured);
     m_pVertexArray->draw();
-    if (bIsTextured) {
-        m_pSurface->deactivate();
-    }
 }
 
 void Shape::discard()
