@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -41,9 +41,9 @@ class AVG_API Shape
         Shape(const MaterialInfo& material);
         virtual ~Shape();
 
-        void setBitmap(const Bitmap * pBmp);
+        void setBitmap(BitmapPtr pBmp);
 
-        virtual void moveToGPU(SDLDisplayEngine* pEngine);
+        virtual void moveToGPU();
         virtual void moveToCPU();
 
         ImagePtr getImage();
