@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
 //
 
 #include "TrackerCalibrator.h"
-#include "TrackerEventSource.h"
+#include "TrackerInputDevice.h"
 
 #include "../base/ObjectCounter.h"
 #include "../base/Exception.h"
