@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -19,11 +19,9 @@
 //  Current versions can be found at www.libavg.de
 //
 #include "VideoNode.h"
-#include "DisplayEngine.h"
 #include "Player.h"
 #include "OGLSurface.h"
 #include "NodeDefinition.h"
-#include "SDLDisplayEngine.h"
 #include "Canvas.h"
 
 #include "../base/Exception.h"
@@ -32,8 +30,10 @@
 #include "../base/XMLHelper.h"
 
 #include "../graphics/Filterfill.h"
+#include "../graphics/GLTexture.h"
+#include "../graphics/TextureMover.h"
 
-#include "../audio/AudioEngine.h"
+#include "../audio/SDLAudioEngine.h"
 
 #include "../video/AsyncVideoDecoder.h"
 #include "../video/FFMpegDecoder.h"
@@ -52,7 +52,7 @@ namespace avg {
 
 NodeDefinition VideoNode::createDefinition()
 {
-    return NodeDefinition("video", VisibleNode::buildNode<VideoNode>)
+    return NodeDefinition("video", Node::buildNode<VideoNode>)
         .extendDefinition(RasterNode::createDefinition())
         .addArg(Arg<UTF8String>("href", "", false, offsetof(VideoNode, m_href)))
         .addArg(Arg<bool>("loop", false, false, offsetof(VideoNode, m_bLoop)))
@@ -61,6 +61,8 @@ NodeDefinition VideoNode::createDefinition()
         .addArg(Arg<int>("queuelength", 8, false, 
                 offsetof(VideoNode, m_QueueLength)))
         .addArg(Arg<double>("volume", 1.0, false, offsetof(VideoNode, m_Volume)))
+        .addArg(Arg<bool>("accelerated", false, false,
+                offsetof(VideoNode, m_bUsesHardwareAcceleration)))
         ;
 }
 
@@ -75,7 +77,8 @@ VideoNode::VideoNode(const ArgList& args)
       m_FramesPlayed(0),
       m_SeekBeforeCanRenderTime(0),
       m_pDecoder(0),
-      m_Volume(1.0)
+      m_Volume(1.0),
+      m_bUsesHardwareAcceleration(false)
 {
     args.setMembers(this);
     m_Filename = m_href;
@@ -105,11 +108,10 @@ VideoNode::~VideoNode()
     ObjectCounter::get()->decRef(&typeid(*this));
 }
 
-void VideoNode::setRenderingEngines(DisplayEngine * pDisplayEngine, 
-        AudioEngine * pAudioEngine)
+void VideoNode::connectDisplay()
 {
     checkReload();
-    RasterNode::setRenderingEngines(pDisplayEngine, pAudioEngine);
+    RasterNode::connectDisplay();
     long long CurTime = Player::get()->getFrameTime(); 
     if (m_VideoState != Unloaded) {
         startDecoding();
@@ -230,12 +232,15 @@ int VideoNode::getNumAudioChannels() const
 
 long long VideoNode::getCurTime() const
 {
-    exceptionIfUnloaded("getCurTime");
-    long long curTime = (long long)(m_pDecoder->getCurTime()*1000);
-    if (curTime > 0) {
-        return curTime;
-    } else {
+    if (m_VideoState == Unloaded) {
         return 0;
+    } else {
+        long long curTime = (long long)(m_pDecoder->getCurTime()*1000);
+        if (curTime > 0) {
+            return curTime;
+        } else {
+            return 0;
+        }
     }
 }
 
@@ -286,6 +291,12 @@ void VideoNode::setEOFCallback(PyObject * pEOFCallback)
     }
 }
 
+bool VideoNode::isAccelerated() const
+{
+    exceptionIfUnloaded("isAccelerated");
+    return m_bUsesHardwareAcceleration;
+}
+
 const UTF8String& VideoNode::getHRef() const
 {
     return m_href;
@@ -337,7 +348,7 @@ void VideoNode::onFrameEnd()
     if (m_bEOFPending) {
         // If the VideoNode is unlinked by python in onEOF, the following line prevents
         // the object from being deleted until we return from this function.
-        NodePtr pTempThis = getThis();
+        NodePtr pTempThis = shared_from_this();
         m_bEOFPending = false;
         onEOF();
     }
@@ -408,7 +419,7 @@ void VideoNode::open()
     m_FramesTooLate = 0;
     m_FramesInRowTooLate = 0;
     m_FramesPlayed = 0;
-    m_pDecoder->open(m_Filename, m_bThreaded);
+    m_pDecoder->open(m_Filename, m_bThreaded, m_bUsesHardwareAcceleration);
     m_pDecoder->setVolume(m_Volume);
     VideoInfo videoInfo = m_pDecoder->getVideoInfo();
     if (!videoInfo.m_bHasVideo) {
@@ -422,15 +433,17 @@ void VideoNode::open()
 
     m_bFirstFrameDecoded = false;
     m_bFrameAvailable = false;
+    m_bUsesHardwareAcceleration = videoInfo.m_bUsesVDPAU;
 }
 
 void VideoNode::startDecoding()
 {
     const AudioParams * pAP = 0;
-    if (getAudioEngine()) {
-        pAP = getAudioEngine()->getParams();
+    SDLAudioEngine* pAudioEngine = SDLAudioEngine::get();
+    if (pAudioEngine) {
+        pAP = pAudioEngine->getParams();
     }
-    m_pDecoder->startDecoding(getDisplayEngine()->isUsingShaders(), pAP);
+    m_pDecoder->startDecoding(GLContext::getCurrent()->isUsingShaders(), pAP);
     VideoInfo videoInfo = m_pDecoder->getVideoInfo();
     if (m_FPS != 0.0) {
         if (videoInfo.m_bHasAudio) {
@@ -440,29 +453,61 @@ void VideoNode::startDecoding()
             m_pDecoder->setFPS(m_FPS);
         }
     }
-    if (videoInfo.m_bHasAudio && getAudioEngine()) {
-        getAudioEngine()->addSource(this);
+    if (videoInfo.m_bHasAudio && pAudioEngine) {
+        pAudioEngine->addSource(this);
     }
     m_bSeekPending = true;
     
     setViewport(-32767, -32767, -32767, -32767);
-    PixelFormat pf = getPixelFormat();
-    getSurface()->create(videoInfo.m_Size, pf);
-    if (pf == B8G8R8X8 || pf == B8G8R8A8) {
-        FilterFill<Pixel32> Filter(Pixel32(0,0,0,255));
-        Filter.applyInPlace(getSurface()->lockBmp());
-        getSurface()->unlockBmps();
-    }
+    createTextures(videoInfo.m_Size);
+   
     if (m_SeekBeforeCanRenderTime != 0) {
         seek(m_SeekBeforeCanRenderTime);
         m_SeekBeforeCanRenderTime = 0;
     }
 }
 
+void VideoNode::createTextures(IntPoint size)
+{
+    PixelFormat pf = getPixelFormat();
+    bool bMipmap = getMaterial().getUseMipmaps();
+    if (pixelFormatIsPlanar(pf)) {
+        m_pTextures[0] = GLTexturePtr(new GLTexture(size, I8, bMipmap));
+        IntPoint halfSize(size.x/2, size.y/2);
+        m_pTextures[1] = GLTexturePtr(new GLTexture(halfSize, I8, bMipmap));
+        m_pTextures[2] = GLTexturePtr(new GLTexture(halfSize, I8, bMipmap));
+        if (pixelFormatHasAlpha(pf)) {
+            m_pTextures[3] = GLTexturePtr(new GLTexture(size, I8, bMipmap));
+        }
+    } else {
+        m_pTextures[0] = GLTexturePtr(new GLTexture(size, pf, bMipmap));
+    }
+    for (unsigned i=0; i<getNumPixelFormatPlanes(pf); ++i) {
+        m_pTextures[i]->enableStreaming();
+    }
+    if (pf == B8G8R8X8 || pf == B8G8R8A8) {
+        FilterFill<Pixel32> Filter(Pixel32(0,0,0,255));
+        BitmapPtr pBmp = m_pTextures[0]->lockStreamingBmp();
+        Filter.applyInPlace(pBmp);
+        m_pTextures[0]->unlockStreamingBmp(true);
+    }
+    if (pixelFormatIsPlanar(pf)) {
+        if (pixelFormatHasAlpha(pf)) {
+            getSurface()->create(pf, m_pTextures[0], m_pTextures[1], m_pTextures[2],
+                    m_pTextures[3]);
+        } else {
+            getSurface()->create(pf, m_pTextures[0], m_pTextures[1], m_pTextures[2]);
+        }
+    } else {
+        getSurface()->create(pf, m_pTextures[0]);
+    }
+}
+
 void VideoNode::close()
 {
-    if (hasAudio() && getAudioEngine()) {
-        getAudioEngine()->removeSource(this);
+    SDLAudioEngine* pAudioEngine = SDLAudioEngine::get();
+    if (hasAudio() && pAudioEngine) {
+        pAudioEngine->removeSource(this);
     }
     m_pDecoder->close();
     if (m_FramesTooLate > 0) {
@@ -485,7 +530,7 @@ PixelFormat VideoNode::getPixelFormat() const
 
 IntPoint VideoNode::getMediaSize()
 {
-    if (m_pDecoder && m_pDecoder->getState() != IVideoDecoder::CLOSED) {
+    if (m_pDecoder && m_pDecoder->getState() != VideoDecoder::CLOSED) {
         return m_pDecoder->getSize();
     } else {
         return IntPoint(0,0);
@@ -549,12 +594,30 @@ void VideoNode::exceptionIfUnloaded(const std::string& sFuncName) const
     }
 }
 
+static ProfilingZoneID PrerenderProfilingZone("VideoNode::prerender");
+
 void VideoNode::preRender()
 {
-    VisibleNode::preRender();
-    if (getEffectiveOpacity() <= 0.01) {
-        if (m_bSeekPending && m_bFirstFrameDecoded) {
-            renderFrame(getSurface());
+    ScopeTimer timer(PrerenderProfilingZone);
+    Node::preRender();
+    if (isVisible()) {
+        if (m_VideoState != Unloaded) {
+            if (m_VideoState == Playing) {
+                bool bNewFrame = renderFrame();
+                m_bFrameAvailable |= bNewFrame;
+            } else { // Paused
+                if (!m_bFrameAvailable) {
+                    m_bFrameAvailable = renderFrame();
+                }
+            }
+            m_bFirstFrameDecoded |= m_bFrameAvailable;
+            if (m_bFirstFrameDecoded) {
+                renderFX(getSize(), Pixel32(255, 255, 255, 255), false);
+            }
+        }
+    } else {
+        if (m_bSeekPending && m_bFirstFrameDecoded && m_VideoState != Unloaded) {
+            renderFrame();
         }
         if (m_VideoState == Playing) {
             // Throw away frames that are not visible to make sure the video 
@@ -572,44 +635,30 @@ static ProfilingZoneID RenderProfilingZone("VideoNode::render");
 
 void VideoNode::render(const DRect& rect)
 {
-    switch (m_VideoState) {
-        case Playing:
-            {
-                bool bNewFrame = renderFrame(getSurface());
-                m_bFrameAvailable = m_bFrameAvailable | bNewFrame;
-                if (m_bFrameAvailable) {
-                    m_bFirstFrameDecoded = true;
-                }
-                if (m_bFirstFrameDecoded) {
-                    blt32(getSize(), getEffectiveOpacity(), getBlendMode());
-                }
-            }
-            break;
-        case Paused:
-            if (!m_bFrameAvailable) {
-                m_bFrameAvailable = renderFrame(getSurface());
-            }
-            if (m_bFrameAvailable) {
-                m_bFirstFrameDecoded = true;
-            }
-            if (m_bFirstFrameDecoded) {
-                blt32(getSize(), getEffectiveOpacity(), getBlendMode());
-            }
-            break;
-        case Unloaded:
-            break;
+    ScopeTimer timer(RenderProfilingZone);
+    if (m_VideoState != Unloaded && m_bFirstFrameDecoded) {
+        blt32(getSize(), getEffectiveOpacity(), getBlendMode());
     }
 }
 
-bool VideoNode::renderFrame(OGLSurface * pSurface)
+VideoNode::VideoAccelType VideoNode::getVideoAccelConfig()
 {
-    ScopeTimer timer(RenderProfilingZone);
-    FrameAvailableCode frameAvailable = renderToSurface(pSurface);
+#ifdef AVG_ENABLE_VDPAU
+    if (VDPAU::isAvailable()) {
+        return VDPAU;
+    }
+#endif
+    return NONE;
+}
+
+bool VideoNode::renderFrame()
+{
+    FrameAvailableCode frameAvailable = renderToSurface();
     if (m_pDecoder->isEOF()) {
 //        AVG_TRACE(Logger::PROFILE, "------------------ EOF -----------------");
         updateStatusDueToDecoderEOF();
         if (m_bLoop) {
-            frameAvailable = renderToSurface(pSurface);
+            frameAvailable = renderToSurface();
         }
     }
 
@@ -665,21 +714,24 @@ bool VideoNode::renderFrame(OGLSurface * pSurface)
     }
 
     return (frameAvailable == FA_NEW_FRAME);
+    return false;
 }
 
-FrameAvailableCode VideoNode::renderToSurface(OGLSurface * pSurface)
+FrameAvailableCode VideoNode::renderToSurface()
 {
     FrameAvailableCode frameAvailable;
     PixelFormat pf = m_pDecoder->getPixelFormat();
+    std::vector<BitmapPtr> pBmps;
+    for (unsigned i=0; i<getNumPixelFormatPlanes(pf); ++i) {
+        pBmps.push_back(m_pTextures[i]->lockStreamingBmp());
+    }
     if (pixelFormatIsPlanar(pf)) {
-        std::vector<BitmapPtr> pBmps;
-        for (unsigned i = 0; i < getNumPixelFormatPlanes(pf); ++i) {
-            pBmps.push_back(pSurface->lockBmp(i));
-        }
         frameAvailable = m_pDecoder->renderToBmps(pBmps, getNextFrameTime()/1000.0);
     } else {
-        BitmapPtr pBmp = pSurface->lockBmp();
-        frameAvailable = m_pDecoder->renderToBmp(pBmp, getNextFrameTime()/1000.0);
+        frameAvailable = m_pDecoder->renderToBmp(pBmps[0], getNextFrameTime()/1000.0);
+    }
+    for (unsigned i=0; i<getNumPixelFormatPlanes(pf); ++i) {
+        m_pTextures[i]->unlockStreamingBmp(frameAvailable == FA_NEW_FRAME);
     }
 
     // Even with vsync, frame duration has a bit of jitter. If the video frames rendered
@@ -693,7 +745,6 @@ FrameAvailableCode VideoNode::renderToSurface(OGLSurface * pSurface)
             m_JitterCompensation -= 1;
         }
     }
-    pSurface->unlockBmps();
     return frameAvailable;
 }
 
