@@ -9,7 +9,7 @@ def startAnim():
 player = avg.Player.get()
 player.loadFile("text.avg")
 
-node = player.getElementByID("ClashText")
+node = player.getElementByID("HelloText")
 animObj = LinearAnim(node, "x", 2000, 0, 200)
 player.setTimeout(0, startAnim)
 
