@@ -6,7 +6,7 @@ from libavg import avg
 player = avg.Player.get()
 
 player.loadFile("text.avg")
-node = player.getElementByID("ClashText")
+node = player.getElementByID("HelloText")
 print node.x
 node.x = 200
 player.play()
