@@ -28,7 +28,7 @@ def onMouseUp(event):
 player = avg.Player.get()
 
 player.loadFile("text.avg")
-node = player.getElementByID("ClashText")
+node = player.getElementByID("HelloText")
 node.setEventHandler(avg.CURSORDOWN, avg.MOUSE, onMouseDown)
 node.setEventHandler(avg.CURSORMOTION, avg.MOUSE, onMouseMove)
 node.setEventHandler(avg.CURSORUP, avg.MOUSE, onMouseUp)
