@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2008 Ulrich von Zadow
+# Copyright (C) 2003-2011 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -28,18 +28,6 @@ import sys
 from libavg import avg
 from testcase import *
 
-def keyUp(Event):
-    print "keyUp"
-
-def keyDown(Event):
-    print "keyDown"
-    print Event
-    print "  Type: "+str(Event.type)
-    print "  keystring: "+Event.keystring
-    print "  scancode: "+str(Event.scancode)
-    print "  keycode: "+str(Event.keycode)
-    print "  modifiers: "+str(Event.modifiers)
-
 def dumpMouseEvent(Event):
     print Event
     print "  type: "+str(Event.type)
@@ -62,69 +50,6 @@ def mainMouseDown(Event):
     assert (Event.type == avg.CURSORDOWN)
     mainMouseDownCalled = True
 
-class NodeHandlerTester:
-    def __init__(self, testCase, node):
-        self.__testCase=testCase
-        self.reset()
-        self.__node = node
-        self.setHandlers()
-
-    def assertState(self, down, up, over, out, move):
-        self.__testCase.assert_(down == self.__downCalled)
-        self.__testCase.assert_(up == self.__upCalled)
-        self.__testCase.assert_(over == self.__overCalled)
-        self.__testCase.assert_(out == self.__outCalled)
-        self.__testCase.assert_(move == self.__moveCalled)
-        self.__testCase.assert_(not(self.__touchDownCalled))
-        self.reset()
-
-    def reset(self):
-        self.__upCalled=False
-        self.__downCalled=False
-        self.__overCalled=False
-        self.__outCalled=False
-        self.__moveCalled=False
-        self.__touchDownCalled=False
-
-    def setHandlers(self):
-        self.__node.setEventHandler(avg.CURSORDOWN, avg.MOUSE, self.__onDown) 
-        self.__node.setEventHandler(avg.CURSORUP, avg.MOUSE, self.__onUp) 
-        self.__node.setEventHandler(avg.CURSOROVER, avg.MOUSE, self.__onOver) 
-        self.__node.setEventHandler(avg.CURSOROUT, avg.MOUSE, self.__onOut) 
-        self.__node.setEventHandler(avg.CURSORMOTION, avg.MOUSE, self.__onMove) 
-        self.__node.setEventHandler(avg.CURSORDOWN, avg.TOUCH, self.__onTouchDown) 
-
-    def clearHandlers(self):
-        self.__node.setEventHandler(avg.CURSORDOWN, avg.MOUSE, None) 
-        self.__node.setEventHandler(avg.CURSORUP, avg.MOUSE, None) 
-        self.__node.setEventHandler(avg.CURSOROVER, avg.MOUSE, None) 
-        self.__node.setEventHandler(avg.CURSOROUT, avg.MOUSE, None) 
-        self.__node.setEventHandler(avg.CURSORMOTION, avg.MOUSE, None) 
-        self.__node.setEventHandler(avg.CURSORDOWN, avg.TOUCH, None) 
-
-    def __onDown(self, Event):
-        self.__testCase.assert_(Event.type == avg.CURSORDOWN)
-        self.__downCalled = True
-    
-    def __onUp(self, Event):
-        self.__testCase.assert_(Event.type == avg.CURSORUP)
-        self.__upCalled = True
-
-    def __onOver(self, Event):
-        self.__testCase.assert_(Event.type == avg.CURSOROVER)
-        self.__overCalled = True
-    
-    def __onOut(self, Event):
-        self.__testCase.assert_(Event.type == avg.CURSOROUT)
-        self.__outCalled = True
-    
-    def __onMove(self, Event):
-        self.__testCase.assert_(Event.type == avg.CURSORMOTION)
-        self.__moveCalled = True
-    
-    def __onTouchDown(self, Event):
-        self.__touchDownCalled = True
-        
 
 class EventTestCase(AVGTestCase):
     def __init__(self, testFuncName):
@@ -139,60 +64,31 @@ class EventTestCase(AVGTestCase):
             if Event.keystring == 'A' and Event.keycode == 65 and Event.unicode == 65:
                 self.keyUpCalled = True
         
-        self.loadEmptyScene()
-        Player.getRootNode().setEventHandler(avg.KEYDOWN, avg.NONE, onKeyDown)
-        Player.getRootNode().setEventHandler(avg.KEYUP, avg.NONE, onKeyUp)
-        self.start(None, 
-                (lambda: Helper.fakeKeyEvent(avg.KEYDOWN, 65, 65, "A", 65, 
+        root = self.loadEmptyScene()
+        root.setEventHandler(avg.KEYDOWN, avg.NONE, onKeyDown)
+        root.setEventHandler(avg.KEYUP, avg.NONE, onKeyUp)
+        self.start(( 
+                 lambda: Helper.fakeKeyEvent(avg.KEYDOWN, 65, 65, "A", 65, 
                         avg.KEYMOD_NONE),
                  lambda: self.assert_(self.keyDownCalled),
                  lambda: Helper.fakeKeyEvent(avg.KEYUP, 65, 65, "A", 65, avg.KEYMOD_NONE),
                  lambda: self.assert_(self.keyUpCalled)
                 ))
 
-    def testGlobalEvents(self):
-        global mainMouseUpCalled
-        global mainMouseDownCalled
-
-        def reset():
-            global mainMouseUpCalled
-            global mainMouseDownCalled
-            mainMouseUpCalled = False
-            mainMouseDownCalled = False
-        
-        Player.loadString("""
-            <avg width="160" height="120"  
-                    oncursordown="mainMouseDown" oncursorup="mainMouseUp"/>
-        """)
-        
-        self.start(None, 
-                (lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
-                        10, 10, 1),
-                 lambda: self.assert_(mainMouseDownCalled and not(mainMouseUpCalled)),
-                 reset,
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
-                        10, 10, 1),
-                 lambda: self.assert_(not(mainMouseDownCalled) and mainMouseUpCalled)
-                ))
-
     def testSimpleEvents(self):
         def getMouseState():
             Event = Player.getMouseState()
-            self.assert_(Event.pos == avg.Point2D(10,10))
-            self.assert_(Event.lastdownpos == avg.Point2D(10,10))
-            # Make sure we're getting a Point2D as return value.
-            self.assert_(Event.lastdownpos/2 == avg.Point2D(5, 5))
+            self.assertEqual(Event.pos, avg.Point2D(10,10))
         
-        self.loadEmptyScene()
-        root = Player.getRootNode()
+        root = self.loadEmptyScene()
         img1 = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=root)
         handlerTester1 = NodeHandlerTester(self, img1)
 
         img2 = avg.ImageNode(pos=(64,0), href="rgb24-65x65.png", parent=root)
         handlerTester2 = NodeHandlerTester(self, img2)
 
-        self.start(None, 
-                (# down, getMouseState(), move, up.
+        self.start(( 
+                 # down, getMouseState(), move, up.
                  # events are inside img1 but outside img2.
                  lambda: self.assert_(not(Player.isMultitouchAvailable())),
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
@@ -208,7 +104,7 @@ class EventTestCase(AVGTestCase):
                  lambda: handlerTester1.assertState(
                         down=False, up=False, over=False, out=False, move=True),
 
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         12, 12, 1),
                  lambda: handlerTester1.assertState(
                         down=False, up=True, over=False, out=False, move=False)
@@ -216,33 +112,32 @@ class EventTestCase(AVGTestCase):
                 ))
 
     def testTilted(self):
-        self.loadEmptyScene()
-        root = Player.getRootNode()
+        root = self.loadEmptyScene()
+        root = root
         img = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", angle=0.785, parent=root)
         handlerTester = NodeHandlerTester(self, img)
         
-        self.start(None, 
-                (lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
+        self.start(( 
+                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         32, 32, 1),
                  lambda: handlerTester.assertState(
                         down=True, up=False, over=True, out=False, move=False),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         0, 0, 1),
                  lambda: handlerTester.assertState(
                         down=False, up=False, over=False, out=True, move=False),
                 ))
 
     def testDivEvents(self):
-        self.loadEmptyScene()
-        root = Player.getRootNode()
+        root = self.loadEmptyScene()
         div = avg.DivNode(pos=(0,0), parent=root)
         divHandlerTester = NodeHandlerTester(self, div)
 
         img = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=div)
         imgHandlerTester = NodeHandlerTester(self, img)
         
-        self.start(None, 
-                (# down, move, up.
+        self.start(( 
+                 # down, move, up.
                  # events are inside img and therefore should bubble to div. 
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         10, 10, 1),
@@ -258,7 +153,7 @@ class EventTestCase(AVGTestCase):
                  lambda: imgHandlerTester.assertState(
                         down=False, up=False, over=False, out=False, move=True),
         
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         12, 12, 1),
                  lambda: divHandlerTester.assertState(
                         down=False, up=True, over=False, out=False, move=False),
@@ -289,8 +184,7 @@ class EventTestCase(AVGTestCase):
         def unlinkHandler(event):
             self.img.disconnectEventHandler(self)
 
-        self.loadEmptyScene()
-        root = Player.getRootNode()
+        root = self.loadEmptyScene()
         self.img = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=root)
         connectTwoHandlers()
         self.img.disconnectEventHandler(self, onDown1)
@@ -299,31 +193,27 @@ class EventTestCase(AVGTestCase):
         self.img.disconnectEventHandler(self)
 
         resetDownCalled()
-        self.start(None, 
-                (connectTwoHandlers,
-                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
-                        10, 10, 1),
+        self.start(( 
+                 connectTwoHandlers,
+                 lambda: self.fakeClick(10,10),
                  lambda: self.assert_(self.down1Called and self.down2Called),
                  resetDownCalled,
                  lambda: self.img.disconnectEventHandler(self, onDown1),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
-                        10, 10, 1),
+                 lambda: self.fakeClick(10,10),
                  lambda: self.assert_(not(self.down1Called) and self.down2Called),
                  connectUnlinkHandler,
-                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
-                        10, 10, 1)
+                 lambda: self.fakeClick(10,10),
                 ))
 
     def testObscuringEvents(self):
-        self.loadEmptyScene()
-        root = Player.getRootNode()
+        root = self.loadEmptyScene()
         img1 = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=root)
         handlerTester1 = NodeHandlerTester(self, img1)
 
         img2 = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=root)
         handlerTester2 = NodeHandlerTester(self, img2)
-        self.start(None, 
-                (# down, move, up.
+        self.start(( 
+                 # down, move, up.
                  # events should only arrive at img2 because img1 is obscured by img1.
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         10, 10, 1),
@@ -339,7 +229,7 @@ class EventTestCase(AVGTestCase):
                  lambda: handlerTester2.assertState(
                         down=False, up=False, over=False, out=False, move=True),
 
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         12, 12, 1),
                  lambda: handlerTester1.assertState(
                         down=False, up=False, over=False, out=False, move=False),
@@ -349,53 +239,70 @@ class EventTestCase(AVGTestCase):
 
     def testSensitive(self):
         # Tests both sensitive and active attributes.
-        def activateNode(useSensitiveAttr, b):
+        def activateNode(node, useSensitiveAttr, b):
             if useSensitiveAttr:
-                self.img.sensitive = b
+                node.sensitive = b
             else:
-                self.img.active = b
+                node.active = b
+
+        def onNode2Down(event):
+            self.__node2Down = True
+        
         for useSensitiveAttr in (True, False):
-            self.loadEmptyScene()
-            root = Player.getRootNode()
-            self.img = avg.ImageNode(id="img", pos=(0,0), href="rgb24-65x65.png", 
-                    parent=root)
+            root = self.loadEmptyScene()
+            self.img = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=root)
             handlerTester = NodeHandlerTester(self, self.img)
 
-            activateNode(useSensitiveAttr, False)
-            self.start(None,
-                    (# Node is inactive -> no events.
+            activateNode(self.img, useSensitiveAttr, False)
+            self.start((
+                     # Node is inactive -> no events.
                      lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                             10, 10, 1),
                      lambda: handlerTester.assertState(
                             down=False, up=False, over=False, out=False, move=False),
-                     lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                     lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                             10, 10, 1),
 
                      # Activate the node -> events arrive.
-                     lambda: activateNode(useSensitiveAttr, True),
+                     lambda: activateNode(self.img, useSensitiveAttr, True),
                      lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                             10, 10, 1),
                      lambda: handlerTester.assertState(
                             down=True, up=False, over=True, out=False, move=False),
-                     lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                     lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                             10, 10, 1),
                     ))
             self.img = None
 
+            # Check if sensitive is deactivated immediately, not at the end of the frame.
+            root = self.loadEmptyScene()
+            self.img1 = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=root)
+            self.img2 = avg.ImageNode(pos=(64,0), href="rgb24-65x65.png", parent=root)
+            self.img1.connectEventHandler(avg.CURSORDOWN, avg.TOUCH, self, 
+                    lambda event: activateNode(self.img2, useSensitiveAttr, False))
+            self.img2.connectEventHandler(avg.CURSORDOWN, avg.TOUCH, self, onNode2Down)
+            self.__node2Down = False
+
+            self.start((
+                        lambda: self._sendTouchEvents((
+                                (1, avg.CURSORDOWN, 10, 10),
+                                (2, avg.CURSORDOWN, 80, 10),)),
+                        lambda: self.assert_(not(self.__node2Down)),
+                      ))
+
     def testChangingHandlers(self):
-        self.loadEmptyScene()
-        root = Player.getRootNode()
+        root = self.loadEmptyScene()
         img = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=root)
         handlerTester = NodeHandlerTester(self, img)
         
-        self.start(None, 
-                (lambda: handlerTester.clearHandlers(),
+        self.start(( 
+                 lambda: handlerTester.clearHandlers(),
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         10, 10, 1),
                  lambda: handlerTester.assertState(
                         down=False, up=False, over=False, out=False, move=False),
                  lambda: handlerTester.setHandlers(),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         10, 10, 1),
                  lambda: handlerTester.assertState(
                         down=False, up=True, over=False, out=False, move=False),
@@ -432,31 +339,30 @@ class EventTestCase(AVGTestCase):
         self.mouseDownCalled = False
         self.mainMouseDownCalled = False
 
-        self.loadEmptyScene()
-        root = Player.getRootNode()
+        root = self.loadEmptyScene()
         root.setEventHandler(avg.CURSORDOWN, avg.MOUSE, onMainMouseDown)
         self.img = avg.ImageNode(pos=(0,0), href="rgb24-65x65.png", parent=root)
         self.img.setEventHandler(avg.CURSORDOWN, avg.MOUSE, onMouseDown)
 
-        self.start(None,
-                (lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
+        self.start((
+                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         10, 10, 1),
                  lambda: self.assert_(self.mouseDownCalled),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         10, 10, 1),
                  captureEvent,
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         100, 10, 1),
                  lambda: self.assert_(self.mouseDownCalled and 
                         self.mainMouseDownCalled),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         100, 10, 1),
                  noCaptureEvent,
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         100, 10, 1),
                  lambda: self.assert_(not(self.mouseDownCalled) and 
                         self.mainMouseDownCalled),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         100, 10, 1),
                  doubleCaptureEvent,
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
@@ -464,6 +370,8 @@ class EventTestCase(AVGTestCase):
                  lambda: self.assert_(self.mouseDownCalled and 
                         self.mainMouseDownCalled),
                  releaseTooMuch,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
+                        100, 10, 1),
                 ))
         self.img = None
 
@@ -508,19 +416,20 @@ class EventTestCase(AVGTestCase):
             Parent.removeChild(Parent.indexOf(img1))
         
         Helper = Player.getTestHelper()
-        Player.loadFile("mouseover.avg")
-        img2 = Player.getElementByID("img2")
+        root = self.loadEmptyScene()
+        img1 = avg.ImageNode(href="rgb24-65x65.png", parent=root)
+        div = avg.DivNode(pos=(65,0), parent=root)
+        img3 = avg.ImageNode(href="rgb24-65x65.png", parent=div)
+        img2 = avg.ImageNode(pos=(0,65), href="rgb24-65x65.png", parent=div)
+        
         img2.setEventHandler(avg.CURSOROVER, avg.MOUSE, onImg2MouseOver)
         img2.setEventHandler(avg.CURSOROUT, avg.MOUSE, onImg2MouseOut)
-        div = Player.getElementByID("div1")
         div.setEventHandler(avg.CURSOROVER, avg.MOUSE, onDivMouseOver)
         div.setEventHandler(avg.CURSOROUT, avg.MOUSE, onDivMouseOut)
-        avgNode = Player.getRootNode()
-        avgNode.setEventHandler(avg.CURSOROVER, avg.MOUSE, onAVGMouseOver)
-        img1 = Player.getElementByID("img1")
+        root.setEventHandler(avg.CURSOROVER, avg.MOUSE, onAVGMouseOver)
         img1.setEventHandler(avg.CURSOROVER, avg.MOUSE, onImg1MouseOver)
-        self.start(None, 
-                (resetState,
+        self.start(( 
+                 resetState,
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         70, 70, 1),
                  lambda: self.assert_(
@@ -530,7 +439,8 @@ class EventTestCase(AVGTestCase):
                         not(self.img2MouseOutCalled) and 
                         not(self.divMouseOutCalled) and 
                         not(self.img1MouseOverCalled)),
-
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
+                        70, 70, 1),
                  resetState,
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
                         70, 10, 1),
@@ -541,6 +451,8 @@ class EventTestCase(AVGTestCase):
                         self.img2MouseOutCalled and 
                         not(self.divMouseOutCalled) and 
                         not(self.img1MouseOverCalled)),
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
+                        70, 10, 1),
                  
                  resetState,
                  lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
@@ -555,7 +467,7 @@ class EventTestCase(AVGTestCase):
 
                  resetState,
                  killNodeUnderCursor,
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         10, 10, 1),
                  lambda: self.assert_(
                         not(self.img2MouseOverCalled) and 
@@ -565,9 +477,11 @@ class EventTestCase(AVGTestCase):
                         not(self.divMouseOutCalled) and 
                         not(self.img1MouseOverCalled)),
 
+                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
+                        10, 10, 1),
                  resetState,
-                 lambda: Player.getElementByID("img2").setEventCapture(),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: img2.setEventCapture(),
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         70, 70, 1),
                  lambda: self.assert_(
                         self.img2MouseOverCalled and 
@@ -577,8 +491,10 @@ class EventTestCase(AVGTestCase):
                         not(self.divMouseOutCalled) and 
                         not(self.img1MouseOverCalled)),
 
+                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
+                        70, 70, 1),
                  resetState,
-                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, True, False, False,
+                 lambda: Helper.fakeMouseEvent(avg.CURSORUP, False, False, False,
                         10, 10, 1),
                  lambda: self.assert_(
                         not(self.img2MouseOverCalled) and 
@@ -593,11 +509,11 @@ class EventTestCase(AVGTestCase):
         def onErrMouseOver(Event):
             undefinedFunction()
 
-        self.loadEmptyScene()
-        Player.getRootNode().setEventHandler(avg.CURSORDOWN, avg.MOUSE, onErrMouseOver)
+        root = self.loadEmptyScene()
+        root.setEventHandler(avg.CURSORDOWN, avg.MOUSE, onErrMouseOver)
         self.assertException(lambda:
-                self.start(None,
-                        (lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, 
+                self.start((
+                         lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, 
                                 False, False, False, 10, 10, 0),
                 )))
     
@@ -617,55 +533,190 @@ class EventTestCase(AVGTestCase):
             elif isinstance(event, avg.KeyEvent):
                 self.ehookKeyboardEvent = True
             else:
-                self.assert_(False)
+                self.fail()
             
-        self.loadEmptyScene()
+        root = self.loadEmptyScene()
         resetState()
 
         Player.setEventHook(handleEvent)
-        self.start(None,
-                (lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, 
-                        False, False, False, 10, 10, 0),
-                lambda: self.assert_(self.ehookMouseEvent),
-                lambda: Helper.fakeKeyEvent(avg.KEYDOWN, 65, 65, "A", 65, 0),
-                lambda: self.assert_(self.ehookKeyboardEvent),
-                cleanup,
-                lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, 
-                        False, False, False, 10, 10, 0),
-                lambda: self.assert_(not self.ehookMouseEvent),
-                lambda: Helper.fakeKeyEvent(avg.KEYDOWN, 65, 65, "A", 65, 0),
-                lambda: self.assert_(not self.ehookKeyboardEvent),
+        self.start((
+                 lambda: self.fakeClick(10, 10),
+                 lambda: self.assert_(self.ehookMouseEvent),
+                 lambda: Helper.fakeKeyEvent(avg.KEYDOWN, 65, 65, "A", 65, 0),
+                 lambda: self.assert_(self.ehookKeyboardEvent),
+                 cleanup,
+                 lambda: self.fakeClick(10, 10),
+                 lambda: self.assert_(not self.ehookMouseEvent),
+                 lambda: Helper.fakeKeyEvent(avg.KEYDOWN, 65, 65, "A", 65, 0),
+                 lambda: self.assert_(not self.ehookKeyboardEvent),
             ))
         
     def testException(self):
+
         class TestException(Exception):
             pass
         
         def throwException(event):
             raise TestException
         
-        rect = libavg.RectNode(size = (50, 50))
+        rect = avg.RectNode(size = (50, 50))
         rect.setEventHandler(avg.CURSORDOWN, avg.MOUSE, throwException)
         
-        self.loadEmptyScene()
-        libavg.Player.get().getRootNode().appendChild(rect)
+        root = self.loadEmptyScene()
+        root.appendChild(rect)
         
         self.__exceptionThrown = False
         try:
-            self.start(None,
-                (lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 10, 10, 0),
+            self.start((
+                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 10, 
+                        10, 0),
                  lambda: None))
         except TestException:
             self.__exceptionThrown = True
             
         self.assert_(self.__exceptionThrown)
-        #libavg.Player.get().getRootNode().removeChild(rect)
-                
+               
+    def testContacts(self):
+
+        def onDown(event):
+            contact = event.contact
+            self.assertEqual(event.cursorid, contact.id)
+            self.assertEqual(contact.age, 0)
+            self.assertEqual(contact.distancefromstart, 0)
+            self.assertEqual(contact.motionangle, 0)
+            self.assertEqual(contact.motionvec, (0,0))
+            self.assertEqual(contact.distancetravelled, 0)
+            self.assertEqual(contact.events[0].pos, event.pos)
+            self.assertEqual(len(contact.events), 1)
+            contact.connectListener(onMotion, onUp)
+
+        def onMotion(event):
+            contact = event.contact
+            self.assertEqual(event.cursorid, contact.id)
+            self.assertEqual(contact.age, 40)
+            self.assertEqual(contact.distancefromstart, 10)
+            self.assertEqual(contact.motionangle, 0)
+            self.assertEqual(contact.motionvec, (10,0))
+            self.assertEqual(contact.distancetravelled, 10)
+            self.assertEqual(contact.events[-1].pos, event.pos)
+            self.assert_(len(contact.events) > 1)
+            self.numContactCallbacks += 1
+ 
+        def onUp(event):
+            contact = event.contact
+            self.assertEqual(event.cursorid, contact.id)
+            self.assertEqual(contact.age, 80)
+            self.assertEqual(contact.distancefromstart, 0)
+            self.assertEqual(contact.motionangle, 0)
+            self.assertEqual(contact.motionvec, (0,0))
+            self.assertEqual(contact.distancetravelled, 20)
+            self.assertEqual(contact.events[-1].pos, event.pos)
+            self.assert_(len(contact.events) > 1)
+            self.numContactCallbacks += 1
+
+        def onOver(event): 
+            self.numOverCallbacks += 1
+            self.assertEqual(event.cursorid, event.contact.id)
+
+        def onOut(event): 
+            self.numOutCallbacks += 1
+            self.assertEqual(event.cursorid, event.contact.id)
+
+        root = self.loadEmptyScene()
+        root.connectEventHandler(avg.CURSORDOWN, avg.TOUCH, self, onDown)
+        self.numContactCallbacks = 0
+        rect = avg.RectNode(pos=(5,5), size=(10,10), parent=root)
+        rect.connectEventHandler(avg.CURSOROVER, avg.TOUCH, self, onOver)
+        self.numOverCallbacks = 0
+        rect.connectEventHandler(avg.CURSOROUT, avg.TOUCH, self, onOut)
+        self.numOutCallbacks = 0
+        Player.setFakeFPS(25)
+        self.start((
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORDOWN, avg.TOUCH, (10,10)),
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORMOTION, avg.TOUCH, (20,10)),
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORUP, avg.TOUCH, (10,10)),
+            ))
+        self.assertEqual(self.numContactCallbacks, 2)
+        self.assertEqual(self.numOverCallbacks, 2)
+        self.assertEqual(self.numOutCallbacks, 2)
+        
+        root = self.loadEmptyScene()
+        root.connectEventHandler(avg.CURSORDOWN, avg.MOUSE, self, onDown)
+        self.numContactCallbacks = 0
+        self.start((
+             lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, 1, 0, 0, 10, 10, 0),
+             lambda: Helper.fakeMouseEvent(avg.CURSORMOTION, 1, 0, 0, 20, 10, 0),
+             lambda: Helper.fakeMouseEvent(avg.CURSORUP, 0, 0, 0, 10, 10, 0),
+            ))
+        self.assertEqual(self.numContactCallbacks, 2)
+
+    def testContactRegistration(self):
+
+        def onDown(event):
+            root.setEventCapture(event.cursorid)
+            root.releaseEventCapture(event.cursorid)
+
+        def onMotion(event):
+            contact = event.contact
+            self.contactID = contact.connectListener(onContactMotion, None)
+            self.numMotionCallbacks += 1
+            root.disconnectEventHandler(self)
+
+        def onContactMotion(event):
+            contact = event.contact
+            contact.disconnectListener(self.contactID)
+            self.assertException(lambda: contact.disconnectListener(self.contactID))
+            self.numContactCallbacks += 1
+       
+        root = self.loadEmptyScene()
+        root.connectEventHandler(avg.CURSORDOWN, avg.TOUCH, self, onDown)
+        self.numMotionCallbacks = 0
+        root.connectEventHandler(avg.CURSORMOTION, avg.TOUCH, self, onMotion)
+        self.numContactCallbacks = 0
+        Player.setFakeFPS(25)
+        self.start((
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORDOWN, avg.TOUCH, (10,10)),
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORMOTION, avg.TOUCH, (20,10)),
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORMOTION, avg.TOUCH, (30,10)),
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORMOTION, avg.TOUCH, (40,10)),
+            ))
+        self.assertEqual(self.numContactCallbacks, 1)
+        self.assertEqual(self.numMotionCallbacks, 1)
+        
+    def testMultiContactRegistration(self):
+
+        def onDown(event):
+            contact = event.contact
+            self.contactid = contact.connectListener(onContact2, onContact2)
+            contact.connectListener(onContact1, onContact1)
+
+        def onContact1(event):
+            if self.numContact1Callbacks == 0:
+                event.contact.disconnectListener(self.contactid)
+            self.numContact1Callbacks += 1
+
+        def onContact2(event):
+            self.assertEqual(self.numContact1Callbacks, 0)
+            self.numContact2Callbacks += 1
+        
+        root = self.loadEmptyScene()
+        root.connectEventHandler(avg.CURSORDOWN, avg.TOUCH, self, onDown)
+        Player.setFakeFPS(25)
+        self.numContact1Callbacks = 0
+        self.numContact2Callbacks = 0
+        self.start((
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORDOWN, avg.TOUCH, (10,10)),
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORMOTION, avg.TOUCH, (20,10)),
+             lambda: Helper.fakeTouchEvent(1, avg.CURSORUP, avg.TOUCH, (10,10)),
+            ))
+        self.assertEqual(self.numContact1Callbacks, 2)
+        # The order of callbacks is unspecified, so onContact2 might be called once.
+        self.assert_(self.numContact2Callbacks <= 1)
+
 
 def eventTestSuite(tests):
     availableTests = (
             "testKeyEvents",
-            "testGlobalEvents",
             "testSimpleEvents",
             "testTilted",
             "testDivEvents",
@@ -677,10 +728,12 @@ def eventTestSuite(tests):
             "testMouseOver",
             "testEventErr",
             "testEventHook",
-            "testException"
+            "testException",
+            "testContacts",
+            "testContactRegistration",
+            "testMultiContactRegistration",
             )
     return createAVGTestSuite(availableTests, EventTestCase, tests)
 
 Player = avg.Player.get()
 Helper = Player.getTestHelper()
-
