@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2008 Ulrich von Zadow
+# Copyright (C) 2003-2011 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,10 +20,8 @@
 # Current versions can be found at www.libavg.de
 #
 
-import unittest
 
 import shutil
-import math
 
 from libavg import avg
 from testcase import *
@@ -38,17 +36,16 @@ class ImageTestCase(AVGTestCase):
         def createXmlNode(pos):
             node = Player.createNode(
                     """<image pos="%s" href="rgb24-32x32.png"/>"""%str(pos))
-            self.assert_(node.getMediaSize() == avg.Point2D(32, 32))
+            self.assertEqual(node.getMediaSize(), avg.Point2D(32, 32))
             return node
 
         def createDictNode(root, p):
             node = avg.ImageNode(pos=p, href="rgb24-32x32.png", parent=root)
-            self.assert_(node.getMediaSize() == avg.Point2D(32, 32))
-            self.assert_(node.size == avg.Point2D(32, 32))
+            self.assertEqual(node.getMediaSize(), avg.Point2D(32, 32))
+            self.assertEqual(node.size, avg.Point2D(32, 32))
             return node
 
         def addNodes(y):
-            root = Player.getRootNode()
             
             xmlNode = createXmlNode((16, y))
             root.appendChild(xmlNode)
@@ -57,19 +54,18 @@ class ImageTestCase(AVGTestCase):
             
             noAttachNode = createXmlNode((80, y))
             noAttachNode.href = "rgb24alpha-32x32.png"
-            self.assert_(noAttachNode.getMediaSize() == avg.Point2D(32, 32))
-            self.assert_(noAttachNode.size == avg.Point2D(32,32))
+            self.assertEqual(noAttachNode.getMediaSize(), avg.Point2D(32, 32))
+            self.assertEqual(noAttachNode.size, avg.Point2D(32,32))
             root.appendChild(noAttachNode)
 
             attachNode = createXmlNode((112, y))
             root.appendChild(attachNode)
             attachNode.href = "rgb24alpha-32x32.png"
-            self.assert_(attachNode.getMediaSize() == avg.Point2D(32, 32))
-            self.assert_(attachNode.size == avg.Point2D(32,32))
+            self.assertEqual(attachNode.getMediaSize(), avg.Point2D(32, 32))
+            self.assertEqual(attachNode.size, avg.Point2D(32,32))
 
         def setUnicodeHref():
             if self._isCurrentDirWriteable():
-                root = Player.getRootNode()
                 # Can't check unicode filenames into svn or the windows client breaks.
                 # So we rename the file locally.
                 shutil.copyfile("oe.png", u"ö.png")
@@ -82,10 +78,10 @@ class ImageTestCase(AVGTestCase):
             if self._isCurrentDirWriteable():
                 self.compareImage("testImgHRef3", False)
 
-        self.loadEmptyScene()
+        root = self.loadEmptyScene()
         addNodes(16)
-        self.start(None,
-                (lambda: self.compareImage("testImgHRef1", False),
+        self.start((
+                 lambda: self.compareImage("testImgHRef1", False),
                  lambda: addNodes(48),
                  lambda: self.compareImage("testImgHRef2", False),
                  setUnicodeHref,
@@ -105,7 +101,6 @@ class ImageTestCase(AVGTestCase):
             self.assertException(node.pos.y == 23)
 
         def addNodes(y):
-            root = Player.getRootNode()
             xmlNode = createXmlNode((16, y))
             root.appendChild(xmlNode)
             createDictNode(root, (48, y))
@@ -118,10 +113,10 @@ class ImageTestCase(AVGTestCase):
             attachNode.pos = avg.Point2D(112, y)
             illegalMove(attachNode)
 
-        self.loadEmptyScene()
+        root = self.loadEmptyScene()
         addNodes(16)
-        self.start(None,
-                (lambda: self.compareImage("testImgPos1", False),
+        self.start((
+                 lambda: self.compareImage("testImgPos1", False),
                  lambda: addNodes(48),
                  lambda: self.compareImage("testImgPos2", False),
                 ))
@@ -136,26 +131,25 @@ class ImageTestCase(AVGTestCase):
             return avg.ImageNode(pos=p, size=s, href="rgb24-64x64.png")
 
         def addNodes(y):
-            root = Player.getRootNode()
             xmlNode = createXmlNode((16, y), (32, 32))
-            self.assert_(xmlNode.size == avg.Point2D(32, 32))
+            self.assertEqual(xmlNode.size, avg.Point2D(32, 32))
             root.appendChild(xmlNode)
             dictNode = createDictNode((48, y), (32, 32))
-            self.assert_(dictNode.size == avg.Point2D(32, 32))
+            self.assertEqual(dictNode.size, avg.Point2D(32, 32))
             root.appendChild(dictNode)
             noAttachNode = createXmlNode((80, y), (0, 0))
             noAttachNode.size = avg.Point2D(32, 32)
-            self.assert_(noAttachNode.size == avg.Point2D(32, 32))
+            self.assertEqual(noAttachNode.size, avg.Point2D(32, 32))
             root.appendChild(noAttachNode)
             attachNode = createXmlNode((112, y), (0, 0))
             root.appendChild(attachNode)
             attachNode.size = avg.Point2D(32, 32)
-            self.assert_(attachNode.size == avg.Point2D(32, 32))
+            self.assertEqual(attachNode.size, avg.Point2D(32, 32))
 
-        self.loadEmptyScene()
+        root = self.loadEmptyScene()
         addNodes(16)
-        self.start(None,
-                (lambda: self.compareImage("testImgSize1", False),
+        self.start((
+                 lambda: self.compareImage("testImgSize1", False),
                  lambda: addNodes(48),
                  lambda: self.compareImage("testImgSize2", False),
                 ))
@@ -171,7 +165,6 @@ class ImageTestCase(AVGTestCase):
             node.setWarpedVertexCoords(grid)
 
         def testEarlyAccessException():
-            root = Player.getRootNode()
             node = createNode((16, 16))
             root.appendChild(node)
             self.assertException(node.getWarpedVertexCoords)
@@ -179,32 +172,32 @@ class ImageTestCase(AVGTestCase):
 
         def addNode():
             node = createNode((16, 16))
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
             moveVertex(node)
         
-        self.loadEmptyScene()
+        root = self.loadEmptyScene()
         testEarlyAccessException()
-        self.start(None,
-                (lambda: addNode(),
+        self.start((
+                 lambda: addNode(),
                  lambda: self.compareImage("testImgWarp1", False),
                 ))
 
     def testBitmap(self):
         def getBitmap(node):
             bmp = node.getBitmap()
-            self.assert_(bmp.getSize() == (65,65))
+            self.assertEqual(bmp.getSize(), (65,65))
             self.assert_(bmp.getFormat() == avg.R8G8B8X8 or 
                     bmp.getFormat() == avg.B8G8R8X8)
             node.setBitmap(bmp)
-            self.assert_(node.getMediaSize() == (65,65))
+            self.assertEqual(node.getMediaSize(), (65,65))
         
         def loadFromBitmap(p, orighref):
             node = avg.ImageNode(pos=p, size=(32, 32), href=orighref)
             bmp = avg.Bitmap('rgb24-65x65.png')
-            self.assert_(bmp.getSize() == (65,65))
+            self.assertEqual(bmp.getSize(), (65,65))
             node.setBitmap(bmp)
-            self.assert_(node.getMediaSize() == (65,65))
-            Player.getRootNode().appendChild(node)
+            self.assertEqual(node.getMediaSize(), (65,65))
+            root.appendChild(node)
         
         def testStringConversion():
             bmp = avg.Bitmap('rgb24-65x65.png')
@@ -223,14 +216,14 @@ class ImageTestCase(AVGTestCase):
 
         def testGetPixel():
             bmp = avg.Bitmap('rgb24-65x65.png')
-            self.assert_(bmp.getPixel((1,1)) == (255,0,0,255))
-            self.assert_(bmp.getPixel((33,1)) == (0,255,0,255))
+            self.assertEqual(bmp.getPixel((1,1)), (255,0,0,255))
+            self.assertEqual(bmp.getPixel((33,1)), (0,255,0,255))
             bmp = avg.Bitmap('rgb24alpha-64x64.png')
-            self.assert_(bmp.getPixel((1,1)) == (0,0,0,0))
-            self.assert_(bmp.getPixel((63,1)) == (83,255,83,142))
+            self.assertEqual(bmp.getPixel((1,1)), (0,0,0,0))
+            self.assertEqual(bmp.getPixel((63,1)), (83,255,83,142))
             bmp = avg.Bitmap('greyscale.png')
-            self.assert_(bmp.getPixel((1,1)) == (255,255,255,255))
-            self.assert_(bmp.getPixel((1,63)) == (0,0,0,255))
+            self.assertEqual(bmp.getPixel((1,1)), (255,255,255,255))
+            self.assertEqual(bmp.getPixel((1,63)), (0,0,0,255))
             self.assertException(lambda: bmp.getPixel((64,0)))
 
         def setNullBitmap():
@@ -239,17 +232,17 @@ class ImageTestCase(AVGTestCase):
         node = avg.ImageNode(href="rgb24-65x65.png", size=(32, 32))
         getBitmap(node)
 
-        self.loadEmptyScene()
+        root = self.loadEmptyScene()
         node = avg.ImageNode(pos=(0,0), size=(32, 32), href="rgb24-65x65.png")
-        Player.getRootNode().appendChild(node)
+        root.appendChild(node)
         getBitmap(node)
-        self.assert_(node.size == (32,32))
+        self.assertEqual(node.size, (32,32))
         loadFromBitmap((32,0), "")
         loadFromBitmap((64,0), "rgb24alpha-64x64.png")
         testStringConversion()
         testUnicode()
-        self.start(None,
-                (lambda: getBitmap(node),
+        self.start((
+                 lambda: getBitmap(node),
                  lambda: loadFromBitmap((32,32), ""),
                  lambda: loadFromBitmap((64,32), "rgb24alpha-64x64.png"),
                  lambda: self.compareImage("testBitmap1", False),
@@ -257,53 +250,87 @@ class ImageTestCase(AVGTestCase):
                  lambda: self.assertException(setNullBitmap)
                 ))
 
+    def testBitmapManager(self):
+        WAIT_TIMEOUT = 2000
+        def expectException(returnValue, nextAction):
+            if isinstance(returnValue, Exception):
+                nextAction()
+            else:
+                raise RuntimeError("Expected exception, got %s (%s)" % (
+                        returnValue, type(returnValue)))
+            
+        def loadValidBitmap():
+            def validBitmapCb(bitmap):
+                self.assert_(not isinstance(bitmap, Exception))
+                Player.setTimeout(0, loadUnexistentBitmap)
+
+            avg.BitmapManager.get().loadBitmap("rgb24alpha-64x64.png", validBitmapCb)
+
+        def loadUnexistentBitmap():
+            avg.BitmapManager.get().loadBitmap("nonexistent.png",
+                    lambda bmp: expectException(
+                            returnValue=bmp,
+                            nextAction=lambda: Player.setTimeout(0, loadBrokenImage)))
+
+        def loadBrokenImage():
+            import tempfile
+            tempFileName = os.path.join(tempfile.gettempdir(),
+                    "broken.png")
+            open(tempFileName, "w")
+
+            def cleanupAndTestReturnValue(returnValue):
+                os.unlink(tempFileName)
+                expectException(returnValue=returnValue, nextAction=Player.stop)
+
+            avg.BitmapManager.get().loadBitmap(tempFileName,
+                    cleanupAndTestReturnValue)
+        
+        def reportStuck():
+            raise RuntimeError("BitmapManager didn't reply "
+                    "within %dms timeout" % WAIT_TIMEOUT)
+            Player.stop()
+            
+        root = self.loadEmptyScene()
+        
+        Player.setTimeout(WAIT_TIMEOUT, reportStuck)
+        Player.setResolution(0, 0, 0, 0)
+        loadValidBitmap()
+        Player.play()
+        
     def testBlendMode(self):
         def setBlendMode():
-            Player.getElementByID("blend").blendmode="add"
+            blendNode.blendmode="add"
         
-        Player.loadString("""
-            <avg width="160" height="120">
-                <image x="0" y="0" href="freidrehen.jpg"/>
-                <image id="blend" x="0" y="0" opacity="0.6" href="rgb24-65x65.png"/>
-                <image x="0" y="48" opacity="0.6" href="rgb24-65x65.png" blendmode="add"/>
-                <image x="48" y="0" opacity="1" href="rgb24-65x65.png" blendmode="min"/>
-                <image x="48" y="48" opacity="1" href="rgb24-65x65.png" 
-                        blendmode="max"/>
-            </avg>
-        """)
-        self.start(None,
-                (lambda: self.compareImage("testBlend1", False),
+        root = self.loadEmptyScene()
+        avg.ImageNode(href="freidrehen.jpg", parent=root)
+        blendNode = avg.ImageNode(opacity=0.6, href="rgb24-65x65.png", parent=root)
+        avg.ImageNode(pos=(0,48), opacity=0.6, href="rgb24-65x65.png", blendmode="add",
+                parent=root)
+        avg.ImageNode(pos=(48,0), opacity=1, href="rgb24-65x65.png", blendmode="min",
+                parent=root)
+        avg.ImageNode(pos=(48,48), opacity=1, href="rgb24-65x65.png", blendmode="max",
+                parent=root)
+
+        self.start((
+                 lambda: self.compareImage("testBlend1", False),
                  setBlendMode,
                  lambda: self.compareImage("testBlend2", False)
                 ))
 
-    def _isMaskSupported(self):
-        global g_IsMaskSupported
-        if g_IsMaskSupported == None:
-            self.loadEmptyScene()
-            node = avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png")
-            Player.getRootNode().appendChild(node)
-            try:
-                self.start(None, [])
-                g_IsMaskSupported = True
-            except RuntimeError:
-                g_IsMaskSupported = False
-        return g_IsMaskSupported
-
     def testImageMask(self):
         def createNode(p):
             node = avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png", 
                     pos=p, size=(32, 32))
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
 
         def setNoAttach(p):
             node = avg.ImageNode(href="rgb24-65x65.png", pos=p, size=(32, 32))
             node.maskhref = "mask.png"
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
 
         def setAttach(p):
             node = avg.ImageNode(href="rgb24-65x65.png", pos=p, size=(32, 32))
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
             node.maskhref = "mask.png"
 
         def changeHRef():
@@ -315,16 +342,15 @@ class ImageTestCase(AVGTestCase):
         def setMaskNotFound():
             node.maskhref = "nonexistentmask.png"        
             
-        if not(self._isMaskSupported()):
-            print "Skipping testImageMask - no shader support."
+        if not(self._hasShaderSupport()):
             return
-        self.loadEmptyScene()
+        root = self.loadEmptyScene()
         createNode((0,0))
-        node = Player.getRootNode().getChild(0)
+        node = root.getChild(0)
         setNoAttach((32,0))
         setAttach((64,0))
-        self.start(None,
-                (lambda: createNode((0, 32)),
+        self.start((
+                 lambda: createNode((0, 32)),
                  lambda: setNoAttach((32,32)),
                  lambda: setAttach((64,32)),
                  lambda: self.compareImage("testImgMask1", False),
@@ -336,48 +362,41 @@ class ImageTestCase(AVGTestCase):
                 ))
 
     def testImageMaskCanvas(self):
-        if not(self._isMaskSupported()):
-            print "Skipping testImageMaskCanvas - no shader support."
+        if not(self._hasShaderSupport()):
             return
-        self.loadEmptyScene()
-        Player.loadCanvasString("""
-            <canvas id="testcanvas" width="64" height="64">
-                <image href="rgb24-64x64.png"/>
-            </canvas>
-        """)
-        node = avg.ImageNode(href="canvas:testcanvas", maskhref="mask.png")
-        Player.getRootNode().appendChild(node)
-        self.start(None,
-                [lambda: self.compareImage("testImgMaskCanvas", False)
-                ])
+        root = self.loadEmptyScene()
+        canvas = Player.createCanvas(id="testcanvas", size=(64,64))
+        avg.ImageNode(href="rgb24-64x64.png", parent=canvas.getRootNode())
+        avg.RectNode(size=(160,120), fillcolor="FFFFFF", fillopacity=1, parent=root)
+        node = avg.ImageNode(href="canvas:testcanvas", maskhref="mask.png", parent=root)
+        self.start([lambda: self.compareImage("testImgMaskCanvas", False)])
 
     def testImageMaskPos(self):
         def createNode(p):
             node = avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png", 
                     pos=p, size=(32, 32), maskpos=(32, 32))
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
             
         def setNoAttach(p):
             node = avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png", 
                     pos=p, size=(32, 32))
             node.maskpos = (32, 32)
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
 
         def setAttach(p):
             node = avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png", 
                     pos=p, size=(32, 32))
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
             node.maskpos = (32, 32)
 
-        self.loadEmptyScene()
-        if not(self._isMaskSupported()):
-            print "Skipping testImageMaskPos - no shader support."
+        root = self.loadEmptyScene()
+        if not(self._hasShaderSupport()):
             return
         createNode((0,0))
         setNoAttach((32,0))
         setAttach((64,0))
-        self.start(None,
-                (lambda: createNode((0, 32)),
+        self.start((
+                 lambda: createNode((0, 32)),
                  lambda: setNoAttach((32,32)),
                  lambda: setAttach((64,32)),
                  lambda: self.compareImage("testImgMaskPos", False)
@@ -385,18 +404,18 @@ class ImageTestCase(AVGTestCase):
 
     def testImageMaskSize(self):
         def createNode(p):
-            node = avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png", 
-                    pos=p, size=(32, 32), masksize=(48, 48), parent=Player.getRootNode())
+            avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png", 
+                    pos=p, size=(32, 32), masksize=(48, 48), parent=root)
             
         def setNoAttach(p):
             node = avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png", 
                     pos=p, size=(32, 32))
             node.masksize = (48, 48)
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
 
         def setAttach(p):
             node = avg.ImageNode(href="rgb24-65x65.png", maskhref="mask.png", 
-                    pos=p, size=(32, 32), parent=Player.getRootNode())
+                    pos=p, size=(32, 32), parent=root)
             node.masksize = (48, 48)
 
         def setPos():
@@ -406,16 +425,15 @@ class ImageTestCase(AVGTestCase):
             node.maskpos = (0, 0)
             node.masksize = (0, 0)
 
-        self.loadEmptyScene()
-        if not(self._isMaskSupported()):
-            print "Skipping testImageMaskPos - no shader support."
+        root = self.loadEmptyScene()
+        if not(self._hasShaderSupport()):
             return
         createNode((0,0))
-        node = Player.getRootNode().getChild(0)
+        node = root.getChild(0)
         setNoAttach((32,0))
         setAttach((64,0))
-        self.start(None,
-                (lambda: createNode((0, 32)),
+        self.start((
+                 lambda: createNode((0, 32)),
                  lambda: setNoAttach((32,32)),
                  lambda: setAttach((64,32)),
                  lambda: self.compareImage("testImgMaskSize1", False),
@@ -426,16 +444,11 @@ class ImageTestCase(AVGTestCase):
                 ))
 
     def testImageMipmap(self):
-        Player.loadString("""
-            <avg id="imageavg" width="160" height="120">
-                <image width="64" height="64" href="checker.png"/>
-                <image x="64" width="64" height="64" href="checker.png" mipmap="true"/>
-            </avg>
-        
-        """)
-        self.start(None, 
-                [lambda: self.compareImage("testMipmap", False)
-                ])
+        root = self.loadEmptyScene()
+        avg.ImageNode(size=(64,64), href="checker.png", parent=root)
+        avg.ImageNode(pos=(64,0), size=(64,64), href="checker.png", mipmap=True, 
+                parent=root)
+        self.start([lambda: self.compareImage("testMipmap", False)])
 
     def testImageCompression(self):
         def loadBitmap():
@@ -444,26 +457,22 @@ class ImageTestCase(AVGTestCase):
 
         def relink():
             self.image.unlink(False)
-            Player.getRootNode().appendChild(self.image)
+            root.appendChild(self.image)
 
         def checkAlpha():
             self.image.href="rgb24alpha-64x64.png"
 
-        Player.loadString("""
-            <avg id="imageavg" width="160" height="120">
-                <image id="img" href="rgb24-64x64.png" compression="B5G6R5"/>
-            </avg>
-        
-        """)
-        self.image = Player.getElementByID("img")
-        self.assert_(self.image.compression == "B5G6R5")
-        self.start(None, 
-                [lambda: self.compareImage("testTexCompression1", False),
+        root = self.loadEmptyScene()
+        self.image = avg.ImageNode(href="rgb24-64x64.png", compression="B5G6R5",
+                parent=root)
+        self.assertEqual(self.image.compression, "B5G6R5")
+        self.start([
+                 lambda: self.compareImage("testTexCompression1", False),
                  loadBitmap,
                  lambda: self.compareImage("testTexCompression2", False),
                  relink,
                  lambda: self.compareImage("testTexCompression2", False),
-                 lambda: self.assertException(checkAlpha),
+                 checkAlpha,
                 ])
 
 
@@ -474,6 +483,7 @@ def imageTestSuite(tests):
             "testImageSize",
             "testImageWarp",
             "testBitmap",
+            "testBitmapManager",
             "testBlendMode",
             "testImageMask",
             "testImageMaskCanvas",
