@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2008 Ulrich von Zadow
+# Copyright (C) 2003-2011 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -33,25 +33,25 @@ class OffscreenTestCase(AVGTestCase):
         def createCanvas(isFirst, canvasName, x):
             canvas = self.__createOffscreenCanvas(canvasName, False)
             canvas.getElementByID("test1").x = x
-            node = avg.ImageNode(parent=Player.getRootNode(), id="imagenode")
+            node = avg.ImageNode(parent=root, id="imagenode")
             node.href="canvas:"+canvasName
             if isFirst:
-                self.assert_(canvas.getNumDependentCanvases() == 0)
+                self.assertEqual(canvas.getNumDependentCanvases(), 0)
                 self.canvas1 = canvas
             else:
-                self.assert_(canvas.getNumDependentCanvases() == 1)
+                self.assertEqual(canvas.getNumDependentCanvases(), 1)
                 self.canvas2 = canvas
 
         def unlink():
             self.node = Player.getElementByID("imagenode")
             self.node.unlink()
-            self.assert_(self.canvas1.getNumDependentCanvases() == 0)
+            self.assertEqual(self.canvas1.getNumDependentCanvases(), 0)
             gc.collect()
 
         def relink():
-            Player.getRootNode().appendChild(self.node)
+            root.appendChild(self.node)
             self.node = None
-            self.assert_(self.canvas1.getNumDependentCanvases() == 1)
+            self.assertEqual(self.canvas1.getNumDependentCanvases(), 1)
             
         def changeHRef(href):
             Player.getElementByID("imagenode").href = href
@@ -71,11 +71,11 @@ class OffscreenTestCase(AVGTestCase):
             Player.deleteCanvas("testcanvas2")
 #            self.assertException(lambda: Player.deleteCanvas("foo"))
 
-        self.loadEmptyScene()
+        root = self.loadEmptyScene()
         createCanvas(True, "testcanvas1", 0)
         firstNode = Player.getElementByID("imagenode")
-        self.start(None, 
-                (lambda: self.compareImage("testOffscreen1", False),
+        self.start(( 
+                 lambda: self.compareImage("testOffscreen1", False),
                  unlink,
                  lambda: self.compareImage("testOffscreen2", False), 
                  relink,
@@ -84,14 +84,14 @@ class OffscreenTestCase(AVGTestCase):
                  lambda: createCanvas(False, "testcanvas2", 80),
                  lambda: self.compareImage("testOffscreen3", False),
                  lambda: changeHRef("canvas:testcanvas1"),
-                 lambda: self.assert_(self.canvas1.getNumDependentCanvases() == 1),
-                 lambda: self.assert_(self.canvas2.getNumDependentCanvases() == 0),
+                 lambda: self.assertEqual(self.canvas1.getNumDependentCanvases(), 1),
+                 lambda: self.assertEqual(self.canvas2.getNumDependentCanvases(), 0),
                  lambda: self.compareImage("testOffscreen1", False),
                  lambda: changeHRef("rgb24-65x65.png"),
-                 lambda: self.assert_(self.canvas1.getNumDependentCanvases() == 0),
+                 lambda: self.assertEqual(self.canvas1.getNumDependentCanvases(), 0),
                  lambda: self.compareImage("testOffscreen4", False),
                  lambda: changeHRef("canvas:testcanvas1"),
-                 lambda: self.assert_(self.canvas1.getNumDependentCanvases() == 1),
+                 lambda: self.assertEqual(self.canvas1.getNumDependentCanvases(), 1),
                  lambda: self.compareImage("testOffscreen1", False),
                  setBitmap,
                  lambda: self.compareImage("testOffscreen4", False),
@@ -102,12 +102,11 @@ class OffscreenTestCase(AVGTestCase):
     def testCanvasLoadAfterPlay(self):
         def createOffscreenCanvas():
             offscreenCanvas = self.__createOffscreenCanvas("offscreencanvas", False)
-            self.node = avg.ImageNode(parent=Player.getRootNode(), 
+            self.node = avg.ImageNode(parent=root, 
                     href="canvas:offscreencanvas")
     
-        self.loadEmptyScene()
-        self.start(None,
-                (
+        root = self.loadEmptyScene()
+        self.start((
                  createOffscreenCanvas,
                  lambda: self.compareImage("testOffscreen1", False),
                 ))
@@ -117,8 +116,8 @@ class OffscreenTestCase(AVGTestCase):
             self.node.size = (80, 60)
 
         mainCanvas, offscreenCanvas = self.__setupCanvas(False)
-        self.start(None,
-                (setSize,
+        self.start((
+                 setSize,
                  lambda: self.compareImage("testCanvasResize", False)
                 ))
 
@@ -126,12 +125,11 @@ class OffscreenTestCase(AVGTestCase):
         self.loadEmptyScene()
         # Missing size
         self.assertException(
-                lambda: Player.loadCanvasString("""<canvas id="foo"/>"""))
+                lambda: Player.createCanvas(id="foo"))
         # Duplicate canvas id
-        Player.loadCanvasString("""<canvas id="foo" size="(160, 120)"/>""")
+        Player.createCanvas(id="foo", size=(160, 120))
         self.assertException(
-                lambda: Player.loadCanvasString(
-                        """<canvas id="foo" size="(160, 120)"/>"""))
+                lambda: Player.createCanvas(id="foo", size=(160, 120)))
 
     def testCanvasAPI(self):
         def checkMainScreenshot():
@@ -145,19 +143,19 @@ class OffscreenTestCase(AVGTestCase):
 
         def createCompressed():
             avg.ImageNode(href="canvas:offscreencanvas", compression="B5G6R5", 
-                    parent=Player.getRootNode())
+                    parent=root)
 
-        mainCanvas = self.loadEmptyScene()
-        self.assert_(mainCanvas == Player.getMainCanvas())
-        self.assert_(mainCanvas.getRootNode() == Player.getRootNode())
+        root = self.loadEmptyScene()
+        mainCanvas = Player.getMainCanvas()
+        self.assertEqual(mainCanvas.getRootNode(), root)
         offscreenCanvas = self.__createOffscreenCanvas("offscreencanvas", False)
-        self.assert_(offscreenCanvas == Player.getCanvas("offscreencanvas"))
-        self.assert_(offscreenCanvas.getElementByID("test1").href == "rgb24-65x65.png")
-        self.assert_(offscreenCanvas.getElementByID("missingnode") == None)
+        self.assertEqual(offscreenCanvas, Player.getCanvas("offscreencanvas"))
+        self.assertEqual(offscreenCanvas.getElementByID("test1").href, "rgb24-65x65.png")
+        self.assertEqual(offscreenCanvas.getElementByID("missingnode"), None)
         self.assertException(Player.screenshot)
         self.assertException(createCompressed)
-        self.start(None, 
-                (checkMainScreenshot,
+        self.start(( 
+                 checkMainScreenshot,
                  checkCanvasScreenshot))
 
     def testCanvasEvents(self):
@@ -179,33 +177,26 @@ class OffscreenTestCase(AVGTestCase):
         offscreenImage = offscreenCanvas.getElementByID("test1")
         offscreenImage.setEventHandler(avg.CURSORDOWN, avg.MOUSE, onOffscreenImageDown)
         Player.getRootNode().setEventHandler(avg.CURSORDOWN, avg.MOUSE, onMainDown)
-        helper = Player.getTestHelper()
         self.__offscreenImageDownCalled = False
         self.__mainDownCalled = False
-        self.start(None,
-                (lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        10, 10, 1),
+        self.start((
+                 lambda: self.fakeClick(10, 10),
                  lambda: self.assert_(self.__offscreenImageDownCalled),
                  reset,
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        80, 10, 1),
+                 lambda: self.fakeClick(80, 10),
                  lambda: self.assert_(not(self.__offscreenImageDownCalled)),
                  reset,
                  setPos,
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        70, 65, 1),
+                 lambda: self.fakeClick(70, 65),
                  lambda: self.assert_(not(self.__offscreenImageDownCalled)),
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        120, 65, 1),
+                 lambda: self.fakeClick(120, 65),
                  lambda: self.assert_(not(self.__offscreenImageDownCalled)),
                  reset,
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        110, 65, 1),
+                 lambda: self.fakeClick(110, 65),
                  lambda: self.assert_(self.__offscreenImageDownCalled and 
                         self.__mainDownCalled),
                  reset,
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        1, 1, 1),
+                 lambda: self.fakeClick(1, 1),
                  lambda: self.assert_(not(self.__offscreenImageDownCalled) and 
                         self.__mainDownCalled),
                 ))
@@ -220,18 +211,18 @@ class OffscreenTestCase(AVGTestCase):
         helper = Player.getTestHelper()
         self.__offscreenImageDownCalled = False
         offscreenImage.setEventCapture()
-        self.start(None,
-                (lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
+        self.start((
+                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
                         80, 10, 1),
                  lambda: self.assert_(self.__offscreenImageDownCalled),
                 ))
                 
     def testCanvasRender(self):
         def createCanvas():
-            return Player.loadCanvasString("""
-                <canvas id="testcanvas" width="160" height="120" autorender="False">
-                    <image id="test" href="rgb24-65x65.png"/>
-                </canvas>""")
+            canvas = Player.createCanvas(id="testcanvas", size=(160,120),
+                    autorender=False)
+            avg.ImageNode(id="test", href="rgb24-65x65.png", parent=canvas.getRootNode())
+            return canvas
 
         def testEarlyScreenshotException():
             self.assertException(self.__offscreenCanvas.screenshot)
@@ -248,11 +239,11 @@ class OffscreenTestCase(AVGTestCase):
         def recreateCanvas():
             self.__offscreenCanvas = createCanvas()
 
-        mainCanvas = self.loadEmptyScene()
+        self.loadEmptyScene()
         self.__offscreenCanvas = createCanvas()
         self.assertException(renderCanvas)
-        self.start(None,
-                (testEarlyScreenshotException,
+        self.start((
+                 testEarlyScreenshotException,
                  renderCanvas,
                  deleteCanvas,
                  recreateCanvas,
@@ -262,7 +253,7 @@ class OffscreenTestCase(AVGTestCase):
     def testCanvasAutoRender(self):
         def createCanvas():
             canvas = self.__createOffscreenCanvas("testcanvas", False)
-            avg.ImageNode(href="canvas:testcanvas", parent=Player.getRootNode())
+            avg.ImageNode(href="canvas:testcanvas", parent=root)
             return canvas
 
         def disableAutoRender():
@@ -274,10 +265,10 @@ class OffscreenTestCase(AVGTestCase):
         def changeContent():
             self.__offscreenCanvas.getElementByID("test1").x = 42
 
-        mainCanvas = self.loadEmptyScene()
+        root = self.loadEmptyScene()
         self.__offscreenCanvas = createCanvas()
-        self.start(None,
-                (lambda: self.assert_(self.__offscreenCanvas.autorender),
+        self.start((
+                 lambda: self.assert_(self.__offscreenCanvas.autorender),
                  lambda: self.compareImage("testOffscreenAutoRender1", False),
                  disableAutoRender,
                  lambda: self.assert_(not(self.__offscreenCanvas.autorender)),
@@ -289,47 +280,33 @@ class OffscreenTestCase(AVGTestCase):
                 ))
 
     def testCanvasCrop(self):
-        mainCanvas = self.loadEmptyScene()
-        canvas = Player.loadCanvasString("""
-            <canvas id="testcanvas" width="160" height="120">
-                <div pos="(40, 30)" size="(80, 60)" crop="True">
-                    <image id="test1" pos="(-32, -32)" href="rgb24-65x65.png"/>
-                </div>
-            </canvas>
-        """)
-        node = avg.ImageNode(parent=Player.getRootNode(), 
-                href="canvas:testcanvas")
-        self.start(None,
-                (lambda: self.compareImage("testCanvasCrop", False),
-                ))
+        root = self.loadEmptyScene()
+        canvas = Player.createCanvas(id="testcanvas", size=(160,120))
+        div = avg.DivNode(pos=(40,30), size=(80,60), crop=True, 
+                parent=canvas.getRootNode())
+        avg.ImageNode(id="test1", pos=(-32, -32), href="rgb24-65x65.png", parent=div)
+        avg.ImageNode(parent=root, href="canvas:testcanvas")
+        self.start((lambda: self.compareImage("testCanvasCrop", False),))
 
     def testCanvasAlpha(self):
-        mainCanvas = self.loadEmptyScene()
-        canvas = Player.loadCanvasString("""
-            <canvas id="testcanvas" width="80" height="120">
-                <image id="test1" href="rgb24alpha-64x64.png"/>
-            </canvas>
-        """)
-        avg.RectNode(parent=Player.getRootNode(), fillcolor="FFFFFF",
+        root = self.loadEmptyScene()
+        canvas = Player.createCanvas(id="testcanvas", size=(80,120))
+        avg.ImageNode(id="test1", href="rgb24alpha-64x64.png", 
+                parent=canvas.getRootNode())
+        avg.RectNode(parent=root, fillcolor="FFFFFF",
                 pos=(0.5, 0.5), size=(160, 48), fillopacity=1)
-        node = avg.ImageNode(parent=Player.getRootNode(), 
-                href="canvas:testcanvas")
-        avg.ImageNode(parent=Player.getRootNode(), x=64, href="rgb24alpha-64x64.png")
-        self.start(None,
-                (lambda: self.compareImage("testCanvasAlpha", False),
-                ))
+        avg.ImageNode(parent=root, href="canvas:testcanvas")
+        avg.ImageNode(parent=root, x=64, href="rgb24alpha-64x64.png")
+        self.start((lambda: self.compareImage("testCanvasAlpha", False),))
     
     def testCanvasBlendModes(self):
         def createBaseCanvas():
-            return Player.loadCanvasString("""
-                <canvas id="testcanvas" width="64" height="64">
-                    <image x="0" y="0" href="rgb24alpha-64x64.png"/>
-                </canvas>
-            """)
+            canvas = Player.createCanvas(id="testcanvas", size=(64,64))
+            avg.ImageNode(href="rgb24alpha-64x64.png", parent=canvas.getRootNode())
+            return canvas
        
-        mainCanvas = self.loadEmptyScene()
-        canvas = createBaseCanvas()
-        root = Player.getRootNode()
+        root = self.loadEmptyScene()
+        createBaseCanvas()
         avg.RectNode(parent=root, pos=(48,0), size=(32, 120), strokewidth=2, 
                 fillopacity=1, fillcolor="808080")
         avg.ImageNode(parent=root, href="canvas:testcanvas")
@@ -339,15 +316,12 @@ class OffscreenTestCase(AVGTestCase):
                 blendmode="add")
         avg.ImageNode(parent=root, pos=(64,64), href="canvas:testcanvas", 
                 opacity=0.6, blendmode="add")
-        self.start(None,
-                (lambda: self.compareImage("testCanvasBlendModes", False),
-                ))
+        self.start((lambda: self.compareImage("testCanvasBlendModes", False),))
 
     def testCanvasMultisampling(self):
         def testIllegalSamples():
-            self.canvas = Player.loadCanvasString(
-                    """<canvas id="brokencanvas" width="160" height="120" 
-                            multisamplesamples="42"/>""")
+            self.canvas = Player.createCanvas(id="brokencanvas", size=(160,120), 
+                            multisamplesamples=42)
 
         def screenshot():
             bmp = self.canvas.screenshot()
@@ -355,30 +329,25 @@ class OffscreenTestCase(AVGTestCase):
 
         def createCanvas():
             if not(avg.OffscreenCanvas.isMultisampleSupported()):
-                print
-                print "Offscreen multisampling not supported - skipping test."
+                self.skip("Offscreen multisampling not supported")
                 Player.stop()
                 return
             try:
-                self.canvas = Player.loadCanvasString("""
-                    <canvas id="testcanvas" width="160" height="120" 
-                            multisamplesamples="2">
-                        <image id="test1" href="rgb24-65x65.png" angle="0.1"/>
-                    </canvas>
-                """)
+                self.canvas = Player.createCanvas(id="testcanvas", size=(160,120),
+                        multisamplesamples=2)
+                avg.ImageNode(id="test1", href="rgb24-65x65.png", angle=0.1,
+                        parent=self.canvas.getRootNode())
             except RuntimeError:
-                print
-                print "Offscreen multisampling init failed - skipping test."
+                self.skip("Offscreen multisampling init failed")
                 Player.stop()
                 return
-            self.assert_(self.canvas.multisamplesamples == 2)
-            node = avg.ImageNode(parent=Player.getRootNode(), 
-                    href="canvas:testcanvas")
+            self.assertEqual(self.canvas.multisamplesamples, 2)
+            avg.ImageNode(parent=root, href="canvas:testcanvas")
             
 
-        mainCanvas = self.loadEmptyScene()
-        self.start(None,
-                (createCanvas,
+        root = self.loadEmptyScene()
+        self.start((
+                 createCanvas,
                  lambda: self.compareImage("testCanvasMultisample", False),
                  screenshot,
                  lambda: self.assertException(testIllegalSamples),
@@ -386,33 +355,33 @@ class OffscreenTestCase(AVGTestCase):
         self.canvas = None
        
     def testCanvasMipmap(self):
-        mainCanvas = self.loadEmptyScene()
+        root = self.loadEmptyScene()
 
-        canvas = Player.loadCanvasString("""
-            <canvas id="testcanvas" width="80" height="120" mipmap="True">
-                <image id="test1" href="rgb24alpha-64x64.png"/>
-            </canvas>
-        """)
-        node = avg.ImageNode(parent=Player.getRootNode(), size=(40, 30), 
-                href="canvas:testcanvas")
+        canvas = Player.createCanvas(id="testcanvas", size=(80,120), mipmap=True)
+        avg.ImageNode(id="test1", href="rgb24alpha-64x64.png", 
+                parent=canvas.getRootNode())
+        avg.ImageNode(parent=root, size=(40, 30), href="canvas:testcanvas")
         try:
-            self.start(None,
-                    (lambda: self.compareImage("testCanvasMipmap", False),
-                    ))
+            self.start((lambda: self.compareImage("testCanvasMipmap", False),))
         except RuntimeError:
-                print
-                print "Offscreen mipmap init failed - skipping test."
-                Player.stop()
-                return
+            self.skip("Offscreen mipmap init failed.")
+            return
 
     def testCanvasDependencies(self):
         def makeCircularRef():
             self.offscreen1.getElementByID("test1").href = "canvas:offscreencanvas2"
             
+        def makeSelfRef1():
+            avg.ImageNode(href="canvas:offscreencanvas1", 
+                    parent=self.offscreen1.getRootNode())
+
+        def makeSelfRef2():
+            self.offscreen1.getElementByID("test1").href = "canvas:offscreencanvas1"
+
         def createTwoCanvases():
             self.offscreen1 = self.__createOffscreenCanvas("offscreencanvas1", False)
             self.offscreen2 = self.__createOffscreenCanvas("offscreencanvas2", False)
-            self.node = avg.ImageNode(parent=Player.getRootNode(), 
+            self.node = avg.ImageNode(parent=root, 
                     href="canvas:offscreencanvas1")
             node = self.offscreen1.getElementByID("test1")
             node.href = "canvas:offscreencanvas2"
@@ -424,16 +393,13 @@ class OffscreenTestCase(AVGTestCase):
             self.node.href = "canvas:offscreencanvas2"
             
         def loadCanvasDepString():
-            Player.loadCanvasString('<canvas id="canvas1" size="(160, 120)"/>')
-            Player.loadCanvasString('''
-                <canvas id="canvas2" size="(160, 120)">
-                    <image href="canvas:canvas1"/>
-                </canvas>
-                ''')
+            Player.createCanvas(id="canvas1", size=(160, 120))
+            canvas2 = Player.createCanvas(id="canvas2", size=(160, 120))
+            avg.ImageNode(href="canvas:canvas1", parent=canvas2.getRootNode())
             Player.deleteCanvas('canvas2')
             Player.deleteCanvas('canvas1')
 
-        mainCanvas = self.loadEmptyScene()
+        root = self.loadEmptyScene()
         createTwoCanvases()
         self.offscreen1.getElementByID("test1").href = ""
         self.offscreen1 = None
@@ -442,28 +408,29 @@ class OffscreenTestCase(AVGTestCase):
         self.node = None
         Player.deleteCanvas("offscreencanvas1")
         Player.deleteCanvas("offscreencanvas2")
-        self.start(None,
-                   (createTwoCanvases,
+        self.start((
+                    createTwoCanvases,
                     lambda: self.compareImage("testCanvasDependencies1", False),
                     exchangeCanvases,
                     lambda: self.compareImage("testCanvasDependencies2", False),
                     lambda: self.assertException(makeCircularRef),
+                    lambda: self.assertException(makeSelfRef1),
+                    lambda: self.assertException(makeSelfRef2),
                     loadCanvasDepString,
                   ))
 
     def __setupCanvas(self, handleEvents):
-        mainCanvas = self.loadEmptyScene()
+        root = self.loadEmptyScene()
+        mainCanvas = Player.getMainCanvas()
         offscreenCanvas = self.__createOffscreenCanvas("offscreencanvas", handleEvents)
-        self.node = avg.ImageNode(parent=Player.getRootNode(), 
-                href="canvas:offscreencanvas")
+        self.node = avg.ImageNode(parent=root, href="canvas:offscreencanvas")
         return (mainCanvas, offscreenCanvas)
 
     def __createOffscreenCanvas(self, canvasName, handleEvents):
-        return Player.loadCanvasString("""
-            <canvas id="%s" width="160" height="120" handleevents="%s">
-                <image id="test1" href="rgb24-65x65.png"/>
-            </canvas>
-        """%(canvasName, str(handleEvents)))
+        canvas=Player.createCanvas(id=canvasName, size=(160,120), 
+                handleevents=handleEvents)
+        avg.ImageNode(id="test1", href="rgb24-65x65.png", parent=canvas.getRootNode())
+        return canvas
 
 def isOffscreenSupported():
     
