@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2008 Ulrich von Zadow
+# Copyright (C) 2003-2011 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -19,80 +19,64 @@
 # Current versions can be found at www.libavg.de
 #
 
-import unittest
-
 import time
 import math
 
 from libavg import avg
 from testcase import *
 
-def almostEqual(a,b):
-    try:
-        bOk = True
-        for i in range(len(a)):
-            if math.fabs(a[i]-b[i]) > 0.000001:
-                bOk = False
-        return bOk
-    except:
-        return math.fabs(a-b) < 0.000001
-
-
 class PlayerTestCase(AVGTestCase):
     def __init__(self, testFuncName):
         AVGTestCase.__init__(self, testFuncName)
 
     def testPoint(self):
-        def almostEqual(p1, p2):
-            return (abs(p1.x-p2.x) < 0.0001 and abs(p1.y-p2.y) < 0.0001)
-
         def testHash():
             ptMap = {}
             ptMap[avg.Point2D(0,0)] = 0
             ptMap[avg.Point2D(1,0)] = 1
             ptMap[avg.Point2D(0,0)] = 2
-            self.assert_(len(ptMap) == 2)
-            self.assert_(ptMap[avg.Point2D(0,0)] == 2)
+            self.assertEqual(len(ptMap), 2)
+            self.assertEqual(ptMap[avg.Point2D(0,0)], 2)
 
         pt = avg.Point2D()
-        self.assert_(pt == avg.Point2D(0,0))
+        self.assertEqual(pt, avg.Point2D(0,0))
         pt = avg.Point2D(10, 10)
-        self.assert_(pt[0]==pt.x)
-        self.assert_(pt[1]==pt.y)
-        self.assert_(pt == avg.Point2D(10, 10))
-        self.assert_(pt == (10, 10))
-        self.assert_(pt == avg.Point2D([10, 10]))
-        self.assert_(pt != avg.Point2D(11, 10))
-        self.assert_(str(pt) == "(10,10)")
+        self.assertEqual(pt[0], pt.x)
+        self.assertEqual(pt[1], pt.y)
+        self.assertEqual(pt, avg.Point2D(10, 10))
+        self.assertEqual(pt, (10, 10))
+        self.assertEqual(pt, avg.Point2D([10, 10]))
+        self.assertNotEqual(pt, avg.Point2D(11, 10))
+        self.assertEqual(str(pt), "(10,10)")
         pt2 = eval(repr(pt))
-        self.assert_(pt2 == pt)
+        self.assertEqual(pt2, pt)
         testHash()
-        self.assert_(almostEqual(avg.Point2D(10,0).getNormalized(), avg.Point2D(1,0)))
-        self.assert_(almostEqual(pt.getRotated(math.pi, (5,5)), avg.Point2D(0,0)))
-        self.assert_(-pt == (-10, -10))
-        self.assert_(pt-(10,0) == (0,10))
-        self.assert_(pt+(10,0) == (20,10))
-        self.assert_(pt*2 == (20,20))
-        self.assert_(2*pt == (20,20))
+        self.assertAlmostEqual(avg.Point2D(10,0).getNormalized(), avg.Point2D(1,0))
+        self.assertAlmostEqual(pt.getRotated(math.pi, (5,5)), avg.Point2D(0,0))
+        self.assertEqual(-pt, (-10, -10))
+        self.assertEqual(pt-(10,0), (0,10))
+        self.assertEqual(pt+(10,0), (20,10))
+        self.assertEqual(pt*2, (20,20))
+        self.assertEqual(2*pt, (20,20))
         pt.x = 21
         pt.y = 23
-        self.assert_(pt == avg.Point2D(21, 23))
+        self.assertEqual(pt, avg.Point2D(21, 23))
         pt.x -= 11
         pt.y -= 13
         pt += avg.Point2D(10, 10)
-        self.assert_(pt == avg.Point2D(20, 20))
+        self.assertEqual(pt, avg.Point2D(20, 20))
         pt -= avg.Point2D(6, 6)
-        self.assert_(pt == avg.Point2D(14, 14))
-        self.assert_(pt != avg.Point2D(13, 13))
+        self.assertEqual(pt, avg.Point2D(14, 14))
+        self.assertNotEqual(pt, avg.Point2D(13, 13))
         pt = pt/2.
-        self.assert_(pt == avg.Point2D(7, 7))
+        self.assertEqual(pt, avg.Point2D(7, 7))
         pt = avg.Point2D((10, 10))
-        self.assert_(pt == (10, 10))
-        self.assert_(len(pt) == 2)
-        self.assert_(pt[0]==pt.x)
-        self.assert_(pt[1]==pt.y)
+        self.assertEqual(pt, (10, 10))
+        self.assertEqual(len(pt), 2)
+        self.assertEqual(pt[0], pt.x)
+        self.assertEqual(pt[1], pt.y)
         self.assertException(lambda: pt[2])
-        self.assert_(almostEqual(avg.Point2D(10,0), avg.Point2D.fromPolar(0,10)))
+        self.assertAlmostEqual(avg.Point2D(10,0), avg.Point2D.fromPolar(0,10))
         self.assertException(avg.Point2D(0,0).getNormalized)
         self.assertException(lambda: avg.Point2D(0,))
         self.assertException(lambda: avg.Point2D(0,1,2))
@@ -100,33 +84,47 @@ class PlayerTestCase(AVGTestCase):
             pt = avg.Point2D(point)
             angle = pt.getAngle()
             norm = pt.getNorm()
-            self.assert_(almostEqual(pt, avg.Point2D.fromPolar(angle,norm)))
+            self.assertAlmostEqual(pt, avg.Point2D.fromPolar(angle,norm))
 
     def testBasics(self):
         def getFramerate():
             framerate = Player.getEffectiveFramerate()
             self.assert_(framerate > 0)
 
+        def invalidCreateNode():
+            avg.ImageNode(1, 2, 3)
+
         Player.showCursor(0)
         Player.showCursor(1)
-        Player.loadString("""
-            <avg width="160" height="120">
-                <image id="test1" href="rgb24-65x65.png"/>
-            </avg>
-        """)
-        self.start(None,
-                 (getFramerate,
+        root = self.loadEmptyScene()
+        node = Player.createNode("""<image id="test1" href="rgb24-65x65.png"/>""")
+        root.appendChild(node)
+        self.assertException(invalidCreateNode)
+        self.start((
+                  getFramerate,
                   lambda: self.compareImage("testbasics", False), 
                   lambda: Player.setGamma(0.3, 0.3, 0.3),
                   lambda: Player.showCursor(0),
                   lambda: Player.showCursor(1),
                  ))
 
+    def testFakeTime(self):
+        def checkTime():
+            self.assertEqual(Player.getFrameTime(), 50)
+            self.assertEqual(Player.getFrameDuration(), 50)
+            self.assertEqual(Player.getEffectiveFramerate(), 20)
+
+        self.loadEmptyScene()
+        Player.setFakeFPS(20)
+        self.start((
+                 checkTime,
+                ))
+
     def testDivResize(self):
         def checkSize (w, h):
-            self.assert_(node.width == w)
-            self.assert_(node.height == h)
-            self.assert_(node.size == (w,h))
+            self.assertEqual(node.width, w)
+            self.assertEqual(node.height, h)
+            self.assertEqual(node.size, (w,h))
         
         def setSize (size):
             node.size = size
@@ -137,18 +135,18 @@ class PlayerTestCase(AVGTestCase):
         def setHeight (h):
             node.height = h
 
-        Player.loadFile("avg.avg")
+        self.__initDefaultScene()
         node = Player.getElementByID('nestedavg')
 
-        self.start(None,
-                (lambda: checkSize(128, 32),
-                    lambda: setSize((14,15)),
-                    lambda: checkSize(14,15),
-                    lambda: setWidth(23),
-                    lambda: checkSize(23,15),
-                    lambda: setHeight(22),
-                    lambda: checkSize(23,22),
-                    ))
+        self.start((
+                 lambda: checkSize(128, 32),
+                 lambda: setSize((14,15)),
+                 lambda: checkSize(14,15),
+                 lambda: setWidth(23),
+                 lambda: checkSize(23,15),
+                 lambda: setHeight(22),
+                 lambda: checkSize(23,22),
+                ))
 
     def testRotate(self):
         def onOuterDown(Event):
@@ -161,52 +159,61 @@ class PlayerTestCase(AVGTestCase):
         def testCoordConversions():
             innerNode = Player.getElementByID("inner")
             relPos = innerNode.getRelPos((90, 80))
-            self.assert_(almostEqual(relPos, (10, 10)))
+            self.assertAlmostEqual(relPos, (10, 10))
             outerNode = Player.getElementByID("outer")
             relPos = outerNode.getRelPos((90, 80))
-            self.assert_(almostEqual(relPos[0], 12.332806394528092) and
-                    almostEqual(relPos[1], 6.9211188716194592))
+            self.assertAlmostEqual(relPos[0], 12.332806394528092)
+            self.assertAlmostEqual(relPos[1], 6.9211188716194592)
             absPos = outerNode.getAbsPos(relPos)
-            self.assert_(almostEqual(absPos, (90, 80)))
-            self.assert_(outerNode.getElementByPos((10, 10)) == innerNode)
-            self.assert_(outerNode.getElementByPos((0, 10)) == outerNode)
-            self.assert_(outerNode.getElementByPos((-10, -110)) == None)
+            self.assertAlmostEqual(absPos, (90, 80))
+            self.assertEqual(outerNode.getElementByPos((10, 10)), innerNode)
+            self.assertEqual(outerNode.getElementByPos((0, 10)), outerNode)
+            self.assertEqual(outerNode.getElementByPos((-10, -110)), None)
         
-        def sendEvent(x, y):
+        def sendEvent(type, x, y):
             Helper = Player.getTestHelper()
-            Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
+            if type == avg.CURSORUP:
+                button = False
+            else:
+                button = True
+            Helper.fakeMouseEvent(type, button, False, False,
                         x, y, 1)
         
         def disableCrop():
             Player.getElementByID("outer").crop = False
             Player.getElementByID("inner").crop = False
-            
-        Player.loadFile("rotate.avg")
+           
+        self.__initDefaultRotateScene()
         Player.getElementByID("outer").setEventHandler(
                 avg.CURSORDOWN, avg.MOUSE, onOuterDown) 
         self.onOuterDownCalled = False
-        self.start(None,
-                (lambda: self.compareImage("testRotate1", False),
+        self.start((
+                 lambda: self.compareImage("testRotate1", False),
                  testCoordConversions,
                  fakeRotate,
                  lambda: self.compareImage("testRotate1a", False),
-                 lambda: sendEvent(85, 70),
+                 lambda: sendEvent(avg.CURSORDOWN, 85, 70),
+                 lambda: sendEvent(avg.CURSORUP, 85, 70),
                  lambda: self.assert_(not(self.onOuterDownCalled)),
-                 lambda: sendEvent(85, 75),
+                 lambda: sendEvent(avg.CURSORDOWN, 85, 75),
+                 lambda: sendEvent(avg.CURSORUP, 85, 75),
                  lambda: self.assert_(self.onOuterDownCalled),
                  disableCrop,
                  lambda: self.compareImage("testRotate1b", False),
                 ))
 
     def testRotate2(self):
-        self.start("rotate2.avg",
-                [lambda: self.compareImage("testRotate2", False),
-                 ])
+        root = self.loadEmptyScene()
+        
+        div1 = avg.DivNode(pos=(80,0), size=(160,120), pivot=(0,0), angle=1.57, 
+                parent=root)
+        avg.ImageNode(size=(16,16), href="rgb24-65x65.png", parent=div1)
+        div2 = avg.DivNode(pos=(40,0), size=(110,80), pivot=(0,0), angle=1.57,
+                crop=True, parent=div1)
+        avg.ImageNode(pos=(0,0), size=(16,16), href="rgb24-65x65.png", parent=div2)
+        avg.ImageNode(pos=(30,-6), size=(16,16), href="rgb24-65x65.png", parent=div2)
+        self.start([lambda: self.compareImage("testRotate2", False)])
         
-    def testRotate3(self):
-        self.start("rotate3.avg",
-                [lambda: self.compareImage("testRotate3", False)])
-
     def testRotatePivot(self):
         def setPivot (pos):
             node.pivot = pos
@@ -214,26 +221,30 @@ class PlayerTestCase(AVGTestCase):
         def addPivot (offset):
             node.pivot += offset
         
-        Player.loadFile("rotate3.avg")
-        node = Player.getElementByID('div1')
-        self.start(None, (
-            lambda: setPivot((10, 10)),
+        root = self.loadEmptyScene()
+        node = avg.DivNode(pos=(80,0), size=(160,120), pivot=(0,0), angle=1.57,
+                crop=True, parent=root)
+        div = avg.DivNode(pos=(40,-20), size=(160,120), pivot=(0,0), angle=0.79,
+                crop=True, parent=node)
+        avg.ImageNode(pos=(-10,-10), size=(128,128), href="rgb24-65x65.png", parent=div)
+        avg.ImageNode(pos=(0,10), size=(32,32), href="rgb24-65x65.png", parent=node)
+        self.start((
             lambda: self.compareImage("testRotatePivot1", False),
-            lambda: addPivot((-8, 0)),
+            lambda: setPivot((10, 10)),
             lambda: self.compareImage("testRotatePivot2", False),
+            lambda: addPivot((-8, 0)),
+            lambda: self.compareImage("testRotatePivot3", False),
            ))
 
     def testOutlines(self):
-        Player.loadFile("rotate.avg")
-        Player.getRootNode().elementoutlinecolor = "FFFFFF"
-        Player.getElementByID("inner").width = 10000
-        Player.getElementByID("inner").height = 10000
-        self.start(None, 
-                [lambda: self.compareImage("testOutlines", False)
-                ])
+        root = self.__initDefaultRotateScene()
+        root.elementoutlinecolor = "FFFFFF"
+        Player.getElementByID("inner").width = 100000
+        Player.getElementByID("inner").height = 100000
+        self.start([lambda: self.compareImage("testOutlines", False)])
 
     def testError(self):
-        Player.loadFile("image.avg")
+        self.initDefaultImageScene()
         Player.setTimeout(1, lambda: undefinedFunction)
         Player.setTimeout(50, Player.stop)
         try:
@@ -248,8 +259,8 @@ class PlayerTestCase(AVGTestCase):
             raise ZeroDivisionError
         
         try:
-            self.start("image.avg",
-                    [throwException])
+            self.initDefaultImageScene()
+            self.start([throwException])
         except ZeroDivisionError:
             self.assert_(1)
         else:
@@ -257,18 +268,22 @@ class PlayerTestCase(AVGTestCase):
 
     def testInvalidImageFilename(self):
         def activateNode():
-            Player.getElementByID("enclosingdiv").active = 1
+            div.active = 1
         
-        self.start("invalidfilename.avg",
-                [activateNode])
+        root = self.loadEmptyScene()
+        div = avg.DivNode(active=False, parent=root)
+        avg.ImageNode(href="filedoesntexist.png", parent=div)
+        self.start([activateNode])
 
     def testInvalidVideoFilename(self):
         def tryplay():
-            assertException(lambda: Player.getElementByID("brokenvideo").play())
+            assertException(lambda: video.play())
         
-        self.start("invalidvideofilename.avg",
-                (lambda: tryplay,
-                 lambda: Player.getElementByID("brokenvideo").stop()
+        root = self.loadEmptyScene()
+        video = avg.VideoNode(href="filedoesntexist.avi", parent=root)
+        self.start((
+                 lambda: tryplay,
+                 lambda: video.stop()
                 ))
 
     def testTimeouts(self):
@@ -300,8 +315,9 @@ class PlayerTestCase(AVGTestCase):
         self.timeout2called = False
         self.__exceptionThrown = False
         try:
-            self.start("image.avg",
-                    (setupTimeouts,
+            self.initDefaultImageScene()
+            self.start((
+                     setupTimeouts,
                      wait,
                      lambda: self.assert_(self.timeout1called),
                      lambda: self.assert_(not(self.timeout2called)),
@@ -327,21 +343,43 @@ class PlayerTestCase(AVGTestCase):
         def loadImage():
             node = Player.getElementByID("pano")
             node.href = "rgb24-65x65.png"
-        self.start("panoimage.avg",
-                (lambda: self.compareImage("testPanoImage", False),
+
+        root = self.loadEmptyScene()
+        avg.ImageNode(size=(320,240), href="rgb24-65x65.png", parent=root)
+        avg.PanoImageNode(id="pano", size=(160,120), href="panoimage.png",
+                sensorwidth=4.60, sensorheight=3.97, focallength=12, parent=root)
+        avg.ImageNode(pos=(120,0), size=(40,40), href="rgb24-65x65.png", parent=root)
+        self.start((
+                 lambda: self.compareImage("testPanoImage", False),
                  lambda: time.sleep,
                  changeProperties,
                  loadImage
                 ))
 
+    def testAVGFile(self):
+        Player.loadFile("image.avg")
+        self.start((
+                 lambda: self.compareImage("testAVGFile", False),
+                ))
+        self.assertException(lambda: Player.loadFile("filedoesntexist.avg"))
+
     def testBroken(self):
-        def testBrokenFile(filename):
-            self.assertException(lambda: Player.loadFile(filename))
+        def testBrokenString(string):
+            self.assertException(lambda: Player.loadString(string))
         
-        testBrokenFile("filedoesntexist.avg")
-        testBrokenFile("noxml.avg")
-        testBrokenFile("noavg.avg")
-        testBrokenFile("noavg2.avg")
+        # This isn't xml
+        testBrokenString("""
+            xxx<avg width="400" height="300">
+            </avg>
+        """)
+        # This isn't avg
+        testBrokenString("""
+            <bla>hallo
+            </bla>""")
+        testBrokenString("""
+            <avg width="640" height="480" invalidattribute="bla">
+            </avg>
+        """)
 
     def testMove(self):
         def moveit():
@@ -353,10 +391,11 @@ class PlayerTestCase(AVGTestCase):
         
         def checkRelPos():
             RelPos = Player.getElementByID("obscured").getRelPos((50,52))
-            self.assert_(RelPos == (0, 0))
-        
-        self.start("avg.avg",
-                (lambda: self.compareImage("testMove1", False),
+            self.assertEqual(RelPos, (0, 0))
+      
+        self.__initDefaultScene()
+        self.start((
+                 lambda: self.compareImage("testMove1", False),
                  moveit,
                  checkRelPos
                 ))
@@ -389,8 +428,21 @@ class PlayerTestCase(AVGTestCase):
             Player.getElementByID("nestedavg").angle = 1.0
             Player.getElementByID("bkgd").angle = 1.0
         
-        self.start("crop2.avg",
-                (lambda: self.compareImage("testCropImage1", False),
+        root = self.loadEmptyScene()
+        avg.ImageNode(id="bkgd", href="crop_bkgd.png", parent=root)
+        root.appendChild(
+                Player.createNode("""
+                  <div id="nestedavg" x="40" y="30" width="80" height="60" crop="True">
+                    <div id="nestedavg2" crop="True">
+                      <div id="nestedavg3" crop="True">
+                        <image id="img" x="10" y="10" width="40" height="40" 
+                                href="rgb24-64x64.png"/>
+                      </div>
+                    </div>
+                  </div>
+                """))
+        self.start((
+                 lambda: self.compareImage("testCropImage1", False),
                  moveTLCrop,
                  lambda: self.compareImage("testCropImage2", False),
                  moveBRCrop,
@@ -445,8 +497,18 @@ class PlayerTestCase(AVGTestCase):
             Player.getElementByID("bkgd").angle = 1.0
         
         Player.setFakeFPS(30)
-        self.start("crop.avg",
-                (playMovie,
+        root = self.loadEmptyScene()
+        avg.ImageNode(id="bkgd", href="crop_bkgd.png", parent=root)
+        root.appendChild(
+                Player.createNode("""
+                  <div id="nestedavg" x="40" y="30" width="80" height="60" crop="True">
+                    <video id="movie" x="10" y="10" width="40" height="40" 
+                            threaded="false" href="../video/testfiles/mpeg1-48x48.mpg" 
+                            fps="30"/>
+                  </div>
+                """))
+        self.start((
+                 playMovie,
                  lambda: self.compareImage("testCropMovie1", False),
                  moveTLCrop,
                  lambda: self.compareImage("testCropMovie2", False),
@@ -471,29 +533,31 @@ class PlayerTestCase(AVGTestCase):
 
     def testWarp(self):
         def moveVertex():
-            node = Player.getElementByID("testtiles")
-            grid = node.getWarpedVertexCoords()
+            grid = image.getWarpedVertexCoords()
             grid[1][1] = (grid[1][1][0]+0.06, grid[1][1][1]+0.06)
-            node.setWarpedVertexCoords(grid)
-            node = Player.getElementByID("clogo1")
-            grid = node.getWarpedVertexCoords()
+            image.setWarpedVertexCoords(grid)
+            grid = video.getWarpedVertexCoords()
             grid[0][0] = (grid[0][0][0]+0.06, grid[0][0][1]+0.06)
             grid[1][1] = (grid[1][1][0]-0.06, grid[1][1][1]-0.06)
-            node.setWarpedVertexCoords(grid)
+            video.setWarpedVertexCoords(grid)
         
         def flip():
-            node = Player.getElementByID("testtiles")
-            grid = node.getOrigVertexCoords()
+            grid = image.getOrigVertexCoords()
             grid = [ [ (1-pos[0], pos[1]) for pos in line ] for line in grid]
-            node.setWarpedVertexCoords(grid)
-        
-        Player.loadFile("video.avg")
-        node = Player.getElementByID("testtiles")
-        self.assertException(node.getOrigVertexCoords)
-        self.assertException(node.getWarpedVertexCoords)
+            image.setWarpedVertexCoords(grid)
+       
+        root = self.loadEmptyScene()
+        image = avg.ImageNode(href="rgb24-64x64.png",
+                maxtilewidth=32, maxtileheight=16, parent=root)
+        video = avg.VideoNode(pos=(40,0), size=(80,80), opacity=0.5, loop=True,
+                href="../video/testfiles/mpeg1-48x48.mpg", threaded=False, fps=30,
+                parent=root)
+
+        self.assertException(image.getOrigVertexCoords)
+        self.assertException(image.getWarpedVertexCoords)
         Player.setFakeFPS(30)
-        self.start(None,
-                (lambda: Player.getElementByID("clogo1").play(),
+        self.start((
+                 lambda: video.play(),
                  lambda: self.compareImage("testWarp1", False),
                  moveVertex,
                  lambda: self.compareImage("testWarp2", False),
@@ -504,47 +568,51 @@ class PlayerTestCase(AVGTestCase):
     def testMediaDir(self):
         def createImageNode():
             # Node is not in tree; mediadir should be root node dir.
-            node = avg.ImageNode(href="rgb24-64x64.png")
-            self.assert_(node.size == avg.Point2D(0,0)) # File not found
-            node.href = "rgb24-64x64a.png"
-            self.assert_(node.size == avg.Point2D(64,64)) # File found
-            node = avg.ImageNode(href="rgb24-64x64.png", width=23, height=42)
+            node = avg.ImageNode(href="rgb24-64x64a.png")
+            self.assertEqual(node.size, avg.Point2D(0,0)) # File not found
+            node.href = "rgb24-64x64.png"
+            self.assertEqual(node.size, avg.Point2D(64,64)) # File found
+            node = avg.ImageNode(href="rgb24-64x64a.png", width=23, height=42)
             # File not found, but custom size
-            self.assert_(node.size == avg.Point2D(23,42))
-            node.href = "rgb24-64x64a.png"
+            self.assertEqual(node.size, avg.Point2D(23,42))
+            node.href = "rgb24-64x64.png"
             # File found, custom size stays
-            self.assert_(node.size == avg.Point2D(23,42))
+            self.assertEqual(node.size, avg.Point2D(23,42))
             node.size = (0,0)
             # File found, custom size cleared. Media size should be used.
-            self.assert_(node.size == avg.Point2D(64,64))
+            self.assertEqual(node.size, avg.Point2D(64,64))
 
         def setDir():
-            Player.getElementByID("main").mediadir="../video/testfiles"
+            div.mediadir="../video/testfiles"
         
         def setAbsDir():
             def absDir():
                 # Should not find any media here...
-                Player.getElementByID("main").mediadir="/testmediadir"
+                div.mediadir="/testmediadir"
 
             self.assertException(absDir)
         
         def createNode():
             node = avg.VideoNode(href="mjpeg1-48x48.avi", fps=30)
-        
-        self.start("mediadir.avg",
-                (createImageNode,
-                 lambda: Player.getElementByID("video").play(),
+
+        root = self.loadEmptyScene()
+        div = avg.DivNode(mediadir="testmediadir", parent=root)
+        image = avg.ImageNode(pos=(0,30), href="rgb24-64x64a.png", parent=div)
+        video = avg.VideoNode(href="mjpeg-48x48.avi", threaded=False, parent=div)
+        self.start((
+                 createImageNode,
+                 lambda: video.play(),
                  lambda: self.compareImage("testMediaDir1", False),
                  setDir,
-                 lambda: Player.getElementByID("video").play(), 
+                 lambda: video.play(), 
                  lambda: self.compareImage("testMediaDir2", False),
-                 lambda: self.assert_(Player.getElementByID("img").width == 0),
+                 lambda: self.assertEqual(image.width, 0),
                  createNode,
                  setAbsDir
                 ))
 
     def testMemoryQuery(self):
-        self.assert_(avg.getMemoryUsage() != 0)
+        self.assertNotEqual(avg.getMemoryUsage(), 0)
 
     def testStopOnEscape(self):
         def pressEscape():
@@ -572,14 +640,17 @@ class PlayerTestCase(AVGTestCase):
             self.testStopOnEscapeAlive = True
 
         self.testStopOnEscapeAlive = False
-        self.start('avg.avg',
-                (testEscape1,
-                testEscape2,
-                setAlive))
+        self.__initDefaultScene()
+        self.start((
+                 testEscape1,
+                 testEscape2,
+                 setAlive
+                ))
         self.assert_(self.testStopOnEscapeAlive)
-        self.start('avg.avg',
-                (testEscape3, # this should exit the player
-                lambda: self.assert_(False),
+        self.__initDefaultScene()
+        self.start((
+                 testEscape3, # this should exit the player
+                 lambda: self.fail(),
                 ))
 
     # Not executed due to bug #145 - hangs with some window managers.
@@ -588,21 +659,90 @@ class PlayerTestCase(AVGTestCase):
             Player.setWindowFrame(True)
 
         Player.setWindowFrame(False)
-        self.start('avg.avg',[revertWindowFrame])
+        self.__initDefaultScene()
+        self.start([revertWindowFrame])
 
-    def testGetScreenResolution(self):
+    def testScreenDimensions(self):
         res = Player.getScreenResolution()
         self.assert_(res.x > 0 and res.y > 0 and res.x < 10000 and res.y < 10000)
+        ppmm = Player.getPixelsPerMM()
+        self.assert_(ppmm > 0 and ppmm < 10000)
+        mm = Player.getPhysicalScreenDimensions()
+        self.assert_(mm.x > 0 and mm.y > 0 and mm.x < 10000 and mm.y < 10000)
+        print res, ppmm, mm
+        Player.assumePixelsPerMM(ppmm)
+        newPPMM = Player.getPixelsPerMM()
+        newMM = Player.getPhysicalScreenDimensions()
+        self.assertAlmostEqual(newPPMM, ppmm)
+        self.assertEqual(newMM, mm)
 
+    def testSVG(self):
+        svgFile = avg.SVG("rect.svg", False)
+
+        # renderElement
+        bmp = svgFile.renderElement("rect")
+        self.compareBitmapToFile(bmp, "testSvgBmp", False)
+        self.assertEqual(svgFile.getElementSize("rect"), avg.Point2D(22,12))
+        bmp = svgFile.renderElement("pos_rect")
+        self.compareBitmapToFile(bmp, "testSvgPosBmp", False)
+        bmp = svgFile.renderElement("rect", 5)
+        self.compareBitmapToFile(bmp, "testSvgScaleBmp1", False)
+        bmp = svgFile.renderElement("rect", (20,20))
+        self.compareBitmapToFile(bmp, "testSvgScaleBmp2", False)
+
+        # error handling
+        self.assertException(lambda: avg.SVG("filedoesntexist.svg", False))
+        self.assertException(lambda: svgFile.renderElement("missing_id"))
+
+        # unescapeIllustratorIDs
+        svgIllustratorFile = avg.SVG("illustratorRect.svg", True)
+        svgIllustratorFile.getElementSize("pos_rect")
+
+        # createImageNode
+        root = self.loadEmptyScene()
+        self.start((
+                 lambda: svgFile.createImageNode("rect", {"pos":(10,10), "parent":root}),
+                 lambda: self.compareImage("testSvgNode", False),
+                 lambda: svgFile.createImageNode("rect", {"pos":(5,5), "parent":root},
+                        5),
+                 lambda: self.compareImage("testSvgScaledNode1", False),
+                 lambda: svgFile.createImageNode("rect", {"pos":(1,1), "parent":root},
+                        (40,40)),
+                 lambda: self.compareImage("testSvgScaledNode2", False)
+                ))
+
+    def __initDefaultScene(self):
+        root = self.loadEmptyScene()
+        avg.ImageNode(id="mainimg", size=(100, 75), href="rgb24-65x65.png", parent=root)
+        div = avg.DivNode(id="nestedavg", pos=(0,32), opacity=1, size=(128, 32),
+                crop=True, parent=root)
+        avg.ImageNode(id="obscured", pos=(0,20), size=(96,40), href="rgb24-65x65.png",
+                parent=div)
+        avg.ImageNode(id="nestedimg1", size=(96,48), href="rgb24-65x65.png",
+                parent=div)
+        avg.ImageNode(id="nestedimg2", pos=(65,0), href="rgb24alpha-64x64.png",
+                parent=div)
+
+    def __initDefaultRotateScene(self):
+        root = self.loadEmptyScene()
+        div = avg.DivNode(pos=(80,10), size=(80,60), pivot=(0,0), angle=0.274,
+                crop=True, parent=root)
+        avg.ImageNode(pos=(10,10), size=(32,32), href="rgb24-65x65.png", parent=div)
+        outerDiv = avg.DivNode(id="outer", pos=(80,70), size=(80,60),
+                pivot=(0,0), angle=0.274, crop=True, parent=root)
+        innerDiv = avg.DivNode(id="inner", size=(80,60), pivot=(0,0), angle=-0.274,
+                crop=True, parent=outerDiv)
+        avg.ImageNode(pos=(10,10), size=(32,32), href="rgb24-65x65.png", parent=innerDiv)
+        return root
 
 def playerTestSuite(tests):
     availableTests = (
             "testPoint",
             "testBasics",
+            "testFakeTime",
             "testDivResize",
             "testRotate",
             "testRotate2",
-            "testRotate3",
             "testRotatePivot",
             "testOutlines",
             "testError",
@@ -611,6 +751,7 @@ def playerTestSuite(tests):
             "testInvalidVideoFilename",
             "testTimeouts",
             "testPanoImage",
+            "testAVGFile",
             "testBroken",
             "testMove",
             "testCropImage",
@@ -619,7 +760,8 @@ def playerTestSuite(tests):
             "testMediaDir",
             "testMemoryQuery",
             "testStopOnEscape",
-            "testGetScreenResolution",
+            "testScreenDimensions",
+            "testSVG",
 #            "testWindowFrame",
             )
     return createAVGTestSuite(availableTests, PlayerTestCase, tests)
