@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2008 Ulrich von Zadow
+# Copyright (C) 2003-2011 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,8 +20,6 @@
 # Current versions can be found at www.libavg.de
 #
 
-import unittest
-
 import platform
 
 from libavg import avg
@@ -33,9 +31,7 @@ class PluginTestCase(AVGTestCase):
 
     def testColorNodePlugin(self):
         def loadPlugin():
-            if platform.system() == 'Windows':
-                Player.loadPlugin("ColorNode")
-            else:
+            if platform.system() != 'Windows':
                 if os.getenv('srcdir') in ('.', None):
                     # make check or ./Test.py
                     addpth = './'
@@ -44,27 +40,24 @@ class PluginTestCase(AVGTestCase):
                     addpth = '../../_build/src/test/'
 
                 Player.pluginPath += ":"+addpth+"plugin/.libs"
-                Player.loadPlugin("libColorNode")
+            Player.loadPlugin("colorplugin")
             
         def usePlugin1():
             node = colorplugin.ColorNode(fillcolor="7f7f00", id="mynode1")
-            Player.getElementByID("container").appendChild(node)
+            root.appendChild(node)
             
             mynode = Player.getElementByID("mynode1")
-            self.assert_(mynode.fillcolor == "7f7f00")
+            self.assertEqual(mynode.fillcolor, "7f7f00")
  
         def usePlugin2():
             node = Player.createNode('<colornode fillcolor="0f3f7f" id="mynode2" />')
-            Player.getElementByID("container").appendChild(node)
+            root.appendChild(node)
 
             mynode = Player.getElementByID("mynode2")
-            self.assert_(mynode.fillcolor == "0f3f7f")
+            self.assertEqual(mynode.fillcolor, "0f3f7f")
 
-       
-        Player.loadString("""
-            <avg width="160" height="120" id="container" />""")
-
-        self.start(None, (
+        root = self.loadEmptyScene()
+        self.start(( 
             loadPlugin,
             usePlugin1,
             lambda: self.compareImage("testplugin1", False),
