@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2008 Ulrich von Zadow
+# Copyright (C) 2003-2011 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -29,12 +29,9 @@ class VectorTestCase(AVGTestCase):
         AVGTestCase.__init__(self, testFuncName)
 
     def makeEmptyCanvas(self):
-        Player.loadString("""
-            <avg width="160" height="120">
-              <div id="canvas" x="0" y="0" width="160" height="120"/>
-            </avg>
-        """)
-        return Player.getElementByID("canvas")
+        root = self.loadEmptyScene()
+        canvas = avg.DivNode(id="canvas", size=(160,120), parent=root)
+        return canvas
 
     def testLine(self):
         def addLines():
@@ -68,8 +65,8 @@ class VectorTestCase(AVGTestCase):
         canvas = self.makeEmptyCanvas()
         addLines()
         line = canvas.getChild(0)
-        self.start(None,
-                (lambda: self.compareImage("testline1", False), 
+        self.start((
+                 lambda: self.compareImage("testline1", False), 
                  changeLine,
                  lambda: self.compareImage("testline2", False),
                  moveLine,
@@ -86,8 +83,8 @@ class VectorTestCase(AVGTestCase):
                 canvas.appendChild(line)
        
         canvas = self.makeEmptyCanvas()
-        self.start(None,
-                (addLines,
+        self.start((
+                 addLines,
                  lambda: self.compareImage("testlotsoflines", False), 
                 ))
 
@@ -117,8 +114,8 @@ class VectorTestCase(AVGTestCase):
         
         canvas = self.makeEmptyCanvas()
         addLine()
-        self.start(None,
-                (lambda: self.compareImage("testtexturedline1", False), 
+        self.start((
+                 lambda: self.compareImage("testtexturedline1", False), 
                  removeLine,
                  lambda: self.compareImage("testtexturedline2", False), 
                  addLine,
@@ -145,8 +142,8 @@ class VectorTestCase(AVGTestCase):
             canvas.getChild(0).opacity = 0.25
         
         canvas = self.makeEmptyCanvas()
-        self.start(None,
-                (addLine,
+        self.start((
+                 addLine,
                  lambda: self.compareImage("testlineopacity1", False), 
                  changeCanvasOpacity,
                  lambda: self.compareImage("testlineopacity2", False), 
@@ -184,22 +181,19 @@ class VectorTestCase(AVGTestCase):
         canvas = self.makeEmptyCanvas()
         rect = addRect()
         helper = Player.getTestHelper()
-        self.start(None,
-                (lambda: self.compareImage("testRect1", False),
+        self.start((
+                 lambda: self.compareImage("testRect1", False),
                  moveRect,
                  lambda: self.compareImage("testRect2", False),
                  rotateRect,
                  lambda: self.compareImage("testRect3", False),
                  addRect2,
                  lambda: self.compareImage("testRect4", False),
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        100, 100, 1),
-                 lambda: self.assert_(self.__mouseDownCalled == False),
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        55, 50, 1),
-                 lambda: self.assert_(self.__mouseDownCalled == False),
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        65, 60, 1),
+                 lambda: self.fakeClick(100, 100),
+                 lambda: self.assertEqual(self.__mouseDownCalled, False),
+                 lambda: self.fakeClick(55, 50),
+                 lambda: self.assertEqual(self.__mouseDownCalled, False),
+                 lambda: self.fakeClick(65, 60),
                  lambda: self.assert_(self.__mouseDownCalled)
                 ))
 
@@ -247,8 +241,8 @@ class VectorTestCase(AVGTestCase):
 
         canvas = self.makeEmptyCanvas()
         addRect()
-        self.start(None,
-                (lambda: self.compareImage("testTexturedRect1", False),
+        self.start((
+                 lambda: self.compareImage("testTexturedRect1", False),
                  newRect,
                  lambda: self.compareImage("testTexturedRect2", False),
                  setTexCoords,
@@ -290,8 +284,8 @@ class VectorTestCase(AVGTestCase):
         
         canvas = self.makeEmptyCanvas()
         curve = addCurve()
-        self.start(None,
-                (lambda: self.compareImage("testCurve1", False),
+        self.start((
+                 lambda: self.compareImage("testCurve1", False),
                  changeCurve,
                  lambda: self.compareImage("testCurve2", False),
                  moveCurve,
@@ -313,8 +307,8 @@ class VectorTestCase(AVGTestCase):
         
         canvas = self.makeEmptyCanvas()
         curve = addCurve()
-        self.start(None,
-                (lambda: self.compareImage("testTexturedCurve1", False),
+        self.start((
+                 lambda: self.compareImage("testTexturedCurve1", False),
                  setTexCoords,
                  lambda: self.compareImage("testTexturedCurve2", False)
                 )) 
@@ -359,8 +353,8 @@ class VectorTestCase(AVGTestCase):
         
         canvas = self.makeEmptyCanvas()
         polyline = addPolyLine()
-        self.start(None,
-                (lambda: self.compareImage("testPolyLine1", False),
+        self.start((
+                 lambda: self.compareImage("testPolyLine1", False),
                  changePolyLine,
                  lambda: self.compareImage("testPolyLine2", False),
                  miterPolyLine,
@@ -395,8 +389,8 @@ class VectorTestCase(AVGTestCase):
         
         canvas = self.makeEmptyCanvas()
         polyline = texturePolyLine()
-        self.start(None,
-                (lambda: self.compareImage("testTexturedPolyLine1", False),
+        self.start((
+                 lambda: self.compareImage("testTexturedPolyLine1", False),
                  miter,
                  lambda: self.compareImage("testTexturedPolyLine2", False),
                  setTexCoords,
@@ -456,8 +450,8 @@ class VectorTestCase(AVGTestCase):
         canvas = self.makeEmptyCanvas()
         polygon = addPolygon()
         helper = Player.getTestHelper()
-        self.start(None,
-                (lambda: self.compareImage("testPolygon1", True),
+        self.start((
+                 lambda: self.compareImage("testPolygon1", True),
                  changePolygon,
                  lambda: self.compareImage("testPolygon2", True),
                  fillPolygon,
@@ -468,11 +462,9 @@ class VectorTestCase(AVGTestCase):
                  lambda: self.compareImage("testPolygon5", True),
                  miterPolygons,
                  lambda: self.compareImage("testPolygon6", False),
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        50, 50, 1),
-                 lambda: self.assert_(self.__mouseDownCalled == False),
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        20, 87, 1),
+                 lambda: self.fakeClick(50, 50),
+                 lambda: self.assertEqual(self.__mouseDownCalled, False),
+                 lambda: self.fakeClick(20, 87),
                  lambda: self.assert_(self.__mouseDownCalled),
                  addEmptyPolygon
                 ))
@@ -485,7 +477,7 @@ class VectorTestCase(AVGTestCase):
             canvas.insertChild(polygon, 0)
                 
         canvas = self.makeEmptyCanvas()
-        self.assertException(lambda: self.start(None, [addPolygon]))
+        self.assertException(lambda: self.start([addPolygon]))
 
     def testTexturedPolygon(self):
         def texturePolygon():
@@ -515,8 +507,8 @@ class VectorTestCase(AVGTestCase):
         
         canvas = self.makeEmptyCanvas()
         polygon = texturePolygon()
-        self.start(None,
-                (lambda: self.compareImage("testTexturedPolygon1", False),
+        self.start((
+                 lambda: self.compareImage("testTexturedPolygon1", False),
                  miter,
                  lambda: self.compareImage("testTexturedPolygon2", False),
                  setTexCoords,
@@ -571,8 +563,8 @@ class VectorTestCase(AVGTestCase):
         canvas = self.makeEmptyCanvas()
         circle = addCircle()
         helper = Player.getTestHelper()
-        self.start(None,
-                (lambda: self.compareImage("testCircle1", False), 
+        self.start((
+                 lambda: self.compareImage("testCircle1", False), 
                  changeCircle,
                  lambda: self.compareImage("testCircle2", False),
                  textureCircle,
@@ -581,11 +573,9 @@ class VectorTestCase(AVGTestCase):
                  lambda: self.compareImage("testCircle4", False),
                  setFillTexCoords,
                  lambda: self.compareImage("testCircle5", False),
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        32, 32, 1),
+                 lambda: self.fakeClick(32, 32),
                  lambda: self.assert_(self.__mouseDownCalled == False),
-                 lambda: helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False, 
-                        67, 50, 1),
+                 lambda: self.fakeClick(67, 50),
                  lambda: self.assert_(self.__mouseDownCalled)
                 ))
         
@@ -637,8 +627,8 @@ class VectorTestCase(AVGTestCase):
         self.assertException(setIllegalVertexes)
         self.assertException(setIllegalTextures)
         self.assertException(setIllegalIndexes)
-        self.start(None,
-                (lambda: self.compareImage("testMesh1", False),
+        self.start((
+                 lambda: self.compareImage("testMesh1", False),
                  setVertexCoords,
                  lambda: self.compareImage("testMesh2", False),
                  setTexCoords,
@@ -673,14 +663,12 @@ class VectorTestCase(AVGTestCase):
         vNode = addVectorNode()
         fvNode = addFilledVectorNode()
         self.onDownCalled = False
-        self.start(None,
-                (lambda: self.compareImage("testInactiveVector1", False),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
-                        20, 20, 1),
+        self.start((
+                 lambda: self.compareImage("testInactiveVector1", False),
+                 lambda: self.fakeClick(20, 20),
                  lambda: self.assert_(self.onDownCalled),
                  lambda: self.compareImage("testInactiveVector2", False),
-                 lambda: Helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
-                        20, 20, 1),
+                 lambda: self.fakeClick(20, 20),
                  lambda: self.assert_(self.onDownCalled)
                 ))
         
