@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2008 Ulrich von Zadow
+# Copyright (C) 2003-2011 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,8 +20,6 @@
 # Current versions can be found at www.libavg.de
 #
 
-import unittest
-
 import platform
 
 from libavg import avg
@@ -33,44 +31,51 @@ class WordsTestCase(AVGTestCase):
     
     def testSimpleWords(self):
         def checkFont():
-            node = Player.getElementByID("sanstext")
-            self.assert_(node.variant=="bold")
+            self.assertEqual(node.variant, "bold")
     
         def checkUnicodeText():
-            node = Player.getElementByID("sanstext")
             node.text = u"föa"
             newNode = avg.WordsNode(text=u"öäü")
-        
+       
         fontList = avg.WordsNode.getFontFamilies()
         try:
             fontList.index("Bitstream Vera Sans")
         except ValueError:
-            self.assert_(False)
+            self.fail("Font not found")
         variantList = avg.WordsNode.getFontVariants("Bitstream Vera Sans")
         self.assert_(len(variantList) >= 4)
-        Player.loadString("""
-          <avg width="160" height="120">
-            <words x="1" y="1" fontsize="12" font="Bitstream Vera Sans" 
-                text="Bitstream Vera Sans" variant="roman"/>
-            <words id="sanstext" x="1" y="16" fontsize="12" font="Bitstream Vera Sans" 
-                variant="bold" text="Bold"/>
-          </avg>
-        """)
-        node = Player.getElementByID("sanstext")
-        self.assert_(node.size != (0,0))
+        root = self.loadEmptyScene()
+        avg.WordsNode (pos=(1,1), fontsize=12, font="Bitstream Vera Sans",
+                text="Bitstream Vera Sans", variant="roman", parent=root)
+        node = avg.WordsNode (pos=(1,16), fontsize=12, font="Bitstream Vera Sans",
+                text="Bold", variant="bold", parent=root)
+        self.assertNotEqual(node.size, (0,0))
         pos = node.getGlyphPos(0)
-        self.start(None,
-                (lambda: self.compareImage("testSimpleWords", True),
+        self.start((
+                 lambda: self.compareImage("testSimpleWords", True),
                  checkFont,
-                 checkUnicodeText
+                 checkUnicodeText,
                 ))
 
+    def testRedrawOnDemand(self):
+
+        def changeText(newText):
+            size = node.size
+            node.text = newText 
+            self.assertNotEqual(node.size, size)
+
+        root = self.loadEmptyScene()
+        node = avg.WordsNode(font="Bitstream Vera Sans", fontsize=12, text="foo", 
+                parent=root)
+        changeText("foobar")
+        self.start((lambda: changeText("bar"),))
+
     def testGlyphPos(self):
         def posAlmostEqual(pos1, pos2):
             return math.fabs(pos1[0]-pos2[0]) <= 2 and math.fabs(pos1[1]-pos2[1]) <= 2
         
         node = avg.WordsNode(text="Bold", font="Bitstream Vera Sans")
-        self.assert_(node.getGlyphPos(0) == (0,0))
+        self.assertEqual(node.getGlyphPos(0), (0,0))
         size = node.getGlyphSize(0)
         self.assert_(posAlmostEqual(size, (10, 18)))
         self.assert_(posAlmostEqual(node.getGlyphPos(3), (22,0)))
@@ -84,62 +89,51 @@ class WordsTestCase(AVGTestCase):
         self.assertException(lambda: node.getGlyphPos(3))
 
     def testParaWords(self):
-        Player.loadString("""
-          <avg width="160" height="120">
-              <line pos1="(0.5, 0)" pos2="(0.5, 50)" color="FF0000"/>
-              <line pos1="(119.5, 0.5)" pos2="(119.5, 50)" color="FF0000"/>
-              <line pos1="(74.5, 60)" pos2="(74.5, 110)" color="FF0000"/>
-              <words id="para" x="1" y="1" fontsize="12" width="70" 
-                      font="Bitstream Vera Sans"
-                      text="Left-justified paragraph."/>
-              <words id="paracenter" x="120" y="1" fontsize="12" width="70" 
-                      font="Bitstream Vera Sans" alignment="center"
-                      text="Centered paragraph"/>
-              <words id="pararight" x="75" y="60" fontsize="12" width="70" 
-                      font="Bitstream Vera Sans" alignment="right">
-                      Right-justified paragraph.<i>l</i></words>
-              <words id="paralinespacing" x="80" y="60" fontsize="12" width="70" 
-                      font="Bitstream Vera Sans" linespacing="-4"
-                      text="Paragraph with custom line spacing."/>
-          </avg>
-        """)
-        self.start(None,
-                [lambda: self.compareImage("testParaWords", True)])
+        root = self.loadEmptyScene()
+        avg.LineNode(pos1=(0.5, 0), pos2=(0.5, 50), color="FF0000", parent=root)
+        avg.LineNode(pos1=(119.5, 0.5), pos2=(119.5, 50), color="FF0000", parent=root)
+        avg.LineNode(pos1=(74.5,60), pos2=(74.5, 110), color="FF0000", parent=root)
+        avg.WordsNode(id="para", pos=(1,1), fontsize=12, width=70,
+                font="Bitstream Vera Sans", text="Left-justified paragraph.", 
+                parent=root)
+        avg.WordsNode(id="paracenter", pos=(120,1), fontsize=12, width=70, 
+                font="Bitstream Vera Sans", text="Centered paragraph", 
+                alignment="center", parent=root)
+        avg.WordsNode(id="pararight", pos=(75,60), fontsize=12, width=70,
+                font="Bitstream Vera Sans", alignment="right",
+                text="Right-justified paragraph.<i>l</i>",
+                parent=root)
+        avg.WordsNode(id="paralinespacing", pos=(80,60), fontsize=12, width=70,
+                font="Bitstream Vera Sans", linespacing=-4,
+                text="Paragraph with custom line spacing.",
+                parent=root)
+        self.start([lambda: self.compareImage("testParaWords", True)])
 
     def testJustify(self):
-        Player.loadString("""
-          <avg width="160" height="120">
-            <words x="1" y="1" fontsize="12" font="Bitstream Vera Sans"
-                variant="roman" justify="true" width="100"
-                text="Justified paragraph more than one line long."/>
-          </avg>
-        """)
-        self.start(None,
-                [lambda: self.compareImage("testJustify", True)])
+        root = self.loadEmptyScene()
+        avg.WordsNode(pos=(1,1), fontsize=12, font="Bitstream Vera Sans",
+                variant="roman", justify=True, width=100,
+                text="Justified paragraph more than one line long.", parent=root)
+        self.start([lambda: self.compareImage("testJustify", True)])
 
     def testWrapMode(self):
         def setCharMode():
-            node = Player.getElementByID("words")
             node.wrapmode = 'char'
         
         def setWordMode():
-            node = Player.getElementByID("words")
             node.wrapmode = 'word'
         
         def setWordCharMode():
-            node = Player.getElementByID("words")
             node.wrapmode = 'wordchar'
             
-        Player.loadString("""
-        <avg width="160" height="120">
-          <words x="1" y="1" fontsize="12" font="Bitstream Vera Sans"
-              variant="roman" width="100" id="words"
-              text="Wrapped paragraph more than one line long.
-                Withaverylongpackedlinewithnobreaks"/>
-        </avg>
-        """)
-        self.start(None,
-            [lambda: self.compareImage("testWrapMode1", True),
+        root = self.loadEmptyScene()
+        node = avg.WordsNode(pos=(1,1), fontsize=12, font="Bitstream Vera Sans",
+                variant="roman", width=100, 
+                text="""Wrapped paragraph more than one line long.
+                        Withaverylongpackedlinewithnobreaks""",
+                parent=root)
+        self.start([
+             lambda: self.compareImage("testWrapMode1", True),
              setCharMode,
              lambda: self.compareImage("testWrapMode2", True),
              setWordMode,
@@ -153,7 +147,7 @@ class WordsTestCase(AVGTestCase):
             try:
                 node.maskhref = "mask1.png"
             except RuntimeError:
-                print "Skipping testWordsMask - no shader support."
+                self.skip("no shader support")
                 Player.stop()
            
         def setColor():
@@ -164,7 +158,7 @@ class WordsTestCase(AVGTestCase):
 
         def setSize():
             rect = avg.RectNode(pos=(39.5, 30.5), size=(80, 60))
-            Player.getRootNode().appendChild(rect)
+            root.appendChild(rect)
             node.masksize = (160, 120)
             node.opacity = 1 
 
@@ -175,11 +169,10 @@ class WordsTestCase(AVGTestCase):
         def setDefaultSize():
             node.masksize = (0,0)
 
-        Player.loadString("""
-        <avg width="160" height="120">
-          <words x="0" y="0" fontsize="8" linespacing="-4" font="Bitstream Vera Sans"
-              variant="roman" width="160" id="words"
-              text="Ich bin nur ein kleiner Blindtext. Wenn ich gross bin, will ich \
+        root = self.loadEmptyScene()
+        node = avg.WordsNode(fontsize=8, linespacing=-4, font="Bitstream Vera Sans",
+                variant="roman", width=160,
+                text="Ich bin nur ein kleiner Blindtext. Wenn ich gross bin, will ich \
                     Ulysses von James Joyce werden. Aber jetzt lohnt es sich noch nicht, \
                     mich weiterzulesen. Denn vorerst bin ich nur ein kleiner Blindtext. \
                     Ich bin nur ein kleiner Blindtext. Wenn ich gross bin, will ich \
@@ -187,12 +180,10 @@ class WordsTestCase(AVGTestCase):
                     mich weiterzulesen. Denn vorerst bin ich nur ein kleiner Blindtext. \
                     Ich bin nur ein kleiner Blindtext. Wenn ich gross bin, will ich \
                     Ulysses von James Joyce werden. Aber jetzt lohnt es sich noch nicht, \
-                    mich weiterzulesen. Denn vorerst bin ich nur ein kleiner Blindtext."/>
-        </avg>
-        """)
-        node = Player.getElementByID("words")
-        self.start(None,
-                (setMask,
+                    mich weiterzulesen. Denn vorerst bin ich nur ein kleiner Blindtext.",
+                parent=root)
+        self.start((
+                 setMask,
                  lambda: self.compareImage("testWordsMask1", False),
                  setColor,
                  lambda: self.compareImage("testWordsMask2", False),
@@ -208,7 +199,6 @@ class WordsTestCase(AVGTestCase):
 
     def testHinting(self):
         def checkPositions():
-            root = Player.getRootNode()
 #            node0 = root.getChild(0)
 #            for i in range(len(node0.text)):
 #                print node0.getGlyphPos(i)
@@ -216,35 +206,29 @@ class WordsTestCase(AVGTestCase):
             hint = root.getChild(1)
             posNoHint = noHint.getGlyphPos(6)
             posHint = hint.getGlyphPos(6)
-            self.assert_(posNoHint != posHint)
+            self.assertNotEqual(posNoHint, posHint)
             noHint.hint = True
             hint.hint = False
-            self.assert_(posNoHint == hint.getGlyphPos(6))
-            self.assert_(posHint == noHint.getGlyphPos(6))
+            self.assertEqual(posNoHint, hint.getGlyphPos(6))
+            self.assertEqual(posHint, noHint.getGlyphPos(6))
 
         if platform.system() == "Linux":
-            print "Skipping testHinting - Linux support requires modified font config."
+            self.skip("Linux support requires modified font config")
         else:
-            Player.loadString("""
-            <avg width="160" height="120">
-              <words x="1" y="1" fontsize="12" font="Bitstream Vera Sans"
-                  variant="roman" hint="false"
-                  text="Lorem ipsum dolor (no hinting)"/>
-              <words x="1" y="15" fontsize="12" font="Bitstream Vera Sans"
-                  variant="roman" hint="true"
-                  text="Lorem ipsum dolor (hinting)"/>
-            </avg>
-            """)
-            self.start(None,
-                [checkPositions
-                ])
+            root = self.loadEmptyScene()
+            avg.WordsNode(pos=(1,1), fontsize=12, font="Bitstream Vera Sans",
+                    variant="roman", hint=False, text="Lorem ipsum dolor (no hinting)",
+                    parent=root)
+            avg.WordsNode(pos=(1,15), fontsize=12, font="Bitstream Vera Sans",
+                    variant="roman", hint=True, text="Lorem ipsum dolor (hinting)",
+                    parent=root)
+            self.start([checkPositions])
 
 
     def testSpanWords(self):
         def setTextAttrib():
             self.baselineBmp = Player.screenshot()
-            node = Player.getElementByID("words")
-            node.text = self.text
+            Player.getElementByID("words").text = self.text
         
         def checkSameImage():
             bmp = Player.screenshot()
@@ -254,24 +238,24 @@ class WordsTestCase(AVGTestCase):
             Player.getElementByID("words").unlink()
             node = avg.WordsNode(id="words", pos=(1,1), fontsize=12, width=120,
                     font="Bitstream Vera Sans", variant="roman", text=self.text)
-            Player.getRootNode().appendChild(node)
+            root.appendChild(node)
         
         self.text = """
               Markup: 
               <span size='14000' rise='5000' foreground='red'>span</span>, 
               <i>italics</i>, <b>bold</b>
         """
-        Player.loadString("""
-          <avg width="160" height="120">
+        root = self.loadEmptyScene()
+        node = Player.createNode("""
             <words id="words" x="1" y="1" fontsize="12" width="120" 
                 font="Bitstream Vera Sans" variant="roman">
         """
         +self.text+
         """
             </words>
-          </avg>
         """)
-        self.start(None,
+        root.appendChild(node)
+        self.start(
                 [lambda: self.compareImage("testSpanWords", True),
                  setTextAttrib,
                  lambda: self.compareImage("testSpanWords", True),
@@ -283,43 +267,41 @@ class WordsTestCase(AVGTestCase):
     
     def testDynamicWords(self):
         def changeText():
-            node = Player.getElementByID("dynamictext")
-            oldwidth = node.width
-            node.text = "blue" 
-            self.assert_(node.width != oldwidth)
-            node.color = "404080"
-            node.x += 10
+            oldwidth = words.width
+            words.text = "blue" 
+            self.assertNotEqual(words.width, oldwidth)
+            words.color = "404080"
+            words.x += 10
         
         def changeHeight():
-            node = Player.getElementByID("dynamictext")
-            node.height = 28
+            words.height = 28
         
         def activateText():
-            Player.getElementByID('dynamictext').active = 1
+            words.active = True
         
         def deactivateText():
-            Player.getElementByID('dynamictext').active = 0
+            words.active = False
         
         def changeFont():
-            node = Player.getElementByID("dynamictext")
-            node.font = "Bitstream Vera Sans"
-            node.height = 0
-            node.fontsize = 30
+            words.font = "Bitstream Vera Sans"
+            words.height = 0
+            words.fontsize = 30
         
         def changeFont2():
-            node = Player.getElementByID("dynamictext")
-            node.fontsize = 18
+            words.fontsize = 18
         
         def changeTextWithInvalidTag():
-            node = Player.getElementByID("dynamictext")
             try:
-                node.text = "This <invalid_tag/>bombs"
+                words.text = "This <invalid_tag/>bombs"
             except:
-                node.text = "except"
-            self.assert_(node.text == "except")
-        
-        self.start("dynamictext.avg",
-                (lambda: self.compareImage("testDynamicWords1", True),
+                words.text = "except"
+            self.assertEqual(words.text, "except")
+       
+        root = self.loadEmptyScene()
+        words = avg.WordsNode(pos=(1,1), fontsize=12, font="Bitstream Vera Sans",
+                text="foo", parent=root)
+        self.start((
+                 lambda: self.compareImage("testDynamicWords1", True),
                  changeText,
                  changeHeight,
                  changeFont,
@@ -334,16 +316,29 @@ class WordsTestCase(AVGTestCase):
 
     def testI18NWords(self):
         def changeUnicodeText():
-            Player.getElementByID("dynamictext").text = "Arabic nonsense: ﯿﭗ"
+            words.text = "Arabic nonsense: ﯿﭗ"
         
         def setNBSP():
-            node = Player.getElementByID("dynamictext")
-            node.width=100
-            node.text=(u"blindtext1\u00A0blindtext2\u00Ablindtext3 "+
+            words.width=100
+            words.text=(u"blindtext1\u00A0blindtext2\u00Ablindtext3 "+
                     u"blindtext4\u00A0blindtext\u00A0blindtext\u00A0")
 
-        self.start("i18ntext.avg",
-                (lambda: self.compareImage("testI18NWords1", True),
+        root = self.loadEmptyScene()
+        avg.WordsNode(pos=(1,1), fontsize=14, font="Bitstream Vera Sans",
+                text="一二三四五六七八九", parent=root)
+        words = avg.WordsNode(pos=(1,24), fontsize=12, font="Bitstream Vera Sans",
+                text="foo", parent=root)
+        root.appendChild(
+                Player.createNode("""
+                    <words x="1" y="48" fontsize="12" font="Bitstream Vera Sans">
+                            &amp;
+                    </words>
+                """))
+        avg.WordsNode(pos=(12,48), fontsize=12, font="Bitstream Vera Sans", text="&amp;",
+                rawtextmode=True, parent=root)
+
+        self.start((
+                 lambda: self.compareImage("testI18NWords1", True),
                  changeUnicodeText,
                  lambda: self.compareImage("testI18NWords2", True),
                  setNBSP,
@@ -355,18 +350,18 @@ class WordsTestCase(AVGTestCase):
             self.dictdnode = avg.WordsNode(text='&lt;test dyndict&amp;', 
                     rawtextmode=True, pos=(1,65), font='Bitstream Vera Sans', 
                     variant='roman', fontsize=12)
-            Player.getRootNode().appendChild(self.dictdnode)
+            root.appendChild(self.dictdnode)
 
             self.xmldnode = Player.createNode("""
                 <words text="&lt;test dynattr&amp;" fontsize="12" 
                         font="Bitstream Vera Sans" variant="roman" rawtextmode="true"
                         x="1" y="85"/>""")
-            Player.getRootNode().appendChild(self.xmldnode)
+            root.appendChild(self.xmldnode)
         
         def switchRawMode():
             self.dictdnode.rawtextmode = False
-            Player.getElementByID('nodeval').rawtextmode = True
-            Player.getElementByID('attrib').rawtextmode = True
+            valNode.rawtextmode = True
+            attribNode.rawtextmode = True
         
         def bombIt():
             def cantRun():
@@ -379,11 +374,23 @@ class WordsTestCase(AVGTestCase):
             self.dictdnode.rawtextmode = True
             self.dictdnode.text = text
             self.xmldnode.text = text
-            Player.getElementByID('nodeval').text = text
-            Player.getElementByID('attrib').text = text
-        
-        self.start("rawtext.avg",
-                (lambda: self.compareImage("testRawText1", True),
+            valNode.text = text
+            attribNode.text = text
+
+        root = self.loadEmptyScene()
+        attribNode = avg.WordsNode(text="ùnicòdé <b>bold</b>",
+                fontsize=12, pos=(1,5), font="Bitstream Vera Sans", parent=root)
+        valNode = Player.createNode("""
+            <words id="nodeval" fontsize="10" x="1" y="25" font="Bitstream Vera Sans"><b>bold</b> ùnicòdé  &lt;</words>""")
+        root.appendChild(valNode)
+        root.appendChild(
+                Player.createNode("""
+                        <words x="1" y="45" fontsize="15" font="Bitstream Vera Sans">
+                            &amp;
+                        </words>"""))
+
+        self.start((
+                 lambda: self.compareImage("testRawText1", True),
                  createDynNodes,
                  lambda: self.compareImage("testRawText2", True),
                  switchRawMode,
@@ -394,45 +401,34 @@ class WordsTestCase(AVGTestCase):
                 ))
 
     def testWordsBR(self):
-        Player.loadString("""
-          <avg width="160" height="120">
-            <words id="words" x="1" y="1" fontsize="12" 
-                font="Bitstream Vera Sans" variant="roman">
-               paragraph 1<br/>paragraph 2
-            </words>
-          </avg>
-        """)
-        self.start(None,
-                [lambda: self.compareImage("testWordsBR", True)
-                ])
+        root = self.loadEmptyScene()
+        avg.WordsNode(pos=(1,1), fontsize=12, font="Bitstream Vera Sans", variant="roman",
+               text="paragraph 1<br/>paragraph 2", parent=root)
+        self.start([lambda: self.compareImage("testWordsBR", True)])
 
     def testLetterSpacing(self):
         def setSpacing():
             Player.getElementByID("words1").letterspacing=-2
             Player.getElementByID("words2").letterspacing=-2
         
-        Player.loadString("""
-          <avg width="160" height="120">
-            <words id="words1" x="1" y="1" fontsize="12" font="Bitstream Vera Sans" 
-                    variant="roman">
-               normal
-               <span letter_spacing="-2048"> packed</span>
-               <span letter_spacing="2048"> spaced</span>
-            </words>
-            <words id="words2" x="1" y="20" fontsize="12" font="Bitstream Vera Sans"
-                    variant="roman" letterspacing="2" text="spaced"/>
-          </avg>
-        """)
-        self.start(None,
-                (lambda: self.compareImage("testLetterSpacing1", True),
+        root = self.loadEmptyScene()
+        avg.WordsNode(id="words1", pos=(1,1), fontsize=12, font="Bitstream Vera Sans",
+                variant="roman",
+                text="""normal
+                   <span letter_spacing="-2048"> packed</span>
+                   <span letter_spacing="2048"> spaced</span>""",
+                parent=root)
+        avg.WordsNode(id="words2", pos=(1,20), fontsize=12, font="Bitstream Vera Sans",
+                variant="roman", letterspacing=2, text="spaced", parent=root)
+        self.start((
+                 lambda: self.compareImage("testLetterSpacing1", True),
                  setSpacing,
                  lambda: self.compareImage("testLetterSpacing2", True)
                 ))
        
     def testPositioning(self):
         def click(pos):
-            helper.fakeMouseEvent(avg.CURSORDOWN, True, False, False,
-                        int(pos[0]), int(pos[1]), 1)
+            self.fakeClick(int(pos[0]), int(pos[1]))
            
         def testInside(bInside):
             ok = bInside == self.clicked
@@ -442,26 +438,25 @@ class WordsTestCase(AVGTestCase):
         def onMouse(event):
             self.clicked = True
 
-        Player.loadString("""
-          <avg width="160" height="120">
-            <line pos1="(4, 20.5)" pos2="(157, 20.5)" color="FF0000"/>
-            <line pos1="(4.5, 20.5)" pos2="(4.5, 110)" color="FF0000"/>
-            <line pos1="(156.5, 20.5)" pos2="(156.5, 110)" color="FF0000"/>
-            <line pos1="(80.5, 20.5)" pos2="(80.5, 110)" color="FF0000"/>
-            <words id="left" x="4" y="20" fontsize="12" font="Bitstream Vera Sans"
-                    variant="roman" text="Norm"/>
-            <words x="45" y="20" fontsize="12" font="Bitstream Vera Sans"
-                    variant="roman" text="orm"/>
-            <words x="75" y="20" fontsize="12" font="Bitstream Vera Sans"
-                    variant="roman" text="ÖÄÜ"/>
-            <words x="4" y="40" fontsize="12" font="Bitstream Vera Sans"
-                    variant="oblique" text="Jtalic"/>
-            <words id="right" x="156" y="60" fontsize="12" alignment="right" 
-                    font="Bitstream Vera Sans" variant="roman" text="Right-aligned"/>
-            <words id="center" x="80" y="80" fontsize="12" alignment="center" 
-                    font="Bitstream Vera Sans" variant="roman" text="Centered"/>
-          </avg>
-        """)
+        root = self.loadEmptyScene()
+        avg.LineNode(pos1=(4, 20.5), pos2=(157, 20.5), color="FF0000", parent=root)
+        avg.LineNode(pos1=(4.5, 20.5), pos2=(4.5, 110), color="FF0000", parent=root)
+        avg.LineNode(pos1=(156.5, 20.5), pos2=(156.5, 110), color="FF0000", parent=root)
+        avg.LineNode(pos1=(80.5, 20.5), pos2=(80.5, 110), color="FF0000", parent=root)
+        avg.WordsNode(id="left", pos=(4,20), fontsize=12, font="Bitstream Vera Sans",
+                variant="roman", text="Norm", parent=root)
+        avg.WordsNode(pos=(45,20), fontsize=12, font="Bitstream Vera Sans",
+                variant="roman", text="orm", parent=root)
+        avg.WordsNode(pos=(75,20), fontsize=12, font="Bitstream Vera Sans",
+                variant="roman", text="ÖÄÜ", parent=root)
+        avg.WordsNode(pos=(4,40), fontsize=12, font="Bitstream Vera Sans",
+                variant="oblique", text="Jtalic", parent=root)
+        avg.WordsNode(id="right", pos=(156,60), fontsize=12, alignment="right",
+                font="Bitstream Vera Sans", variant="roman", text="Right-aligned",
+                parent=root)
+        avg.WordsNode(id="center", pos=(80,80), fontsize=12, alignment="center",
+                font="Bitstream Vera Sans", variant="roman", text="Centered",
+                parent=root)
         for id in ["left", "center", "right"]:
             Player.getElementByID(id).setEventHandler(avg.CURSORDOWN, avg.MOUSE,
                     onMouse)
@@ -471,8 +466,8 @@ class WordsTestCase(AVGTestCase):
         centerWidth = Player.getElementByID("center").getMediaSize()[0]
         rightWidth = Player.getElementByID("right").getMediaSize()[0]
 
-        self.start(None,
-                (lambda: self.compareImage("testPositioning", True),
+        self.start((
+                 lambda: self.compareImage("testPositioning", True),
                  lambda: click((4,20)),
                  lambda: self.assert_(testInside(True)),
                  lambda: click((3,20)),
@@ -517,38 +512,30 @@ class WordsTestCase(AVGTestCase):
         def assignInvalidColor3():
             testColor('xxxxxx')
 
-        Player.loadString("""
-          <avg width="160" height="120">
-          </avg>
-        """)
-        self.start(None, (
-            self.assertException(assignInvalidColor1),
-            self.assertException(assignInvalidColor2),
-            self.assertException(assignInvalidColor3),
-                    ))
+        root = self.loadEmptyScene()
+        self.start((
+                 self.assertException(assignInvalidColor1),
+                 self.assertException(assignInvalidColor2),
+                 self.assertException(assignInvalidColor3),
+                ))
 
     def testFontDir(self):
         avg.WordsNode.addFontDir('extrafonts')
-        Player.loadString("""
-          <avg width="160" height="120">
-            <words font="testaddfontdir" fontsize="50" text="ABAAA"/>
-          </avg>
-        """)
-        self.start(None,
-                (lambda: self.compareImage("testFontDir", True),
-                ))
+        root = self.loadEmptyScene()
+        avg.WordsNode(font="testaddfontdir", fontsize=50, text="ABAAA", parent=root)
+        self.start((lambda: self.compareImage("testFontDir", True),))
     
     def testGetNumLines(self):
         textNode = avg.WordsNode(text="paragraph 1<br/>paragraph 2<br/>paragraph 3")
-        self.assert_(textNode.getNumLines() == 3)
+        self.assertEqual(textNode.getNumLines(), 3)
     
     def testGetLineExtents(self):        
         textNode = avg.WordsNode(fontsize = 100,
                                 font = "Bitstream Vera Sans",
                                 text = "bla <br/> blabli <br/> blabliblabla")
-        self.assert_(textNode.getLineExtents(0) == (184,117))
+        self.assertEqual(textNode.getLineExtents(0), (184,117))
         textNode.parent = Player.getRootNode()
-        self.assert_(textNode.getLineExtents(1) == (303,117))
+        self.assertEqual(textNode.getLineExtents(1), (303,117))
         
     def testGetCharIndexFromPos(self):
         textNode = avg.WordsNode(fontsize=30,
@@ -559,7 +546,7 @@ class WordsTestCase(AVGTestCase):
         for k in (1,2,3,23,42):
             pos = textNode.getGlyphPos(k)
             char = textNode.getCharIndexFromPos(pos)
-            self.assert_(char == k)
+            self.assertEqual(char, k)
         
     def testGetTextAsDisplayed(self):
         orgText = "A<br/>B C <b>D</b> E F G H <i>Ä</i> Ö Ü ? Ì Á<br/>Í Å Ø ∏ ~ ç Ç Å"
@@ -568,13 +555,13 @@ class WordsTestCase(AVGTestCase):
                       font = "Bitstream Vera Sans",
                       text = orgText,
                       width = 300)
-        self.assert_(orgTextWithout == textNode.getTextAsDisplayed())
+        self.assertEqual(orgTextWithout, textNode.getTextAsDisplayed())
 
     def testSetWidth(self):
-        Player.loadString("""<avg width="160" height="120"></avg>""")
+        root = self.loadEmptyScene()
         text = "42 " * 42
         textNode = avg.WordsNode(
-                parent=Player.getRootNode(),
+                parent=root,
                 fontsize = 10,
                 font = "Bitstream Vera Sans",
                 text = text)
@@ -591,14 +578,22 @@ class WordsTestCase(AVGTestCase):
             textNode.width = 50 
             testSize(textNode.size, avg.Point2D(50,182))
             testSize(textNode.getMediaSize(), avg.Point2D(45,182))
-            self.assert_(mediaSize != textNode.getMediaSize())
+            self.assertNotEqual(mediaSize, textNode.getMediaSize())
 
-        self.start(None, [lambda: changeSize()])
+        self.start([lambda: changeSize()])
         
+    def testTooWide(self):
+        root = self.loadEmptyScene()
+        text = "42 " * 42 * 20 
+        node = avg.WordsNode(parent=root, text=text)
+        self.assertException(
+                lambda: self.start((None, None))
+        )
 
 def wordsTestSuite(tests):
     availableTests = (
             "testSimpleWords",
+            "testRedrawOnDemand",
             "testGlyphPos",
             "testParaWords",
             "testJustify",
@@ -619,6 +614,7 @@ def wordsTestSuite(tests):
             "testGetCharIndexFromPos",
             "testGetTextAsDisplayed",
             "testSetWidth",
+            "testTooWide",
             )
     return createAVGTestSuite(availableTests, WordsTestCase, tests)
 
