@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,8 @@
 
 #include "../base/ObjectCounter.h"
 #include "../base/Exception.h"
+#include "../base/ScopeTimer.h"
+#include "FFMpegDecoder.h"
 
 #include <boost/thread/thread.hpp>
 #include <boost/bind.hpp>
@@ -60,14 +62,15 @@ AsyncVideoDecoder::~AsyncVideoDecoder()
     ObjectCounter::get()->decRef(&typeid(*this));
 }
 
-void AsyncVideoDecoder::open(const std::string& sFilename, bool bThreadedDemuxer)
+void AsyncVideoDecoder::open(const std::string& sFilename, bool bThreadedDemuxer,
+        bool bUseHardwareAccelleration)
 {
     m_bAudioEOF = false;
     m_bVideoEOF = false;
     m_bSeekPending = false;
     m_sFilename = sFilename;
 
-    m_pSyncDecoder->open(m_sFilename, bThreadedDemuxer);
+    m_pSyncDecoder->open(m_sFilename, bThreadedDemuxer, bUseHardwareAccelleration);
     m_VideoInfo = m_pSyncDecoder->getVideoInfo();
     // Temporary pf - always assumes shaders will be available.
     m_PF = m_pSyncDecoder->getPixelFormat();
@@ -127,7 +130,7 @@ void AsyncVideoDecoder::close()
     }        
 }
 
-IVideoDecoder::DecoderState AsyncVideoDecoder::getState() const
+VideoDecoder::DecoderState AsyncVideoDecoder::getState() const
 {
     return m_State;
 }
@@ -147,7 +150,6 @@ void AsyncVideoDecoder::seek(double destTime)
     m_bAudioEOF = false;
     m_bVideoEOF = false;
     m_bSeekPending = false;
-    m_LastVideoFrameTime = -1;
     m_bSeekPending = true;
     if (m_pVCmdQ) {
         m_pVCmdQ->pushCmd(boost::bind(&VideoDecoderThread::seek, _1, destTime));
@@ -186,6 +188,9 @@ void AsyncVideoDecoder::loop()
     m_LastVideoFrameTime = -1;
     m_bAudioEOF = false;
     m_bVideoEOF = false;
+    if (getVideoInfo().m_bHasAudio) {
+        seek(0);
+    }
 }
 
 IntPoint AsyncVideoDecoder::getSize() const
@@ -266,6 +271,8 @@ PixelFormat AsyncVideoDecoder::getPixelFormat() const
     return m_PF;
 }
 
+static ProfilingZoneID VDPAUDecodeProfilingZone("AsyncVideoDecoder: VDPAU");
+
 FrameAvailableCode AsyncVideoDecoder::renderToBmps(vector<BitmapPtr>& pBmps,
         double timeWanted)
 {
@@ -274,10 +281,22 @@ FrameAvailableCode AsyncVideoDecoder::renderToBmps(vector<BitmapPtr>& pBmps,
     VideoMsgPtr pFrameMsg = getBmpsForTime(timeWanted, frameAvailable);
     if (frameAvailable == FA_NEW_FRAME) {
         AVG_ASSERT(pFrameMsg);
-        for (unsigned i = 0; i < pBmps.size(); ++i) {
-            pBmps[i]->copyPixels(*(pFrameMsg->getFrameBitmap(i)));
+        if (pFrameMsg->getType() == VideoMsg::VDPAU_FRAME) {
+#ifdef AVG_ENABLE_VDPAU
+            ScopeTimer timer(VDPAUDecodeProfilingZone);
+            vdpau_render_state* pRenderState = pFrameMsg->getRenderState();
+            if (pixelFormatIsPlanar(m_PF)) {
+                getPlanesFromVDPAU(pRenderState, pBmps[0], pBmps[1], pBmps[2]);
+            } else {
+                getBitmapFromVDPAU(pRenderState, pBmps[0]);
+            }
+#endif
+        } else {
+            for (unsigned i = 0; i < pBmps.size(); ++i) {
+                pBmps[i]->copyPixels(*(pFrameMsg->getFrameBitmap(i)));
+            }
+            returnFrame(pFrameMsg);
         }
-        returnFrame(pFrameMsg);
     }
     return frameAvailable;
 }
@@ -377,7 +396,16 @@ VideoMsgPtr AsyncVideoDecoder::getBmpsForTime(double timeWanted,
                 return VideoMsgPtr();
             }
             while (frameTime-timeWanted < -0.5*timePerFrame && !m_bVideoEOF) {
-                returnFrame(pFrameMsg);
+                if (pFrameMsg) {
+                    if (pFrameMsg->getType() == VideoMsg::FRAME) {
+                        returnFrame(pFrameMsg);
+                    } else {
+#if AVG_ENABLE_VDPAU
+                        vdpau_render_state* pRenderState = pFrameMsg->getRenderState();
+                        VDPAU::unlockSurface(pRenderState);
+#endif
+                    }
+                }
                 pFrameMsg = getNextBmps(false);
                 if (pFrameMsg) {
                     frameTime = pFrameMsg->getFrameTime();
@@ -408,6 +436,7 @@ VideoMsgPtr AsyncVideoDecoder::getNextBmps(bool bWait)
     if (pMsg) {
         switch (pMsg->getType()) {
             case VideoMsg::FRAME:
+            case VideoMsg::VDPAU_FRAME:
                 return pMsg;
             case VideoMsg::END_OF_FILE:
                 m_bVideoEOF = true;
@@ -445,6 +474,8 @@ void AsyncVideoDecoder::waitForSeekDone()
                 case VideoMsg::FRAME:
                     returnFrame(pMsg);
                     break;
+                case VideoMsg::VDPAU_FRAME:
+                    break;
                 default:
                     // TODO: Handle ERROR messages here.
                     break;
