@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -44,11 +44,6 @@ using namespace boost;
 #define SAMPLE_BUFFER_SIZE ((AVCODEC_MAX_AUDIO_FRAME_SIZE*3))
 #define VOLUME_FADE_SAMPLES 100
 
-#if LIBAVFORMAT_BUILD < ((50<<16)+(0<<8)+0)
-#define PIX_FMT_BGRA PIX_FMT_RGBA32
-#define PIX_FMT_YUYV422 PIX_FMT_YUV422
-#endif
-
 namespace avg {
 
 bool FFMpegDecoder::s_bInitialized = false;
@@ -70,6 +65,10 @@ FFMpegDecoder::FFMpegDecoder()
       m_pDemuxer(0),
       m_pVStream(0),
       m_pAStream(0),
+#ifdef AVG_ENABLE_VDPAU
+      m_VDPAU(),
+      m_Opaque(&m_VDPAU),
+#endif
       m_VStreamIndex(-1),
       m_bFirstPacket(false),
       m_VideoStartTimestamp(-1),
@@ -88,9 +87,13 @@ FFMpegDecoder::~FFMpegDecoder()
     ObjectCounter::get()->decRef(&typeid(*this));
 }
 
-
 void avcodecError(const string& sFilename, int err)
 {
+#if LIBAVFORMAT_VERSION_MAJOR > 52
+        char buf[256];
+        av_strerror(err, buf, 256);
+        throw Exception(AVG_ERR_VIDEO_INIT_FAILED, sFilename + ": " + buf);
+#else
     switch(err) {
         case AVERROR_NUMEXPECTED:
             throw Exception(AVG_ERR_VIDEO_INIT_FAILED, 
@@ -106,43 +109,34 @@ void avcodecError(const string& sFilename, int err)
             s << "'" << sFilename <<  "': Error while opening file (Num:" << err << ")";
             throw Exception(AVG_ERR_VIDEO_INIT_FAILED, s.str());
     }
-}
-
-
-void dump_stream_info(AVFormatContext *s)
-{
-    if (s->track != 0)
-        fprintf(stderr, "  Track: %d\n", s->track);
-    if (s->title[0] != '\0')
-        fprintf(stderr, "  Title: %s\n", s->title);
-    if (s->author[0] != '\0')
-        fprintf(stderr, "  Author: %s\n", s->author);
-    if (s->album[0] != '\0')
-        fprintf(stderr, "  Album: %s\n", s->album);
-    if (s->year != 0)
-        fprintf(stderr, "  Year: %d\n", s->year);
-    if (s->genre[0] != '\0')
-        fprintf(stderr, "  Genre: %s\n", s->genre);
-}
-
-int openCodec(AVFormatContext* pFormatContext, int streamIndex)
-{
-    AVCodecContext *enc;
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    enc = &(pFormatContext->streams[streamIndex]->codec);
-#else
-    enc = pFormatContext->streams[streamIndex]->codec;
 #endif
-//    enc->debug = 0x0001; // see avcodec.h
+}
 
-    AVCodec * codec = avcodec_find_decoder(enc->codec_id);
-    if (!codec || avcodec_open(enc, codec) < 0) {
+int FFMpegDecoder::openCodec(int streamIndex, bool bUseHardwareAcceleration)
+{
+    AVCodecContext* pContext;
+    pContext = m_pFormatContext->streams[streamIndex]->codec;
+//    pContext->debug = 0x0001; // see avcodec.h
+
+    AVCodec * pCodec = 0;
+#ifdef AVG_ENABLE_VDPAU
+    if (bUseHardwareAcceleration) {
+        pContext->opaque = &m_Opaque;
+        pCodec = m_VDPAU.openCodec(pContext);
+    } else {
+        pCodec = avcodec_find_decoder(pContext->codec_id);
+    }
+#else
+    pCodec = avcodec_find_decoder(pContext->codec_id);
+#endif
+    if (!pCodec || avcodec_open(pContext, pCodec) < 0) {
         return -1;
     }
     return 0;
 }
 
-void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer)
+void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer,
+        bool bUseHardwareAcceleration)
 {
     mutex::scoped_lock lock(s_OpenMutex);
     m_bThreadedDemuxer = bThreadedDemuxer;
@@ -150,15 +144,18 @@ void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer)
     m_bVideoEOF = false;
     m_bEOFPending = false;
     m_VideoStartTimestamp = -1;
-    AVFormatParameters params;
     int err;
     m_sFilename = sFilename;
 
     AVG_TRACE(Logger::MEMORY, "Opening " << sFilename);
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(53,2,0)
+    err = avformat_open_input(&m_pFormatContext, sFilename.c_str(), 0, 0);
+#else
+    AVFormatParameters params;
     memset(&params, 0, sizeof(params));
-
     err = av_open_input_file(&m_pFormatContext, sFilename.c_str(), 
             0, 0, &params);
+#endif
     if (err < 0) {
         m_sFilename = "";
         avcodecError(sFilename, err);
@@ -169,26 +166,21 @@ void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer)
         throw Exception(AVG_ERR_VIDEO_INIT_FAILED, 
                 sFilename + ": Could not find codec parameters.");
     }
-//    dump_format(m_pFormatContext, 0, sFilename.c_str(), 0);
-
+//    dump_format(m_pFormatContext, 0, sFilename.c_str(), false);
     av_read_play(m_pFormatContext);
     
     // Find audio and video streams in the file
     m_VStreamIndex = -1;
     m_AStreamIndex = -1;
     for (unsigned i = 0; i < m_pFormatContext->nb_streams; i++) {
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-        AVCodecContext *enc = &m_pFormatContext->streams[i]->codec;
-#else         
-        AVCodecContext *enc = m_pFormatContext->streams[i]->codec;
-#endif
-        switch (enc->codec_type) {
-            case CODEC_TYPE_VIDEO:
+        AVCodecContext* pContext = m_pFormatContext->streams[i]->codec;
+        switch (pContext->codec_type) {
+            case AVMEDIA_TYPE_VIDEO:
                 if (m_VStreamIndex < 0) {
                     m_VStreamIndex = i;
                 }
                 break;
-            case CODEC_TYPE_AUDIO:
+            case AVMEDIA_TYPE_AUDIO:
                 // Ignore the audio stream if we're using sync demuxing. 
                 if (m_AStreamIndex < 0 && bThreadedDemuxer) {
                     m_AStreamIndex = i;
@@ -198,8 +190,6 @@ void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer)
                 break;
         }
     }
-//    dump_format(m_pFormatContext, 0, m_sFilename.c_str(), 0);
-//    dump_stream_info(m_pFormatContext);
     
     // Enable video stream demuxing
     if (m_VStreamIndex >= 0) {
@@ -211,16 +201,12 @@ void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer)
         if (m_bUseStreamFPS) {
             m_FPS = getNominalFPS();
         }
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-        m_Size = IntPoint(m_pVStream->codec.width, m_pVStream->codec.height);
-#else
         m_Size = IntPoint(m_pVStream->codec->width, m_pVStream->codec->height);
-#endif
         m_bFirstPacket = true;
         m_sFilename = sFilename;
         m_LastVideoFrameTime = -1;
-    
-        int rc = openCodec(m_pFormatContext, m_VStreamIndex);
+
+        int rc = openCodec(m_VStreamIndex, bUseHardwareAcceleration);
         if (rc == -1) {
             m_VStreamIndex = -1;
             char szBuf[256];
@@ -241,12 +227,12 @@ void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer)
         m_LastAudioFrameTime = 0;
         m_AudioStartTimestamp = 0;
         
-        if (m_pAStream->start_time != AV_NOPTS_VALUE) {
+        if ((unsigned long long)m_pAStream->start_time != AV_NOPTS_VALUE) {
             m_AudioStartTimestamp = double(av_q2d(m_pAStream->time_base))
                     *m_pAStream->start_time;
         }
         m_EffectiveSampleRate = (int)(m_pAStream->codec->sample_rate);
-        int rc = openCodec(m_pFormatContext, m_AStreamIndex);
+        int rc = openCodec(m_AStreamIndex, bUseHardwareAcceleration);
         if (rc == -1) {
             m_AStreamIndex = -1;
             char szBuf[256];
@@ -255,6 +241,12 @@ void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer)
             AVG_TRACE(Logger::WARNING, 
                     sFilename + ": unsupported codec ("+szBuf+"). Disabling audio.");
         }
+        if (m_pAStream->codec->sample_fmt != SAMPLE_FMT_S16) {
+            m_AStreamIndex = -1;
+            m_pAStream = 0; 
+            AVG_TRACE(Logger::WARNING, 
+                    sFilename + ": unsupported sample format (!= S16). Disabling audio.");
+        }
     }
 
     m_State = OPENED;
@@ -262,6 +254,9 @@ void FFMpegDecoder::open(const string& sFilename, bool bThreadedDemuxer)
 
 void FFMpegDecoder::startDecoding(bool bDeliverYCbCr, const AudioParams* pAP)
 {
+#ifdef AVG_ENABLE_VDPAU
+    m_VDPAU.init();
+#endif
     AVG_ASSERT(m_State == OPENED);
     if (m_VStreamIndex >= 0) {
         m_PF = calcPixelFormat(bDeliverYCbCr);
@@ -386,7 +381,7 @@ void FFMpegDecoder::close()
     m_State = CLOSED;
 }
 
-IVideoDecoder::DecoderState FFMpegDecoder::getState() const
+VideoDecoder::DecoderState FFMpegDecoder::getState() const
 {
     return m_State;
 }
@@ -402,7 +397,7 @@ VideoInfo FFMpegDecoder::getVideoInfo() const
             m_pAStream != 0);
     if (m_pVStream) {
         info.setVideoData(m_Size, getStreamPF(), getNumFrames(), getNominalFPS(), m_FPS,
-                m_pVStream->codec->codec->name);
+                m_pVStream->codec->codec->name, usesVDPAU());
     }
     if (m_pAStream) {
         AVCodecContext * pACodec = m_pAStream->codec;
@@ -479,27 +474,23 @@ double FFMpegDecoder::getDuration() const
     long long duration;
     AVRational time_base;
     if (m_pVStream) {
-        duration=m_pVStream->duration;
-        time_base=m_pVStream->time_base;
+        duration = m_pVStream->duration;
+        time_base = m_pVStream->time_base;
     } else {
-        duration=m_pAStream->duration;
-        time_base=m_pAStream->time_base;
+        duration = m_pAStream->duration;
+        time_base = m_pAStream->time_base;
+    }
+    if (duration == AV_NOPTS_VALUE) {
+        return 0;
+    } else {
+        return double(duration)*av_q2d(time_base);
     }
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    return double(duration)/AV_TIME_BASE;
-#else
-    return double(duration)*av_q2d(time_base);
-#endif 
 }
 
 double FFMpegDecoder::getNominalFPS() const
 {
     AVG_ASSERT(m_State != CLOSED);
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    return m_pVStream->r_frame_rate;
-#else
     return av_q2d(m_pVStream->r_frame_rate);
-#endif 
 }
 
 double FFMpegDecoder::getFPS() const
@@ -548,6 +539,7 @@ void copyPlaneToBmp(BitmapPtr pBmp, unsigned char * pData, int stride)
 
 static ProfilingZoneID RenderToBmpProfilingZone("FFMpeg: renderToBmp");
 static ProfilingZoneID CopyImageProfilingZone("FFMpeg: copy image");
+static ProfilingZoneID VDPAUCopyProfilingZone("FFMpeg: VDPAU copy");
 
 FrameAvailableCode FFMpegDecoder::renderToBmps(vector<BitmapPtr>& pBmps, 
         double timeWanted)
@@ -564,10 +556,23 @@ FrameAvailableCode FFMpegDecoder::renderToBmps(vector<BitmapPtr>& pBmps,
     }
     if (!m_bVideoEOF && frameAvailable == FA_NEW_FRAME) {
         if (pixelFormatIsPlanar(m_PF)) {
+#ifdef AVG_ENABLE_VDPAU
+            if (usesVDPAU()) {
+                ScopeTimer timer(VDPAUCopyProfilingZone);
+                vdpau_render_state* pRenderState = (vdpau_render_state *)frame.data[0];
+                getPlanesFromVDPAU(pRenderState, pBmps[0], pBmps[1], pBmps[2]);
+            } else {
+                ScopeTimer timer(CopyImageProfilingZone);
+                for (unsigned i = 0; i < pBmps.size(); ++i) {
+                    copyPlaneToBmp(pBmps[i], frame.data[i], frame.linesize[i]);
+                }
+            }
+#else 
             ScopeTimer timer(CopyImageProfilingZone);
             for (unsigned i = 0; i < pBmps.size(); ++i) {
                 copyPlaneToBmp(pBmps[i], frame.data[i], frame.linesize[i]);
             }
+#endif
         } else {
             convertFrameToBmp(frame, pBmps[0]);
         }
@@ -576,6 +581,28 @@ FrameAvailableCode FFMpegDecoder::renderToBmps(vector<BitmapPtr>& pBmps,
     return FA_USE_LAST_FRAME;
 }
 
+
+#ifdef AVG_ENABLE_VDPAU
+FrameAvailableCode FFMpegDecoder::renderToVDPAU(vdpau_render_state** ppRenderState)
+{
+    AVG_ASSERT(m_State == DECODING);
+    ScopeTimer timer(RenderToBmpProfilingZone);
+    AVFrame frame;
+    FrameAvailableCode frameAvailable;
+    readFrame(frame);
+    frameAvailable = FA_NEW_FRAME;
+    if (!m_bVideoEOF && frameAvailable == FA_NEW_FRAME) {
+        if (usesVDPAU()) {
+            ScopeTimer timer(VDPAUCopyProfilingZone);
+            vdpau_render_state *pRenderState = (vdpau_render_state *)frame.data[0];
+            *ppRenderState = pRenderState;
+        }
+        return FA_NEW_FRAME;
+    }
+    return FA_USE_LAST_FRAME;
+}
+#endif
+
 void FFMpegDecoder::throwAwayFrame(double timeWanted)
 {
     AVG_ASSERT(m_State == DECODING);
@@ -647,9 +674,17 @@ int FFMpegDecoder::copyResampledAudio(unsigned char* pBuffer, int size)
 void FFMpegDecoder::resampleAudio()
 {
     if (!m_pAudioResampleContext) {
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 24, 0)
+        m_pAudioResampleContext = av_audio_resample_init(
+                m_AP.m_Channels, m_pAStream->codec->channels, 
+                m_AP.m_SampleRate, m_EffectiveSampleRate,
+                SAMPLE_FMT_S16, SAMPLE_FMT_S16,
+                16, 10, 0, 0.8);
+#else
         m_pAudioResampleContext = audio_resample_init(
                 m_AP.m_Channels, m_pAStream->codec->channels, 
                 m_AP.m_SampleRate, m_EffectiveSampleRate);
+#endif        
     }
     
     if (!m_pResampleBuffer) {
@@ -677,21 +712,19 @@ int FFMpegDecoder::decodeAudio()
     // Save current size of the audio buffer
     int lastSampleBufferSize = m_SampleBufferLeft;
 
-#if LIBAVCODEC_BUILD > ((51<<16)+(11<<8)+0)
     // Decode some data from packet into the audio buffer
-    int packetBytesDecoded = avcodec_decode_audio2(
-            m_pAStream->codec, 
-            (short*)(m_pSampleBuffer + m_SampleBufferEnd),
-            &m_SampleBufferLeft, 
-            m_AudioPacketData,
-            m_AudioPacketSize);
+#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(52, 31, 0)
+    AVPacket packet;
+    av_init_packet(&packet);
+    packet.data = m_AudioPacketData;
+    packet.size = m_AudioPacketSize;
+    int packetBytesDecoded = avcodec_decode_audio3(m_pAStream->codec, 
+            (short*)(m_pSampleBuffer + m_SampleBufferEnd), &m_SampleBufferLeft, 
+            &packet);
 #else
-    int packetBytesDecoded = avcodec_decode_audio(
-            m_pAStream->codec, 
-            (short*)(m_pSampleBuffer + m_SampleBufferEnd),
-            &m_SampleBufferLeft, 
-            m_AudioPacketData,
-            m_AudioPacketSize);
+    int packetBytesDecoded = avcodec_decode_audio2(m_pAStream->codec, 
+            (short*)(m_pSampleBuffer + m_SampleBufferEnd), &m_SampleBufferLeft, 
+            m_AudioPacketData, m_AudioPacketSize);
 #endif
 
     // Skip frame on error
@@ -795,14 +828,17 @@ int FFMpegDecoder::fillAudioBuffer(AudioBufferPtr pBuffer)
 
 PixelFormat FFMpegDecoder::calcPixelFormat(bool bUseYCbCr)
 {
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-        AVCodecContext *enc = &m_pVStream->codec;
-#else
-        AVCodecContext *enc = m_pVStream->codec;
-#endif
+    AVCodecContext const* pContext = getCodecContext();
     if (bUseYCbCr) {
-        switch(enc->pix_fmt) {
+        switch(pContext->pix_fmt) {
             case PIX_FMT_YUV420P:
+#ifdef AVG_ENABLE_VDPAU
+            case PIX_FMT_VDPAU_H264:
+            case PIX_FMT_VDPAU_MPEG1:
+            case PIX_FMT_VDPAU_MPEG2:
+            case PIX_FMT_VDPAU_WMV3:
+            case PIX_FMT_VDPAU_VC1:
+#endif
                 return YCbCr420p;
             case PIX_FMT_YUVJ420P:
                 return YCbCrJ420p;
@@ -812,7 +848,7 @@ PixelFormat FFMpegDecoder::calcPixelFormat(bool bUseYCbCr)
                 break;
         }
     }
-    if (enc->pix_fmt == PIX_FMT_BGRA || enc->pix_fmt == PIX_FMT_YUVA420P) {
+    if (pContext->pix_fmt == PIX_FMT_BGRA || pContext->pix_fmt == PIX_FMT_YUVA420P) {
         return B8G8R8A8;
     }
     return B8G8R8X8;
@@ -829,12 +865,8 @@ void FFMpegDecoder::convertFrameToBmp(AVFrame& frame, BitmapPtr pBmp)
     unsigned char * pDestBits = pBmp->getPixels();
     destPict.data[0] = pDestBits;
     destPict.linesize[0] = pBmp->getStride();
-#if LIBAVFORMAT_BUILD >= ((50<<16)+(0<<8)+0)
     ::PixelFormat destFmt;
-#else
-    int destFmt;
-#endif
-    switch(pBmp->getPixelFormat()) {
+    switch (pBmp->getPixelFormat()) {
         case R8G8B8X8:
         case R8G8B8A8:
             // XXX: Unused and broken.
@@ -859,14 +891,10 @@ void FFMpegDecoder::convertFrameToBmp(AVFrame& frame, BitmapPtr pBmp)
             AVG_ASSERT(false);
             destFmt = PIX_FMT_BGRA;
     }
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    AVCodecContext *enc = &m_pVStream->codec;
-#else
-    AVCodecContext *enc = m_pVStream->codec;
-#endif
+    AVCodecContext const* pContext = getCodecContext();
     {
-        if (destFmt == PIX_FMT_BGRA && (enc->pix_fmt == PIX_FMT_YUV420P || 
-                enc->pix_fmt == PIX_FMT_YUVJ420P))
+        if (destFmt == PIX_FMT_BGRA && (pContext->pix_fmt == PIX_FMT_YUV420P || 
+                pContext->pix_fmt == PIX_FMT_YUVJ420P))
         {
             ScopeTimer timer(ConvertImageLibavgProfilingZone);
             BitmapPtr pBmpY(new Bitmap(pBmp->getSize(), I8, frame.data[0],
@@ -875,18 +903,24 @@ void FFMpegDecoder::convertFrameToBmp(AVFrame& frame, BitmapPtr pBmp)
                     frame.linesize[1], false));
             BitmapPtr pBmpV(new Bitmap(pBmp->getSize(), I8, frame.data[2],
                     frame.linesize[2], false));
-            pBmp->copyYUVPixels(*pBmpY, *pBmpU, *pBmpV, enc->pix_fmt == PIX_FMT_YUVJ420P);
+            pBmp->copyYUVPixels(*pBmpY, *pBmpU, *pBmpV, 
+                    pContext->pix_fmt == PIX_FMT_YUVJ420P);
+#ifdef AVG_ENABLE_VDPAU
+        } else if (destFmt == PIX_FMT_BGRA && usesVDPAU()) {
+            vdpau_render_state *pRenderState = (vdpau_render_state *)frame.data[0];
+            getBitmapFromVDPAU(pRenderState, pBmp);
+#endif
         } else {
             if (!m_pSwsContext) {
-                m_pSwsContext = sws_getContext(enc->width, enc->height, enc->pix_fmt,
-                            enc->width, enc->height, destFmt, SWS_BICUBIC, 
-                            NULL, NULL, NULL);
+                m_pSwsContext = sws_getContext(pContext->width, pContext->height, 
+                        pContext->pix_fmt, pContext->width, pContext->height, destFmt, 
+                        SWS_BICUBIC, 0, 0, 0);
                 AVG_ASSERT(m_pSwsContext);
             }
             {
                 ScopeTimer timer(ConvertImageSWSProfilingZone);
                 sws_scale(m_pSwsContext, frame.data, frame.linesize, 0, 
-                    enc->height, destPict.data, destPict.linesize);
+                    pContext->height, destPict.data, destPict.linesize);
             }
             if (pBmp->getPixelFormat() == B8G8R8X8) {
                 ScopeTimer timer(SetAlphaProfilingZone);
@@ -924,16 +958,27 @@ void FFMpegDecoder::initVideoSupport()
     }
 }
 
+bool FFMpegDecoder::usesVDPAU() const
+{
+#ifdef AVG_ENABLE_VDPAU
+    AVCodecContext const* pContext = getCodecContext();
+    return pContext->codec && (pContext->codec->capabilities & CODEC_CAP_HWACCEL_VDPAU);
+#else
+    return false;
+#endif
+}
+
 int FFMpegDecoder::getNumFrames() const
 {
     AVG_ASSERT(m_State != CLOSED);
     // This is broken for some videos, but the code here is correct.
     // So fix ffmpeg :-).
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    return m_pVStream->r_frame_rate*(m_pVStream->duration/AV_TIME_BASE);
-#else
-    return int(m_pVStream->nb_frames);
-#endif 
+    int numFrames =  int(m_pVStream->nb_frames);
+    if (numFrames > 0) {
+        return numFrames;
+    } else {
+        return int(getDuration() * calcStreamFPS());
+    }
 }
 
 FrameAvailableCode FFMpegDecoder::readFrameForTime(AVFrame& frame, double timeWanted)
@@ -949,9 +994,16 @@ FrameAvailableCode FFMpegDecoder::readFrameForTime(AVFrame& frame, double timeWa
         // The last frame is still current. Display it again.
         return FA_USE_LAST_FRAME;
     } else {
-        double frameTime = -1;
-        while (frameTime-timeWanted < -0.5*timePerFrame && !m_bVideoEOF) {
-            frameTime = readFrame(frame);
+        bool bInvalidFrame = true;
+        while (bInvalidFrame && !m_bVideoEOF) {
+            double frameTime = readFrame(frame);
+            bInvalidFrame = frameTime-timeWanted < -0.5*timePerFrame;
+#if AVG_ENABLE_VDPAU
+            if (usesVDPAU() && bInvalidFrame && !m_bVideoEOF) {
+                vdpau_render_state *pRenderState = (vdpau_render_state *)frame.data[0];
+                VDPAU::unlockSurface(pRenderState);
+            }
+#endif
 //            cerr << "        readFrame returned time " << frameTime << ", diff= " <<
 //                    frameTime-timeWanted <<  endl;
         }
@@ -972,11 +1024,7 @@ double FFMpegDecoder::readFrame(AVFrame& frame)
         m_bEOFPending = false;
         return m_LastVideoFrameTime;
     }
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    AVCodecContext *enc = &m_pVStream->codec;
-#else
-    AVCodecContext *enc = m_pVStream->codec;
-#endif
+    AVCodecContext* pContext = getCodecContext();
     int bGotPicture = 0;
     AVPacket* pPacket = 0;
     double frameTime = -1;
@@ -984,11 +1032,21 @@ double FFMpegDecoder::readFrame(AVFrame& frame)
         pPacket = m_pDemuxer->getPacket(m_VStreamIndex);
         m_bFirstPacket = false;
         if (pPacket) {
-            int len1 = avcodec_decode_video(enc, &frame, &bGotPicture, pPacket->data,
+#ifdef AVG_ENABLE_VDPAU
+            FrameAge age;
+            m_Opaque.setFrameAge(&age);
+#endif
+#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(52, 31, 0)
+            int len1 = avcodec_decode_video2(pContext, &frame, &bGotPicture, pPacket);
+#else
+            int len1 = avcodec_decode_video(pContext, &frame, &bGotPicture, pPacket->data,
                     pPacket->size);
+#endif
             if (len1 > 0) {
                 AVG_ASSERT(len1 == pPacket->size);
             }
+            else {
+            }
             if (bGotPicture) {
                 frameTime = getFrameTime(pPacket->dts);
             }
@@ -996,7 +1054,14 @@ double FFMpegDecoder::readFrame(AVFrame& frame)
             delete pPacket;
         } else {
             // No more packets -> EOF. Decode the last data we got.
-            avcodec_decode_video(enc, &frame, &bGotPicture, 0, 0);
+#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(52, 31, 0)
+            AVPacket packet;
+            packet.data = 0;
+            packet.size = 0;
+            avcodec_decode_video2(pContext, &frame, &bGotPicture, &packet);
+#else
+            avcodec_decode_video(pContext, &frame, &bGotPicture, 0, 0);
+#endif
             if (bGotPicture) {
                 m_bEOFPending = true;
             } else {
@@ -1052,22 +1117,19 @@ double FFMpegDecoder::getStartTime()
 
 double FFMpegDecoder::calcStreamFPS() const
 {
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    return m_pVStream->r_frame_rate;
-#else
     return (m_pVStream->r_frame_rate.num/m_pVStream->r_frame_rate.den);
-#endif 
 }
 
 string FFMpegDecoder::getStreamPF() const
 {
-#if LIBAVFORMAT_BUILD < ((49<<16)+(0<<8)+0)
-    AVCodecContext *pCodec = &m_pVStream->codec;
-#else
-    AVCodecContext *pCodec = m_pVStream->codec;
-#endif
+    AVCodecContext const* pCodec = getCodecContext();
     ::PixelFormat pf = pCodec->pix_fmt;
-    return avcodec_get_pix_fmt_name(pf);
+    const char* psz = av_get_pix_fmt_name(pf);
+    string s;
+    if (psz) {
+        s = psz;
+    } 
+    return s;
 }
 
 // TODO: this should be logarithmic...
@@ -1099,5 +1161,48 @@ void FFMpegDecoder::volumize(AudioBufferPtr pBuffer)
     m_LastVolume = curVol;
 }
 
+AVCodecContext const* FFMpegDecoder::getCodecContext() const
+{
+    return m_pVStream->codec;
+}
+
+AVCodecContext* FFMpegDecoder::getCodecContext()
+{
+    return m_pVStream->codec;
+}
+
+#ifdef AVG_ENABLE_VDPAU
+void getPlanesFromVDPAU(vdpau_render_state* pRenderState, BitmapPtr pBmpY,
+        BitmapPtr pBmpU, BitmapPtr pBmpV)
+{
+    VdpStatus status;
+    void *dest[3] = {
+        pBmpY->getPixels(),
+        pBmpV->getPixels(),
+        pBmpU->getPixels()
+    };
+    uint32_t pitches[3] = {
+        pBmpY->getStride(),
+        pBmpV->getStride(),
+        pBmpU->getStride()
+    };
+    status = vdp_video_surface_get_bits_y_cb_cr(pRenderState->surface,
+            VDP_YCBCR_FORMAT_YV12, dest, pitches);
+    AVG_ASSERT(status == VDP_STATUS_OK);
+    VDPAU::unlockSurface(pRenderState);
+}
+
+void getBitmapFromVDPAU(vdpau_render_state* pRenderState, BitmapPtr pBmpDest)
+{
+    IntPoint YSize = pBmpDest->getSize();
+    IntPoint UVSize(YSize.x>>1, YSize.y);
+    BitmapPtr pBmpY(new Bitmap(YSize, I8));
+    BitmapPtr pBmpU(new Bitmap(UVSize, I8));
+    BitmapPtr pBmpV(new Bitmap(UVSize, I8));
+    getPlanesFromVDPAU(pRenderState, pBmpY, pBmpU, pBmpV);
+    pBmpDest->copyYUVPixels(*pBmpY, *pBmpU, *pBmpV, false);
+}   
+#endif
+
 }
 
