@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,52 +23,35 @@
 #define _FFMpegDecoder_H_
 
 #include "../avgconfigwrapper.h"
-#include "IVideoDecoder.h"
+#include "VideoDecoder.h"
 #include "IDemuxer.h"
 
 #include "../audio/AudioParams.h"
 #include "../base/ProfilingZone.h"
 #include "../avgconfigwrapper.h"
 
-#ifdef _WIN32
-#define EMULATE_INTTYPES
-#if !defined INT64_C
-#define INT64_C(c) c##i64
-#endif
-#else
-// This is probably GCC-specific.
-#if !defined INT64_C
-#if defined __WORDSIZE && __WORDSIZE == 64
-#define INT64_C(c) c ## L
-#else
-#define INT64_C(c) c ## LL
-#endif
-#endif
+#include "WrapFFMpeg.h"
+
+#ifdef AVG_ENABLE_VDPAU
+#include "VDPAU.h"
+#include "AVCCOpaque.h"
+#include "FrameAge.h"
+#include <libavcodec/vdpau.h>
 #endif
 
-extern "C" {
-#ifdef HAVE_LIBAVFORMAT_AVFORMAT_H
-#include <libavcodec/avcodec.h>
-#include <libavformat/avformat.h>
-#include <libswscale/swscale.h>
-#else
-#include <ffmpeg/avcodec.h>
-#include <ffmpeg/avformat.h>
-#include <ffmpeg/swscale.h>
-#endif
-}
 #include <boost/thread/mutex.hpp>
 
 namespace avg {
 
 class AudioBuffer;
 
-class AVG_API FFMpegDecoder: public IVideoDecoder
+class AVG_API FFMpegDecoder: public VideoDecoder
 {
     public:
         FFMpegDecoder();
         virtual ~FFMpegDecoder();
-        virtual void open(const std::string& sFilename, bool bThreadedDemuxer);
+        virtual void open(const std::string& sFilename, bool bThreadedDemuxer,
+                bool bUseHardwareAcceleration);
         virtual void startDecoding(bool bDeliverYCbCr, const AudioParams* pAP);
         virtual void close();
         virtual DecoderState getState() const;
@@ -87,6 +70,9 @@ class AVG_API FFMpegDecoder: public IVideoDecoder
         virtual void setFPS(double fps);
         virtual FrameAvailableCode renderToBmps(std::vector<BitmapPtr>& pBmps,
                 double timeWanted);
+#ifdef AVG_ENABLE_VDPAU
+        virtual FrameAvailableCode renderToVDPAU(vdpau_render_state** ppRenderState);
+#endif
         virtual void throwAwayFrame(double timeWanted);
         
         // Called from audio decoder thread
@@ -100,6 +86,8 @@ class AVG_API FFMpegDecoder: public IVideoDecoder
 
     private:
         void initVideoSupport();
+        bool usesVDPAU() const;
+        int openCodec(int streamIndex, bool bUseHardwareAcceleration);
         PixelFormat calcPixelFormat(bool bUseYCbCr);
         virtual double getDuration() const;
         virtual int getNumFrames() const;
@@ -111,11 +99,13 @@ class AVG_API FFMpegDecoder: public IVideoDecoder
         bool m_bThreadedDemuxer;
 
         // Used from video thread.
-        FrameAvailableCode readFrameForTime(AVFrame& Frame, double timeWanted);
-        void convertFrameToBmp(AVFrame& Frame, BitmapPtr pBmp);
+        FrameAvailableCode readFrameForTime(AVFrame& frame, double timeWanted);
+        void convertFrameToBmp(AVFrame& frame, BitmapPtr pBmp);
         double getFrameTime(long long dts);
         double calcStreamFPS() const;
         std::string getStreamPF() const;
+        AVCodecContext const * getCodecContext() const;
+        AVCodecContext * getCodecContext();
 
         SwsContext * m_pSwsContext;
         IntPoint m_Size;
@@ -155,6 +145,10 @@ class AVG_API FFMpegDecoder: public IVideoDecoder
         IDemuxer * m_pDemuxer;
         AVStream * m_pVStream;
         AVStream * m_pAStream;
+#ifdef AVG_ENABLE_VDPAU
+        VDPAU m_VDPAU;
+        AVCCOpaque m_Opaque;
+#endif
         int m_VStreamIndex;
         bool m_bEOFPending;
         bool m_bVideoEOF;
@@ -173,6 +167,12 @@ class AVG_API FFMpegDecoder: public IVideoDecoder
         static boost::mutex s_OpenMutex;   
 };
 
+#ifdef AVG_ENABLE_VDPAU
+void getPlanesFromVDPAU(vdpau_render_state* pRenderState, BitmapPtr pBmpY,
+        BitmapPtr pBmpU, BitmapPtr pBmpV);
+void getBitmapFromVDPAU(vdpau_render_state* pRenderState, BitmapPtr pBmpDest);
+#endif
+
 }
 #endif 
 
