@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2008 Ulrich von Zadow
+//  Copyright (C) 2003-2011 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -122,6 +122,13 @@ ConstDPoint::operator DPoint() const
     return DPoint(x,y);
 }
 
+void checkEmptyArgs(const boost::python::tuple &args, int numArgs)
+{
+    if (boost::python::len(args) != numArgs) {
+        throw avg::Exception(AVG_ERR_INVALID_ARGS, 
+                "Nodes must be constructed using named parameters. Positional parameters are not supported.");
+    }
+}
 
 template<class NUM>
 struct Point_to_python_tuple
@@ -132,6 +139,16 @@ struct Point_to_python_tuple
     }
 };
 
+struct Exception_to_python_exception
+{
+    static PyObject* convert (avg::Exception ex)
+    {
+        PyObject *arglist = boost::python::incref(Py_BuildValue("(s)", ex.getStr().c_str()));
+        
+        return boost::python::incref(
+                PyObject_CallObject(PyExc_RuntimeError, arglist));
+    }
+};
 
 template<class NUM>
 struct Triple_to_python_tuple
@@ -224,7 +241,7 @@ struct triple_from_python
 
 void exception_translator(Exception const & e) 
 {
-    PyErr_SetString(PyExc_RuntimeError, e.GetStr().c_str());
+    PyErr_SetString(PyExc_RuntimeError, e.getStr().c_str());
 }
 
 struct UTF8String_to_unicode
@@ -300,6 +317,7 @@ void export_base()
 #if (BOOST_VERSION / 100000) > 1 || ((BOOST_VERSION / 100) % 1000) >= 33
     register_exception_translator<Exception>(exception_translator);
 #endif
+    to_python_converter<Exception, Exception_to_python_exception>();
     to_python_converter<IntPoint, Point_to_python_tuple<int> >();
     to_python_converter<DTriple, Triple_to_python_tuple<double> >();
     point_from_python<DPoint, double>();
